@@ -33,5 +33,11 @@ using namespace boost::placeholders;
 #undef vsnprintf
 #endif
 
+inline void python_deprecated(char const* msg)
+{
+    if (PyErr_WarnEx(PyExc_DeprecationWarning, msg, 1) == -1)
+        boost::python::throw_error_already_set();
+}
+
 #endif
 
