@@ -20,6 +20,7 @@ namespace {
 #if TORRENT_ABI_VERSION == 1
 	torrent_handle _add_magnet_uri(lt::session& s, std::string uri, dict params)
 	{
+        python_deprecated("add_magnet_uri() is deprecated");
 		add_torrent_params p;
 
 		dict_to_add_torrent_params(params, p);
@@ -57,13 +58,18 @@ namespace {
 		for (auto const& i : p.dht_nodes)
 			nodes_list.append(boost::python::make_tuple(i.first, i.second));
 		ret["dht_nodes"] =  nodes_list;
-		ret["info_hash"] = bytes(p.info_hash.to_string());
+		if (p.info_hashes.has_v2())
+			ret["info_hashes"] = bytes(p.info_hashes.v2.to_string());
+		else
+			ret["info_hashes"] = bytes(p.info_hashes.v1.to_string());
+#if TORRENT_ABI_VERSION < 3
+		ret["info_hash"] = bytes(p.info_hashes.get_best().to_string());
+#endif
 		ret["name"] = p.name;
 		ret["save_path"] = p.save_path;
 		ret["storage_mode"] = p.storage_mode;
 #if TORRENT_ABI_VERSION == 1
 		ret["url"] = p.url;
-		ret["uuid"] = p.uuid;
 #endif
 		ret["flags"] = p.flags;
 		return ret;
