@@ -12,6 +12,8 @@
 void bind_utility();
 void bind_fingerprint();
 void bind_sha1_hash();
+void bind_sha256_hash();
+void bind_info_hash();
 void bind_session();
 void bind_entry();
 void bind_torrent_info();
@@ -35,15 +37,17 @@ BOOST_PYTHON_MODULE(libtorrent)
     PyEval_InitThreads();
 
     bind_converters();
+    bind_unicode_string_conversion();
     bind_error_code();
     bind_utility();
     bind_fingerprint();
     bind_sha1_hash();
+    bind_sha256_hash();
+    bind_info_hash();
     bind_entry();
     bind_torrent_handle();
     bind_session();
     bind_torrent_info();
-    bind_unicode_string_conversion();
     bind_torrent_status();
     bind_session_settings();
     bind_version();
