@@ -117,6 +117,7 @@ void bind_peer_info()
     pi.attr("remote_choked") = peer_info::remote_choked;
     pi.attr("supports_extensions") = peer_info::supports_extensions;
     pi.attr("local_connection") = peer_info::local_connection;
+    pi.attr("outgoing_connection") = peer_info::outgoing_connection;
     pi.attr("handshake") = peer_info::handshake;
     pi.attr("connecting") = peer_info::connecting;
 #if TORRENT_ABI_VERSION == 1
@@ -135,8 +136,9 @@ void bind_peer_info()
 #endif
 
     // connection_type
-    pi.attr("standard_bittorrent") = (int)peer_info::standard_bittorrent;
-    pi.attr("web_seed") = (int)peer_info::web_seed;
+    pi.attr("standard_bittorrent") = peer_info::standard_bittorrent;
+    pi.attr("web_seed") = peer_info::web_seed;
+    pi.attr("http_seed") = peer_info::http_seed;
 
     // source
     pi.attr("tracker") = peer_info::tracker;
