@@ -66,7 +66,8 @@ public:
     stream(stream const& other) = delete;
     ~stream()
     {
-        if (m_impl) {
+        if (m_impl)
+        {
             m_impl->abort();
             m_impl->parent = nullptr;
         }
@@ -105,10 +106,7 @@ public:
 #endif
 
     // Warning: ignored
-    error_code set_verify_depth(int, error_code& ec)
-    {
-        return ec;
-    }
+    error_code set_verify_depth(int, error_code& ec) { return ec; }
 
 #ifndef BOOST_NO_EXCEPTIONS
     template <typename VerifyCallback> void set_verify_callback(VerifyCallback callback)
@@ -244,8 +242,7 @@ public:
             return;
         }
 
-        using namespace std::placeholders;
-        m_impl->read_handler = std::bind(callable, _1, _2);
+        m_impl->read_handler = std::bind(callable, std::placeholders::_1, std::placeholders::_2);
         m_impl->bytes_read = 0;
         m_impl->async_schedule();
         return callable.get_completion_result();
@@ -293,8 +290,7 @@ public:
             return;
         }
 
-        using namespace std::placeholders;
-        m_impl->write_handler = std::bind(callable, _1, _2);
+        m_impl->write_handler = std::bind(callable, std::placeholders::_1, std::placeholders::_2);
         m_impl->bytes_written = 0;
         m_impl->async_schedule();
         return callable.get_completion_result();
@@ -309,8 +305,7 @@ public:
 
     error_code handshake(handshake_type type, error_code& ec)
     {
-        if (m_impl->is_handshake_done)
-            return ec = boost::asio::error::operation_not_supported;
+        if (m_impl->is_handshake_done) return ec = boost::asio::error::operation_not_supported;
 
         ensure_impl(type);
         int ret;
@@ -318,7 +313,11 @@ public:
             ret = gnutls_handshake(m_impl->session);
         } while (ret != GNUTLS_E_SUCCESS && !gnutls_error_is_fatal(ret));
 
-        if (ret != GNUTLS_E_SUCCESS) return ec = error_code(ret, error::get_ssl_category());
+        if (ret == GNUTLS_E_PREMATURE_TERMINATION)
+            return ec = error::stream_truncated;
+        else if (ret != GNUTLS_E_SUCCESS)
+            return ec = error_code(ret, error::get_ssl_category());
+
         m_impl->is_handshake_done = true;
         return ec;
     }
@@ -356,7 +355,11 @@ public:
             ret = gnutls_bye(m_impl->session, GNUTLS_SHUT_RDWR);
         } while (ret != GNUTLS_E_SUCCESS && !gnutls_error_is_fatal(ret));
 
-        if (ret != GNUTLS_E_SUCCESS) return ec = error_code(ret, error::get_ssl_category());
+        if (ret == GNUTLS_E_PREMATURE_TERMINATION)
+            return ec = error::stream_truncated;
+        else if (ret != GNUTLS_E_SUCCESS)
+            return ec = error_code(ret, error::get_ssl_category());
+
         m_impl->is_handshake_done = false;
         return ec;
     }
@@ -529,20 +532,20 @@ private:
 
         ~impl() { gnutls_deinit(session); }
 
-        template<typename Function> void post(Function&& function) const
+        template <typename Function> void post(Function&& function) const
         {
-            if(parent)
-                boost::asio::post(parent->get_executor(), std::forward<Function>(function));
+            if (parent) boost::asio::post(parent->get_executor(), std::forward<Function>(function));
         }
 
-        void abort() {
-            if(auto handler = std::exchange(handshake_handler, nullptr))
+        void abort()
+        {
+            if (auto handler = std::exchange(handshake_handler, nullptr))
                 handler(boost::asio::error::operation_aborted);
-            if(auto handler = std::exchange(shutdown_handler, nullptr))
+            if (auto handler = std::exchange(shutdown_handler, nullptr))
                 handler(boost::asio::error::operation_aborted);
-            if(auto handler = std::exchange(read_handler, nullptr))
+            if (auto handler = std::exchange(read_handler, nullptr))
                 handler(boost::asio::error::operation_aborted, std::size_t(0));
-            if(auto handler = std::exchange(write_handler, nullptr))
+            if (auto handler = std::exchange(write_handler, nullptr))
                 handler(boost::asio::error::operation_aborted, std::size_t(0));
         }
 
@@ -560,7 +563,8 @@ private:
 
         void async_schedule()
         {
-            using namespace std::placeholders;
+            constexpr auto wait_read = std::remove_reference<next_layer_type>::type::wait_read;
+            constexpr auto wait_write = std::remove_reference<next_layer_type>::type::wait_write;
 
             if (!parent) return;
             auto& next_layer = parent->m_next_layer;
@@ -571,16 +575,19 @@ private:
                 if (gnutls_record_check_pending(session) > 0 && read_handler)
                     handle_read();
                 else
-                    next_layer.async_wait(
-                        next_layer_type::wait_read,
-                        std::bind(&impl::handle_read, this->shared_from_this(), _1));
+                    next_layer.async_wait(wait_read,
+                                          std::bind(&impl::handle_read,
+                                                    this->shared_from_this(),
+                                                    std::placeholders::_1));
             }
 
             // Start a write operation if GnuTLS wants one
             if (want_write() && !std::exchange(is_writing, true))
             {
-                next_layer.async_wait(next_layer_type::wait_write,
-                        std::bind(&impl::handle_write, this->shared_from_this(), _1));
+                next_layer.async_wait(wait_write,
+                                      std::bind(&impl::handle_write,
+                                                this->shared_from_this(),
+                                                std::placeholders::_1));
             }
         }
 
@@ -618,7 +625,8 @@ private:
 
                 write_buffers.clear();
                 auto handler = std::exchange(write_handler, nullptr);
-                post(std::bind(std::move(handler), ec, std::exchange(bytes_written, std::size_t(0))));
+                post(std::bind(
+                    std::move(handler), ec, std::exchange(bytes_written, std::size_t(0))));
             }
 
             if (handshake_handler) return handle_handshake(ec);
@@ -640,12 +648,22 @@ private:
                 want_direction = direction::none;
                 if (ret == GNUTLS_E_SUCCESS)
                     is_handshake_done = true;
+                else if (ret == GNUTLS_E_PREMATURE_TERMINATION)
+                    ec = error::stream_truncated;
                 else
                     ec = error_code(ret, error::get_ssl_category());
             }
 
-            auto handler = std::exchange(handshake_handler, nullptr);
-            post(std::bind(std::move(handler), ec));
+            if (handshake_handler != nullptr)
+            {
+                auto handler = std::exchange(handshake_handler, nullptr);
+                post(std::bind(std::move(handler), ec));
+            }
+        }
+
+        bool is_safe_renegotiation_enabled()
+        {
+            return gnutls_safe_renegotiation_status(session) != 0;
         }
 
         void handle_shutdown(error_code ec = {})
@@ -682,6 +700,10 @@ private:
                 {
                     if (ret == GNUTLS_E_AGAIN)
                         ec = boost::asio::error::would_block;
+                    else if (ret == GNUTLS_E_PREMATURE_TERMINATION)
+                        ec = error::stream_truncated;
+                    else if (ret == GNUTLS_E_REHANDSHAKE && is_safe_renegotiation_enabled())
+                        handle_handshake(ec);
                     else if (gnutls_error_is_fatal(ret))
                         ec = error_code(ret, error::get_ssl_category());
                     else
@@ -728,6 +750,8 @@ private:
                 {
                     if (ret == GNUTLS_E_AGAIN)
                         ec = boost::asio::error::would_block;
+                    else if (ret == GNUTLS_E_PREMATURE_TERMINATION)
+                        ec = error::stream_truncated;
                     else if (gnutls_error_is_fatal(ret))
                         ec = error_code(ret, error::get_ssl_category());
                     else
@@ -900,7 +924,8 @@ private:
     std::shared_ptr<impl> ensure_impl(handshake_type type)
     {
         if (!m_impl || m_impl->type != type)
-            if (auto old = std::exchange(m_impl, std::make_shared<impl>(this, type))) {
+            if (auto old = std::exchange(m_impl, std::make_shared<impl>(this, type)))
+            {
                 old->abort();
                 old->parent = nullptr;
             }
