@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2016, Arvid Norberg
+Copyright (c) 2016-2020, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -36,6 +36,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <fstream>
 
 #include <libtorrent/session.hpp>
+#include <libtorrent/session_params.hpp>
 #include <libtorrent/add_torrent_params.hpp>
 #include <libtorrent/torrent_handle.hpp>
 #include <libtorrent/alert_types.hpp>
@@ -46,23 +47,33 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <libtorrent/error_code.hpp>
 #include <libtorrent/magnet_uri.hpp>
 
+namespace {
+
 using clk = std::chrono::steady_clock;
 
 // return the name of a torrent status enum
 char const* state(lt::torrent_status::state_t s)
 {
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wcovered-switch-default"
+#endif
 	switch(s) {
 		case lt::torrent_status::checking_files: return "checking";
 		case lt::torrent_status::downloading_metadata: return "dl metadata";
 		case lt::torrent_status::downloading: return "downloading";
 		case lt::torrent_status::finished: return "finished";
 		case lt::torrent_status::seeding: return "seeding";
-		case lt::torrent_status::allocating: return "allocating";
 		case lt::torrent_status::checking_resume_data: return "checking resume";
 		default: return "<>";
 	}
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 }
 
+} // anonymous namespace
+
 int main(int argc, char const* argv[]) try
 {
 	if (argc != 2) {
@@ -85,17 +96,20 @@ int main(int argc, char const* argv[]) try
 	std::vector<char> buf{std::istream_iterator<char>(ifs)
 		, std::istream_iterator<char>()};
 
-	lt::add_torrent_params atp = lt::read_resume_data(buf);
 	lt::add_torrent_params magnet = lt::parse_magnet_uri(argv[1]);
-	if (atp.info_hash != magnet.info_hash) {
-		atp = std::move(magnet);
+	if (buf.size()) {
+		lt::add_torrent_params atp = lt::read_resume_data(buf);
+		if (atp.info_hashes == magnet.info_hashes) magnet = std::move(atp);
 	}
-	atp.save_path = "."; // save in current dir
-	ses.async_add_torrent(std::move(atp));
+	magnet.save_path = "."; // save in current dir
+	ses.async_add_torrent(std::move(magnet));
 
 	// this is the handle we'll set once we get the notification of it being
 	// added
 	lt::torrent_handle h;
+
+	// set when we're exiting
+	bool done = false;
 	for (;;) {
 		std::vector<lt::alert*> alerts;
 		ses.pop_alerts(&alerts);
@@ -106,12 +120,13 @@ int main(int argc, char const* argv[]) try
 			}
 			// if we receive the finished alert or an error, we're done
 			if (lt::alert_cast<lt::torrent_finished_alert>(a)) {
-				h.save_resume_data();
-				goto done;
+				h.save_resume_data(lt::torrent_handle::save_info_dict);
+				done = true;
 			}
 			if (lt::alert_cast<lt::torrent_error_alert>(a)) {
 				std::cout << a->message() << std::endl;
-				goto done;
+				done = true;
+				h.save_resume_data(lt::torrent_handle::save_info_dict);
 			}
 
 			// when resume data is ready, save it
@@ -119,7 +134,12 @@ int main(int argc, char const* argv[]) try
 				std::ofstream of(".resume_file", std::ios_base::binary);
 				of.unsetf(std::ios_base::skipws);
 				auto const b = write_resume_data_buf(rd->params);
-				of.write(b.data(), b.size());
+				of.write(b.data(), int(b.size()));
+				if (done) goto done;
+			}
+
+			if (lt::alert_cast<lt::save_resume_data_failed_alert>(a)) {
+				if (done) goto done;
 			}
 
 			if (auto st = lt::alert_cast<lt::state_update_alert>(a)) {
@@ -128,10 +148,11 @@ int main(int argc, char const* argv[]) try
 				// we only have a single torrent, so we know which one
 				// the status is for
 				lt::torrent_status const& s = st->status[0];
-				std::cout << "\r" << state(s.state) << " "
+				std::cout << '\r' << state(s.state) << ' '
 					<< (s.download_payload_rate / 1000) << " kB/s "
 					<< (s.total_done / 1000) << " kB ("
-					<< (s.progress_ppm / 10000) << "%) downloaded\x1b[K";
+					<< (s.progress_ppm / 10000) << "%) downloaded ("
+					<< s.num_peers << " peers)\x1b[K";
 				std::cout.flush();
 			}
 		}
@@ -143,13 +164,11 @@ int main(int argc, char const* argv[]) try
 
 		// save resume data once every 30 seconds
 		if (clk::now() - last_save_resume > std::chrono::seconds(30)) {
-			h.save_resume_data();
+			h.save_resume_data(lt::torrent_handle::save_info_dict);
 			last_save_resume = clk::now();
 		}
 	}
 
-	// TODO: ideally we should save resume data here
-
 done:
 	std::cout << "\ndone, shutting down" << std::endl;
 }
