@@ -1,6 +1,9 @@
 /*
 
-Copyright (c) 2007-2018, Arvid Norberg
+Copyright (c) 2007-2010, 2015-2020, Arvid Norberg
+Copyright (c) 2016-2017, Alden Torres
+Copyright (c) 2017, Pavel Pimenov
+Copyright (c) 2018, Steven Siloti
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,7 +36,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_NATPMP_HPP
 #define TORRENT_NATPMP_HPP
 
-#include "libtorrent/io_service_fwd.hpp"
+#include "libtorrent/io_context.hpp"
 #include "libtorrent/socket.hpp"
 #include "libtorrent/address.hpp"
 #include "libtorrent/error_code.hpp"
@@ -43,55 +46,56 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/portmap.hpp"
 #include "libtorrent/aux_/vector.hpp"
 #include "libtorrent/enum_net.hpp" // for ip_interface
+#include "libtorrent/aux_/listen_socket_handle.hpp"
 
 namespace libtorrent {
 
-	namespace errors
+namespace errors {
+	// See RFC 6887 Section 7.4
+	enum pcp_errors
 	{
-		// See RFC 6887 Section 7.4
-		enum pcp_errors
-		{
-			pcp_success = 0,
-			pcp_unsupp_version,
-			pcp_not_authorized,
-			pcp_malformed_request,
-			pcp_unsupp_opcode,
-			pcp_unsupp_option,
-			pcp_malformed_option,
-			pcp_network_failure,
-			pcp_no_resources,
-			pcp_unsupp_protocol,
-			pcp_user_ex_quota,
-			pcp_cannot_provide_external,
-			pcp_address_mismatch,
-			pcp_excessive_remote_peers,
-		};
+		pcp_success = 0,
+		pcp_unsupp_version,
+		pcp_not_authorized,
+		pcp_malformed_request,
+		pcp_unsupp_opcode,
+		pcp_unsupp_option,
+		pcp_malformed_option,
+		pcp_network_failure,
+		pcp_no_resources,
+		pcp_unsupp_protocol,
+		pcp_user_ex_quota,
+		pcp_cannot_provide_external,
+		pcp_address_mismatch,
+		pcp_excessive_remote_peers,
+	};
 
-		boost::system::error_code make_error_code(pcp_errors e);
-	}
+	boost::system::error_code make_error_code(pcp_errors e);
+} // namespace errors
 
 	TORRENT_EXPORT boost::system::error_category& pcp_category();
-}
+} // namespace libtorrent
 
-namespace boost { namespace system {
+namespace boost {
+namespace system {
 	template<> struct is_error_code_enum<libtorrent::errors::pcp_errors>
 	{ static const bool value = true; };
-} }
+}
+}
 
 namespace libtorrent {
 
-struct TORRENT_EXTRA_EXPORT natpmp
+struct TORRENT_EXTRA_EXPORT natpmp final
 	: std::enable_shared_from_this<natpmp>
 	, single_threaded
 {
-	natpmp(io_service& ios, aux::portmap_callback& cb);
+	natpmp(io_context& ios, aux::portmap_callback& cb, aux::listen_socket_handle ls);
 
 	void start(ip_interface const& ip);
 
 	// maps the ports, if a port is set to 0
 	// it will not be mapped
-	port_mapping_t add_mapping(portmap_protocol p, int external_port, tcp::endpoint local_ep
-		, std::string const& device);
+	port_mapping_t add_mapping(portmap_protocol p, int external_port, tcp::endpoint local_ep);
 	void delete_mapping(port_mapping_t mapping_index);
 	bool get_mapping(port_mapping_t mapping_index, int& local_port, int& external_port
 		, portmap_protocol& protocol) const;
@@ -201,11 +205,15 @@ private:
 	// the mapping index that will expire next
 	port_mapping_t m_next_refresh{-1};
 
+	io_context& m_ioc;
+
+	aux::listen_socket_handle m_listen_handle;
+
 	bool m_disabled = false;
 
 	bool m_abort = false;
 };
 
-}
+} // namespace libtorrent
 
 #endif
