@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2008-2018, Arvid Norberg
+Copyright (c) 2008-2009, 2014, 2016-2017, 2019, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -39,6 +39,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <string>
 
 #include "libtorrent/error_code.hpp"
+#include "libtorrent/string_view.hpp"
 
 namespace libtorrent {
 
@@ -50,6 +51,16 @@ namespace libtorrent {
 	// split a URL in its base and path parts
 	TORRENT_EXTRA_EXPORT std::tuple<std::string, std::string>
 		split_url(std::string url, error_code& ec);
+
+	// returns true if the hostname contains any IDNA (internationalized domain
+	// name) labels.
+	TORRENT_EXTRA_EXPORT bool is_idna(string_view hostname);
+
+	// the query string is the part of the URL immediately following "?", i.e.
+	// the query string arguments. This function returns true if any of the
+	// arguments are "info_hash", "port", "key", "event", "uploaded",
+	// "downloaded", "left" or "corrupt".
+	TORRENT_EXTRA_EXPORT bool has_tracker_query_string(string_view query_string);
 }
 
 #endif
