@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2003-2014, Arvid Norberg
+Copyright (c) 2003-2004, 2006-2010, 2012, 2014-2020, Arvid Norberg
+Copyright (c) 2016-2017, Alden Torres
+Copyright (c) 2018, Alexandre Janniaux
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,9 +35,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_SOCKET_HPP_INCLUDED
 #define TORRENT_SOCKET_HPP_INCLUDED
 
-#ifdef _MSC_VER
-#pragma warning(push, 1)
-#endif
+#include "libtorrent/config.hpp"
+#include "libtorrent/aux_/noexcept_movable.hpp"
+
+#include "libtorrent/aux_/disable_warnings_push.hpp"
 
 // if building as Objective C++, asio's template
 // parameters Protocol has to be renamed to avoid
@@ -50,79 +53,82 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <winsock2.h>
 #endif
 
-#include <boost/version.hpp>
-
-#if BOOST_VERSION < 103500
-#include <asio/ip/tcp.hpp>
-#include <asio/ip/udp.hpp>
-#include <asio/write.hpp>
-#include <asio/read.hpp>
-#else
 #include <boost/asio/ip/tcp.hpp>
 #include <boost/asio/ip/udp.hpp>
 #include <boost/asio/write.hpp>
 #include <boost/asio/read.hpp>
-#endif
 
-#ifdef __OBJC__ 
+#ifdef __OBJC__
 #undef Protocol
 #endif
 
-#ifdef _MSC_VER
-#pragma warning(pop)
+#if defined TORRENT_BUILD_SIMULATOR
+#include "simulator/simulator.hpp"
 #endif
 
-namespace libtorrent
-{
+#if TORRENT_USE_NETLINK
+#include <linux/netlink.h>
+#ifndef SOL_NETLINK
+#define SOL_NETLINK 270
+#endif
 
-#if BOOST_VERSION < 103500
-	using ::asio::ip::tcp;
-	using ::asio::ip::udp;
-	using ::asio::async_write;
-	using ::asio::async_read;
+// NETLINK_NO_ENOBUFS exists at least since android 2.3, but is not exposed
+#if defined TORRENT_ANDROID && !defined NETLINK_NO_ENOBUFS
+#define NETLINK_NO_ENOBUFS 5
+#endif
+#endif
 
-	typedef ::asio::ip::tcp::socket stream_socket;
-	typedef ::asio::ip::udp::socket datagram_socket;
-	typedef ::asio::ip::tcp::acceptor socket_acceptor;
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
+namespace libtorrent {
+
+#if defined TORRENT_BUILD_SIMULATOR
+struct tcp : sim::asio::ip::tcp {
+	tcp(sim::asio::ip::tcp const& p) : sim::asio::ip::tcp(p) {} // NOLINT
+	using socket = aux::noexcept_move_only<sim::asio::ip::tcp::socket>;
+};
+struct udp : sim::asio::ip::udp {
+	udp(sim::asio::ip::udp const& p) : sim::asio::ip::udp(p) {} // NOLINT
+	using socket = aux::noexcept_move_only<sim::asio::ip::udp::socket>;
+};
+	using sim::asio::async_write;
+	using sim::asio::async_read;
+	using true_tcp_socket = sim::asio::ip::tcp::socket;
 #else
-	using boost::asio::ip::tcp;
-	using boost::asio::ip::udp;
+struct tcp : boost::asio::ip::tcp {
+	tcp(boost::asio::ip::tcp const& p) : boost::asio::ip::tcp(p) {} // NOLINT
+	using socket = aux::noexcept_move_only<boost::asio::ip::tcp::socket>;
+};
+struct udp : boost::asio::ip::udp {
+	udp(boost::asio::ip::udp const& p) : boost::asio::ip::udp(p) {} // NOLINT
+	using socket = aux::noexcept_move_only<boost::asio::ip::udp::socket>;
+};
 	using boost::asio::async_write;
 	using boost::asio::async_read;
-
-	typedef boost::asio::ip::tcp::socket stream_socket;
-	typedef boost::asio::ip::udp::socket datagram_socket;
-	typedef boost::asio::ip::tcp::acceptor socket_acceptor;
-
-	namespace asio = boost::asio;
+	using true_tcp_socket = boost::asio::ip::tcp::socket;
 #endif
 
-#if TORRENT_USE_IPV6
-#ifdef IPV6_V6ONLY
-	struct v6only
-	{
-		v6only(bool enable): m_value(enable) {}
-		template<class Protocol>
-		int level(Protocol const&) const { return IPPROTO_IPV6; }
-		template<class Protocol>
-		int name(Protocol const&) const { return IPV6_V6ONLY; }
-		template<class Protocol>
-		int const* data(Protocol const&) const { return &m_value; }
-		template<class Protocol>
-		size_t size(Protocol const&) const { return sizeof(m_value); }
-		int m_value;
-	};
-#endif
-#endif
-	
+	// internal
+	inline udp::endpoint make_udp(tcp::endpoint const ep)
+	{ return {ep.address(), ep.port()}; }
+
+	// internal
+	inline tcp::endpoint make_tcp(udp::endpoint const ep)
+	{ return {ep.address(), ep.port()}; }
+
 #ifdef TORRENT_WINDOWS
 
+#ifndef PROTECTION_LEVEL_UNRESTRICTED
+#define PROTECTION_LEVEL_UNRESTRICTED 10
+#endif
+
 #ifndef IPV6_PROTECTION_LEVEL
 #define IPV6_PROTECTION_LEVEL 30
 #endif
+
 	struct v6_protection_level
 	{
-		v6_protection_level(int level): m_value(level) {}
+		explicit v6_protection_level(int level): m_value(level) {}
 		template<class Protocol>
 		int level(Protocol const&) const { return IPPROTO_IPV6; }
 		template<class Protocol>
@@ -133,12 +139,26 @@ namespace libtorrent
 		size_t size(Protocol const&) const { return sizeof(m_value); }
 		int m_value;
 	};
-#endif
+
+	struct exclusive_address_use
+	{
+		explicit exclusive_address_use(int enable): m_value(enable) {}
+		template<class Protocol>
+		int level(Protocol const&) const { return SOL_SOCKET; }
+		template<class Protocol>
+		int name(Protocol const&) const { return SO_EXCLUSIVEADDRUSE; }
+		template<class Protocol>
+		int const* data(Protocol const&) const { return &m_value; }
+		template<class Protocol>
+		size_t size(Protocol const&) const { return sizeof(m_value); }
+		int m_value;
+	};
+#endif // TORRENT_WINDOWS
 
 #ifdef IPV6_TCLASS
 	struct traffic_class
 	{
-		traffic_class(char val): m_value(val) {}
+		explicit traffic_class(char val): m_value(val) {}
 		template<class Protocol>
 		int level(Protocol const&) const { return IPPROTO_IPV6; }
 		template<class Protocol>
@@ -153,12 +173,12 @@ namespace libtorrent
 
 	struct type_of_service
 	{
-#ifdef WIN32
-		typedef DWORD tos_t;
+#ifdef _WIN32
+		using tos_t = DWORD;
 #else
-		typedef int tos_t;
+		using tos_t = int;
 #endif
-		type_of_service(char val): m_value(val) {}
+		explicit type_of_service(char val) : m_value(tos_t(val)) {}
 		template<class Protocol>
 		int level(Protocol const&) const { return IPPROTO_IP; }
 		template<class Protocol>
@@ -175,26 +195,24 @@ namespace libtorrent
 #endif
 
 #ifdef TORRENT_HAS_DONT_FRAGMENT
+
+	// the order of these preprocessor tests matters. Windows defines both
+	// IP_DONTFRAGMENT and IP_MTU_DISCOVER, but the latter is not supported
+	// in general, the simple option of just setting the DF bit is preferred, if
+	// it's available
+#if defined IP_DONTFRAG || defined IP_DONTFRAGMENT
+
 	struct dont_fragment
 	{
-		dont_fragment(bool val)
-#ifdef IP_PMTUDISCOVER_DO
-			: m_value(val ? IP_PMTUDISC_DO : IP_PMTUDISC_DONT) {}
-#else
-			: m_value(val) {}
-#endif
+		explicit dont_fragment(bool val) : m_value(val) {}
 		template<class Protocol>
 		int level(Protocol const&) const { return IPPROTO_IP; }
 		template<class Protocol>
 		int name(Protocol const&) const
 #if defined IP_DONTFRAG
 		{ return IP_DONTFRAG; }
-#elif defined IP_MTU_DISCOVER
-		{ return IP_MTU_DISCOVER; }
-#elif defined IP_DONTFRAGMENT
+#else // defined IP_DONTFRAGMENT
 		{ return IP_DONTFRAGMENT; }
-#else
-		{}
 #endif
 		template<class Protocol>
 		int const* data(Protocol const&) const { return &m_value; }
@@ -202,8 +220,63 @@ namespace libtorrent
 		size_t size(Protocol const&) const { return sizeof(m_value); }
 		int m_value;
 	};
+
+#else
+
+	// this is the fallback mechanism using the IP_MTU_DISCOVER option, which
+	// does a little bit more than we want, it makes the kernel track an estimate
+	// of the MTU and rejects packets immediately if they are believed to exceed
+	// it.
+	struct dont_fragment
+	{
+		explicit dont_fragment(bool val)
+			: m_value(val ? IP_PMTUDISC_PROBE : IP_PMTUDISC_DONT) {}
+		template<class Protocol>
+		int level(Protocol const&) const { return IPPROTO_IP; }
+		template<class Protocol>
+		int name(Protocol const&) const { return IP_MTU_DISCOVER; }
+		template<class Protocol>
+		int const* data(Protocol const&) const { return &m_value; }
+		template<class Protocol>
+		size_t size(Protocol const&) const { return sizeof(m_value); }
+		int m_value;
+	};
+
+#endif // IP_DONTFRAG vs. IP_MTU_DISCOVER
+
 #endif // TORRENT_HAS_DONT_FRAGMENT
+
+#if TORRENT_USE_NETLINK
+	struct no_enobufs
+	{
+		explicit no_enobufs(bool val) : m_value(val) {}
+		template<class Protocol>
+		int level(Protocol const&) const { return SOL_NETLINK; }
+		template<class Protocol>
+		int name(Protocol const&) const { return NETLINK_NO_ENOBUFS; }
+		template<class Protocol>
+		int const* data(Protocol const&) const { return &m_value; }
+		template<class Protocol>
+		std::size_t size(Protocol const&) const { return sizeof(m_value); }
+		int m_value;
+	};
+#endif // TORRENT_USE_NETLINK
+
+#ifdef TCP_NOTSENT_LOWAT
+	struct tcp_notsent_lowat
+	{
+		explicit tcp_notsent_lowat(int val) : m_value(val) {}
+		template<class Protocol>
+		int level(Protocol const&) const { return IPPROTO_TCP; }
+		template<class Protocol>
+		int name(Protocol const&) const { return TCP_NOTSENT_LOWAT; }
+		template<class Protocol>
+		int const* data(Protocol const&) const { return &m_value; }
+		template<class Protocol>
+		std::size_t size(Protocol const&) const { return sizeof(m_value); }
+		int m_value;
+	};
+#endif
 }
 
 #endif // TORRENT_SOCKET_HPP_INCLUDED
-
