@@ -271,6 +271,10 @@ namespace libtorrent {
 		// around further down the object to maintain an owner
 		std::shared_ptr<torrent_info> m_torrent_file;
 
+		// This is the sum of all non-pad file sizes. In the next major version
+		// this is stored in file_storage and no longer need to be kept here.
+		std::int64_t m_size_on_disk = 0;
+
 		// a back reference to the session
 		// this torrent belongs to.
 		aux::session_interface& m_ses;
@@ -408,7 +412,8 @@ namespace libtorrent {
 		void init();
 
 		void load_merkle_trees(aux::vector<std::vector<sha256_hash>, file_index_t> t
-			, aux::vector<std::vector<bool>, file_index_t> mask);
+			, aux::vector<std::vector<bool>, file_index_t> mask
+			, aux::vector<std::vector<bool>, file_index_t> verified);
 
 		// find the peer that introduced us to the given endpoint. This is
 		// used when trying to holepunch. We need the introducer so that we
@@ -989,6 +994,14 @@ namespace libtorrent {
 		// *blocks must be sorted in acending order*
 		void piece_failed(piece_index_t index, std::vector<int> blocks = std::vector<int>());
 
+		// the peers in "peers" participated in sending a bad piece. If
+		// "known_bad_peer" is true, we know for sure the peers are guilty,
+		// otherwise only one may be guilty (meaning we can't unconditionally
+		// disconnect)
+		void penalize_peers(std::set<torrent_peer*> const& peers
+			, piece_index_t index
+			, bool known_bad_peer);
+
 		// this is the handler for hash failure piece synchronization
 		// i.e. resetting the piece
 		void on_piece_sync(piece_index_t piece, std::vector<int> const& blocks);
@@ -1037,7 +1050,7 @@ namespace libtorrent {
 			return *m_hash_picker;
 		}
 
-		void need_hash_picker(aux::vector<std::vector<bool>, file_index_t> verified = {});
+		void need_hash_picker();
 		bool has_hash_picker() const
 		{
 			return m_hash_picker.get() != nullptr;
@@ -1130,7 +1143,7 @@ namespace libtorrent {
 		bool are_files_checked() const
 		{ return m_files_checked; }
 
-		void initialize_merkle_trees();
+		error_code initialize_merkle_trees();
 
 		// parses the info section from the given
 		// bencoded tree and moves the torrent
@@ -1302,6 +1315,9 @@ namespace libtorrent {
 		std::int64_t m_total_uploaded = 0;
 		std::int64_t m_total_downloaded = 0;
 
+		// the number of bytes of pad files
+		std::int64_t m_padding_bytes = 0;
+
 		// this is a handle that keeps the storage object in the disk io subsystem
 		// alive, as well as the index referencing the storage/torrent in the disk
 		// I/O. When this destructs, the torrent will be removed from the disk
@@ -1661,14 +1677,11 @@ namespace libtorrent {
 
 		// this is set to true if all piece layers were successfully loaded and
 		// validated. Only for v2 torrents
+		// TODO: this member can probably be removed
 		bool m_v2_piece_layers_validated:1;
 
 // ----
 
-		// the number of (16kiB) blocks that fall entirely in pad files
-		// i.e. blocks that we consider we have on start-up
-		std::uint16_t m_padding_blocks = 0;
-
 		// this is set to the connect boost quota for this torrent.
 		// After having received this many priority peer connection attempts, it
 		// falls back onto the steady state peer connection logic, driven by the
