@@ -1,6 +1,10 @@
 /*
 
-Copyright (c) 2003-2018, Arvid Norberg
+Copyright (c) 2015, Mikhail Titov
+Copyright (c) 2004-2020, Arvid Norberg
+Copyright (c) 2016-2017, Steven Siloti
+Copyright (c) 2016, 2020, Alden Torres
+Copyright (c) 2020, Paul-Louis Ageneau
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -35,161 +39,113 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/config.hpp"
 
-#include "libtorrent/aux_/disable_warnings_push.hpp"
-
 #include <vector>
 #include <string>
 #include <list>
 #include <utility>
-#include <ctime>
-
-#include <boost/shared_ptr.hpp>
-#include <boost/enable_shared_from_this.hpp>
-#include <boost/cstdint.hpp>
-#include <boost/weak_ptr.hpp>
-#include <boost/tuple/tuple.hpp>
-#include <boost/unordered_map.hpp>
-#include <boost/optional.hpp>
-#include <boost/noncopyable.hpp>
-
-#ifdef TORRENT_USE_OPENSSL
-// there is no forward declaration header for asio
-namespace boost {
-namespace asio {
-namespace ssl {
-	struct context;
-}
-}
-}
-#endif
-
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
+#include <cstdint>
+#include <tuple>
+#include <functional>
+#include <memory>
+#include <unordered_map>
+#include <deque>
 
+#include "libtorrent/flags.hpp"
 #include "libtorrent/socket.hpp"
+#include "libtorrent/fwd.hpp"
 #include "libtorrent/address.hpp"
 #include "libtorrent/peer_id.hpp"
 #include "libtorrent/peer.hpp" // peer_entry
 #include "libtorrent/deadline_timer.hpp"
 #include "libtorrent/union_endpoint.hpp"
-#include "libtorrent/udp_socket.hpp" // for udp_socket_observer
-#include "libtorrent/io_service.hpp"
+#include "libtorrent/io_context.hpp"
+#include "libtorrent/span.hpp"
+#include "libtorrent/time.hpp"
+#include "libtorrent/debug.hpp"
+#include "libtorrent/error_code.hpp"
+#include "libtorrent/aux_/listen_socket_handle.hpp"
+#include "libtorrent/udp_socket.hpp"
+#include "libtorrent/aux_/session_settings.hpp"
+#include "libtorrent/ssl.hpp"
+
+namespace libtorrent {
 
-namespace libtorrent
-{
-	struct request_callback;
 	class tracker_manager;
 	struct timeout_handler;
-	struct tracker_connection;
 	class udp_tracker_connection;
 	class http_tracker_connection;
-	class  udp_socket;
-	struct resolver_interface;
 	struct counters;
-	struct ip_filter;
 #if TORRENT_USE_I2P
 	class i2p_connection;
 #endif
-	namespace aux { struct session_logger; struct session_settings; }
+namespace aux {
+	struct session_logger;
+	struct session_settings;
+	struct resolver_interface;
+}
 
-	// returns -1 if gzip header is invalid or the header size in bytes
-	TORRENT_EXTRA_EXPORT int gzip_header(const char* buf, int size);
+using tracker_request_flags_t = flags::bitfield_flag<std::uint8_t, struct tracker_request_flags_tag>;
+
+// Kinds of tracker announces. This is typically indicated as the ``&event=``
+// HTTP query string parameter to HTTP trackers.
+enum class event_t : std::uint8_t
+{
+	none,
+	completed,
+	started,
+	stopped,
+	paused
+};
 
 	struct TORRENT_EXTRA_EXPORT tracker_request
 	{
-		tracker_request()
-			: downloaded(-1)
-			, uploaded(-1)
-			, left(-1)
-			, corrupt(0)
-			, redundant(0)
-			, listen_port(0)
-			, event(none)
-			, kind(announce_request)
-			, key(0)
-			, num_want(0)
-			, send_stats(true)
-			, private_torrent(false)
-			, triggered_manually(false)
-			, second_announce(false)
-#ifdef TORRENT_USE_OPENSSL
-			, ssl_ctx(0)
-#endif
-#if TORRENT_USE_I2P
-			, i2pconn(0)
-#endif
-		{}
+		tracker_request() = default;
 
-		enum event_t
-		{
-			none,
-			completed,
-			started,
-			stopped,
-			paused
-		};
+		static constexpr tracker_request_flags_t scrape_request = 0_bit;
 
-		enum kind_t
-		{
-			// do not compare against announce_request ! check if not scrape instead
-			announce_request = 0,
-			scrape_request = 1,
-			// affects interpretation of peers string in HTTP response
-			// see parse_tracker_response()
-			i2p = 2
-		};
+		// affects interpretation of peers string in HTTP response
+		// see parse_tracker_response()
+		static constexpr tracker_request_flags_t i2p = 1_bit;
 
 		std::string url;
 		std::string trackerid;
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
 		std::string auth;
 #endif
 
-		boost::shared_ptr<const ip_filter> filter;
+		std::shared_ptr<const ip_filter> filter;
 
-		boost::int64_t downloaded;
-		boost::int64_t uploaded;
-		boost::int64_t left;
-		boost::int64_t corrupt;
-		boost::int64_t redundant;
-		boost::uint16_t listen_port;
+		std::int64_t downloaded = -1;
+		std::int64_t uploaded = -1;
+		std::int64_t left = -1;
+		std::int64_t corrupt = 0;
+		std::int64_t redundant = 0;
+		std::uint16_t listen_port = 0;
+		event_t event = event_t::none;
+		tracker_request_flags_t kind = {};
 
-		// values from event_t
-		boost::uint8_t event;
-
-		// values from kind_t
-		boost::uint8_t kind;
-
-		boost::uint32_t key;
-		int num_want;
-
-		address_v4 ipv4;
-#if TORRENT_USE_IPV6
-		address_v6 ipv6;
-#endif
+		std::uint32_t key = 0;
+		int num_want = 0;
+		std::vector<address_v6> ipv6;
+		std::vector<address_v4> ipv4;
 		sha1_hash info_hash;
 		peer_id pid;
-		boost::optional<address> bind_ip;
 
-		bool send_stats;
+		aux::listen_socket_handle outgoing_socket;
 
 		// set to true if the .torrent file this tracker announce is for is marked
 		// as private (i.e. has the "priv": 1 key)
-		bool private_torrent;
+		bool private_torrent = false;
 
 		// this is set to true if this request was triggered by a "manual" call to
 		// scrape_tracker() or force_reannounce()
-		bool triggered_manually;
+		bool triggered_manually = false;
 
-		// this is set when announcing to the next address family. There are only
-		// two address families now, so when this is set, we won't trigger another
-		// automatic announce
-		bool second_announce;
-
-#ifdef TORRENT_USE_OPENSSL
-		boost::asio::ssl::context* ssl_ctx;
+#if TORRENT_USE_SSL
+		ssl::context* ssl_ctx = nullptr;
 #endif
 #if TORRENT_USE_I2P
-		i2p_connection* i2pconn;
+		i2p_connection* i2pconn = nullptr;
 #endif
 	};
 
@@ -207,9 +163,7 @@ namespace libtorrent
 		// peers from the tracker, in various forms
 		std::vector<peer_entry> peers;
 		std::vector<ipv4_peer_entry> peers4;
-#if TORRENT_USE_IPV6
 		std::vector<ipv6_peer_entry> peers6;
-#endif
 		// our external IP address (if the tracker responded with ti, otherwise
 		// INADDR_ANY)
 		address external_ip;
@@ -226,10 +180,10 @@ namespace libtorrent
 		std::string warning_message;
 
 		// re-announce interval, in seconds
-		int interval;
+		seconds32 interval;
 
 		// the lowest force-announce interval
-		int min_interval;
+		seconds32 min_interval;
 
 		// the number of seeds in the swarm
 		int complete;
@@ -238,7 +192,7 @@ namespace libtorrent
 		int incomplete;
 
 		// if supported by the tracker, the number of actively downloading peers.
-		// i.e. partial seeds. If not suppored, -1
+		// i.e. partial seeds. If not supported, -1
 		int downloaders;
 
 		// the number of times the torrent has been downloaded
@@ -262,21 +216,24 @@ namespace libtorrent
 			, struct tracker_response const& response) = 0;
 		virtual void tracker_request_error(
 			tracker_request const& req
-			, int response_code
 			, error_code const& ec
+			, operation_t op
 			, const std::string& msg
-			, int retry_interval) = 0;
+			, seconds32 retry_interval) = 0;
 
 #ifndef TORRENT_DISABLE_LOGGING
-		virtual void debug_log(const char* fmt, ...) const TORRENT_FORMAT(2,3) = 0;
+		virtual bool should_log() const = 0;
+		virtual void debug_log(const char* fmt, ...) const noexcept TORRENT_FORMAT(2,3) = 0;
 #endif
 	};
 
 	struct TORRENT_EXTRA_EXPORT timeout_handler
-		: boost::enable_shared_from_this<timeout_handler>
-		, boost::noncopyable
+		: std::enable_shared_from_this<timeout_handler>
 	{
-		timeout_handler(io_service& str);
+		explicit timeout_handler(io_context&);
+
+		timeout_handler(timeout_handler const&) = delete;
+		timeout_handler& operator=(timeout_handler const&) = delete;
 
 		void set_timeout(int completion_timeout, int read_timeout);
 		void restart_read_timeout();
@@ -284,18 +241,15 @@ namespace libtorrent
 		bool cancelled() const { return m_abort; }
 
 		virtual void on_timeout(error_code const& ec) = 0;
-		virtual ~timeout_handler() {}
+		virtual ~timeout_handler();
 
-		io_service& get_io_service() { return lt::get_io_service(m_timeout); }
+		auto get_executor() { return m_timeout.get_executor(); }
 
 	private:
 
 		void timeout_callback(error_code const&);
 
-		int m_completion_timeout;
-
-		typedef mutex mutex_t;
-		mutable mutex_t m_mutex;
+		int m_completion_timeout = 0;
 
 		// used for timeouts
 		// this is set when the request has been sent
@@ -307,43 +261,39 @@ namespace libtorrent
 		// the asio async operation
 		deadline_timer m_timeout;
 
-		int m_read_timeout;
+		int m_read_timeout = 0;
 
-		bool m_abort;
+		bool m_abort = false;
+#if TORRENT_USE_ASSERTS
+		int m_outstanding_timer_wait = 0;
+#endif
 	};
 
-	// TODO: 2 this class probably doesn't need to have virtual functions.
 	struct TORRENT_EXTRA_EXPORT tracker_connection
 		: timeout_handler
 	{
 		tracker_connection(tracker_manager& man
-			, tracker_request const& req
-			, io_service& ios
-			, boost::weak_ptr<request_callback> r);
+			, tracker_request req
+			, io_context& ios
+			, std::weak_ptr<request_callback> r);
 
-		void update_transaction_id(boost::shared_ptr<udp_tracker_connection> c
-			, boost::uint64_t tid);
-
-		boost::shared_ptr<request_callback> requester() const;
-		virtual ~tracker_connection() {}
+		std::shared_ptr<request_callback> requester() const;
+		~tracker_connection() override {}
 
 		tracker_request const& tracker_req() const { return m_req; }
 
-		void fail(error_code const& ec, int code = -1, char const* msg = ""
-			, int interval = 0, int min_interval = 0);
+		void fail(error_code const& ec, operation_t op, char const* msg = ""
+			, seconds32 interval = seconds32(0), seconds32 min_interval = seconds32(0));
 		virtual void start() = 0;
-		virtual void close();
+		virtual void close() = 0;
+		address bind_interface() const;
+		aux::listen_socket_handle const& bind_socket() const { return m_req.outgoing_socket; }
 		void sent_bytes(int bytes);
 		void received_bytes(int bytes);
-		virtual bool on_receive(error_code const&, udp::endpoint const&
-			, char const* /* buf */, int /* size */) { return false; }
-		virtual bool on_receive_hostname(error_code const&
-			, char const* /* hostname */
-			, char const* /* buf */, int /* size */) { return false; }
 
-		boost::shared_ptr<tracker_connection> shared_from_this()
+		std::shared_ptr<tracker_connection> shared_from_this()
 		{
-			return boost::static_pointer_cast<tracker_connection>(
+			return std::static_pointer_cast<tracker_connection>(
 				timeout_handler::shared_from_this());
 		}
 
@@ -353,85 +303,110 @@ namespace libtorrent
 
 	protected:
 
-		void fail_impl(error_code const& ec, int code = -1, std::string msg = std::string()
-			, int interval = 0, int min_interval = 0);
+		void fail_impl(error_code const& ec, operation_t op, std::string msg = std::string()
+			, seconds32 interval = seconds32(0), seconds32 min_interval = seconds32(0));
 
-		boost::weak_ptr<request_callback> m_requester;
+		std::weak_ptr<request_callback> m_requester;
 
 		tracker_manager& m_man;
 	};
 
-	class TORRENT_EXTRA_EXPORT tracker_manager TORRENT_FINAL
-		: public udp_socket_observer
-		, boost::noncopyable
+	class TORRENT_EXTRA_EXPORT tracker_manager final
+		: single_threaded
 	{
 	public:
 
-		tracker_manager(udp_socket& sock
+		using send_fun_t = std::function<void(aux::listen_socket_handle const&
+			, udp::endpoint const&
+			, span<char const>
+			, error_code&, udp_send_flags_t)>;
+		using send_fun_hostname_t = std::function<void(aux::listen_socket_handle const&
+			, char const*, int
+			, span<char const>
+			, error_code&, udp_send_flags_t)>;
+
+		tracker_manager(send_fun_t send_fun
+			, send_fun_hostname_t send_fun_hostname
 			, counters& stats_counters
-			, resolver_interface& resolver
+			, aux::resolver_interface& resolver
 			, aux::session_settings const& sett
 #if !defined TORRENT_DISABLE_LOGGING || TORRENT_USE_ASSERTS
 			, aux::session_logger& ses
 #endif
 			);
-		virtual ~tracker_manager();
+
+		~tracker_manager();
+
+		tracker_manager(tracker_manager const&) = delete;
+		tracker_manager& operator=(tracker_manager const&) = delete;
 
 		void queue_request(
-			io_service& ios
-			, tracker_request r
-			, boost::weak_ptr<request_callback> c
-				= boost::weak_ptr<request_callback>());
+			io_context& ios
+			, tracker_request&& r
+			, aux::session_settings const& sett
+			, std::weak_ptr<request_callback> c
+				= std::weak_ptr<request_callback>());
+		void queue_request(
+			io_context& ios
+			, tracker_request const& r
+			, aux::session_settings const& sett
+			, std::weak_ptr<request_callback> c
+				= std::weak_ptr<request_callback>()) = delete;
 		void abort_all_requests(bool all = false);
+		void stop();
 
-		void remove_request(tracker_connection const*);
+		void remove_request(http_tracker_connection const* c);
+		void remove_request(udp_tracker_connection const* c);
 		bool empty() const;
 		int num_requests() const;
 
 		void sent_bytes(int bytes);
 		void received_bytes(int bytes);
 
-		virtual bool incoming_packet(error_code const& e, udp::endpoint const& ep
-			, char const* buf, int size) TORRENT_OVERRIDE;
+		void incoming_error(error_code const& ec, udp::endpoint const& ep);
+		bool incoming_packet(udp::endpoint const& ep, span<char const> buf);
 
 		// this is only used for SOCKS packets, since
 		// they may be addressed to hostname
-		virtual bool incoming_packet(error_code const& e, char const* hostname
-			, char const* buf, int size) TORRENT_OVERRIDE;
+		// TODO: 3 make sure the udp_socket supports passing on string-hostnames
+		// too, and that this function is used
+		bool incoming_packet(char const* hostname, span<char const> buf);
 
 		void update_transaction_id(
-			boost::shared_ptr<udp_tracker_connection> c
-			, boost::uint64_t tid);
+			std::shared_ptr<udp_tracker_connection> c
+			, std::uint32_t tid);
 
 		aux::session_settings const& settings() const { return m_settings; }
-		udp_socket& get_udp_socket() { return m_udp_socket; }
-		resolver_interface& host_resolver() { return m_host_resolver; }
+		aux::resolver_interface& host_resolver() { return m_host_resolver; }
+
+		void send_hostname(aux::listen_socket_handle const& sock
+			, char const* hostname, int port, span<char const> p
+			, error_code& ec, udp_send_flags_t flags = {});
+
+		void send(aux::listen_socket_handle const& sock
+			, udp::endpoint const& ep, span<char const> p
+			, error_code& ec, udp_send_flags_t flags = {});
 
 	private:
 
-		typedef mutex mutex_t;
-		mutable mutex_t m_mutex;
-
 		// maps transactionid to the udp_tracker_connection
-		// TODO: this should be unique_ptr in the future
-		typedef boost::unordered_map<boost::uint32_t
-			, boost::shared_ptr<udp_tracker_connection> > udp_conns_t;
-		udp_conns_t m_udp_conns;
+		// These must use shared_ptr to avoid a dangling reference
+		// if a connection is erased while a timeout event is in the queue
+		std::unordered_map<std::uint32_t, std::shared_ptr<udp_tracker_connection>> m_udp_conns;
 
-		typedef std::vector<boost::shared_ptr<http_tracker_connection> > http_conns_t;
-		http_conns_t m_http_conns;
+		std::vector<std::shared_ptr<http_tracker_connection>> m_http_conns;
+		std::deque<std::shared_ptr<http_tracker_connection>> m_queued;
 
-		class udp_socket& m_udp_socket;
-		resolver_interface& m_host_resolver;
+		send_fun_t m_send_fun;
+		send_fun_hostname_t m_send_fun_hostname;
+		aux::resolver_interface& m_host_resolver;
 		aux::session_settings const& m_settings;
 		counters& m_stats_counters;
+		bool m_abort = false;
 #if !defined TORRENT_DISABLE_LOGGING || TORRENT_USE_ASSERTS
 		aux::session_logger& m_ses;
 #endif
-
-		bool m_abort;
 	};
 }
 
 #endif // TORRENT_TRACKER_MANAGER_HPP_INCLUDED
-
