@@ -31,12 +31,15 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "disk_io.hpp"
+#include "utils.hpp"
 #include "libtorrent/torrent_info.hpp"
 #include "libtorrent/io_context.hpp"
 #include "libtorrent/add_torrent_params.hpp"
 #include "libtorrent/error_code.hpp"
 #include "libtorrent/deadline_timer.hpp"
 #include "libtorrent/disk_observer.hpp"
+#include "libtorrent/aux_/apply_pad_files.hpp"
+#include "libtorrent/random.hpp"
 
 #include <utility> // for exchange()
 
@@ -54,38 +57,59 @@ namespace {
 
 } // anonymous namespace
 
-std::array<char, 4> generate_block_fill(lt::piece_index_t const p, int const block)
+std::array<char, 0x4000> generate_block_fill(lt::piece_index_t const p, int const block)
 {
 	int const v = (static_cast<int>(p) << 8) | (block & 0xff);
-	std::array<char, 4> ret;
-	std::memcpy(ret.data(), reinterpret_cast<char const*>(&v), 4);
+	std::array<char, 0x4000> ret;
+	for (int i = 0; i < 0x4000; i += 4)
+	{
+		std::memcpy(ret.data() + i, reinterpret_cast<char const*>(&v), 4);
+	}
 	return ret;
 }
 
-lt::sha1_hash generate_hash1(lt::piece_index_t const p, lt::file_storage const& fs)
+lt::sha1_hash generate_hash1(lt::piece_index_t const p, lt::file_storage const& fs, int const pad_bytes)
 {
 	lt::hasher ret;
 	int const piece_size = fs.piece_size(p);
+	int const payload_size = piece_size - pad_bytes;
 	int offset = 0;
-	for (int block = 0; offset < piece_size; ++block)
+	for (int block = 0; offset < payload_size; ++block)
 	{
 		auto const fill = generate_block_fill(p, block);
-		for (int i = 0; i < lt::default_block_size; i += fill.size(), offset += fill.size())
-			ret.update(fill.data(), std::min(int(fill.size()), piece_size - offset));
+		for (int i = 0; i < lt::default_block_size;)
+		{
+			int const bytes = std::min(int(fill.size()), payload_size - offset);
+			if (bytes <= 0) break;
+			ret.update(fill.data(), bytes);
+			offset += bytes;
+			i += bytes;
+		}
+	}
+	TORRENT_ASSERT(piece_size - offset == pad_bytes);
+	std::array<char, 8> const pad{{0, 0, 0, 0, 0, 0, 0, 0}};
+	while (offset < piece_size)
+	{
+		int const bytes = std::min(int(pad.size()), piece_size - offset);
+		ret.update(pad.data(), bytes);
+		offset += bytes;
 	}
 	return ret.final();
 }
 
 lt::sha1_hash generate_hash2(lt::piece_index_t p, lt::file_storage const& fs
-	, lt::span<lt::sha256_hash> const hashes)
+	, lt::span<lt::sha256_hash> const hashes, int const pad_bytes)
 {
 	int const piece_size = fs.piece_size(p);
+	int const payload_size = piece_size - pad_bytes;
 	int const piece_size2 = fs.piece_size2(p);
 	int const blocks_in_piece = (piece_size + lt::default_block_size - 1) / lt::default_block_size;
 	int const blocks_in_piece2 = fs.blocks_in_piece2(p);
 	TORRENT_ASSERT(int(hashes.size()) >= blocks_in_piece2);
 	int const blocks_to_read = std::max(blocks_in_piece, blocks_in_piece2);
 
+	TORRENT_ASSERT(piece_size - pad_bytes == piece_size2);
+
 	lt::hasher ret;
 	int offset = 0;
 	for (int block = 0; block < blocks_to_read; ++block)
@@ -95,24 +119,34 @@ lt::sha1_hash generate_hash2(lt::piece_index_t p, lt::file_storage const& fs
 
 		bool const v2 = piece_size2 - offset > 0;
 
-		int const block_size = std::min(lt::default_block_size, std::max(piece_size, piece_size2) - offset);
-		for (int i = 0; i < block_size; i += fill.size(), offset += fill.size())
+		int const block_size = std::min(lt::default_block_size, payload_size - offset);
+		for (int i = 0; i < block_size;)
 		{
-			if (piece_size - offset > 0)
-				ret.update(fill.data(), std::min(int(fill.size()), piece_size - offset));
+			int const bytes = std::min(int(fill.size()), payload_size - offset);
+			TORRENT_ASSERT(bytes > 0);
+			ret.update(fill.data(), bytes);
+
 			if (piece_size2 - offset > 0)
 				v2_hash.update(fill.data(), std::min(int(fill.size()), piece_size2 - offset));
+
+			offset += bytes;
+			i += bytes;
+		}
+		if (offset < piece_size)
+		{
+			std::vector<char> padding(piece_size - offset, 0);
+			ret.update(padding);
 		}
 		if (v2)
 			hashes[block] = v2_hash.final();
-		else
-			hashes[block].clear();
 	}
 	return ret.final();
 }
 
 lt::sha256_hash generate_block_hash(lt::piece_index_t p, int const offset)
 {
+	// TODO: this function is not correct for files whose size are not divisible
+	// by the block size (for the last block)
 	lt::hasher256 ret;
 	int const block = offset / lt::default_block_size;
 	auto const fill = generate_block_fill(p, block);
@@ -121,46 +155,112 @@ lt::sha256_hash generate_block_hash(lt::piece_index_t p, int const offset)
 	return ret.final();
 }
 
-void generate_block(char* b, lt::peer_request const& r)
+void generate_block(char* b, lt::peer_request const& r, int const pad_bytes)
 {
 	auto const fill = generate_block_fill(r.piece, (r.start / lt::default_block_size));
-	for (int i = 0; i < lt::default_block_size; i += fill.size())
+
+	// for now we don't support unaligned start address
+	TORRENT_ASSERT((r.start % fill.size()) == 0);
+	char* end = b + r.length - pad_bytes;
+	while (b < end)
 	{
-		std::memcpy(b, fill.data(), fill.size());
-		b += fill.size();
+		int const bytes = std::min(int(fill.size()), int(end - b));
+		std::memcpy(b, fill.data(), bytes);
+		b += bytes;
 	}
+
+	if (pad_bytes > 0)
+		std::memset(b, 0, pad_bytes);
+}
+
+std::unordered_map<lt::piece_index_t, int> compute_pad_bytes(lt::file_storage const& fs)
+{
+	std::unordered_map<lt::piece_index_t, int> ret;
+	lt::aux::apply_pad_files(fs, [&](lt::piece_index_t p, int bytes)
+	{
+		ret.emplace(p, bytes);
+	});
+	return ret;
+}
+
+int pads_in_piece(std::unordered_map<lt::piece_index_t, int> const& pb, lt::piece_index_t const p)
+{
+	auto it = pb.find(p);
+	return (it == pb.end()) ? 0 : it->second;
+}
+
+int pads_in_req(std::unordered_map<lt::piece_index_t, int> const& pb
+	, lt::peer_request const& r, int const piece_size)
+{
+	auto it = pb.find(r.piece);
+	if (it == pb.end()) return 0;
+
+	int const pad_start = piece_size - it->second;
+	int const req_end = r.start + r.length;
+	return std::max(0, std::min(req_end - pad_start, r.length));
 }
 
 std::shared_ptr<lt::torrent_info> create_test_torrent(int const piece_size
-	, int const num_pieces, lt::create_flags_t const flags)
+	, int const num_pieces, lt::create_flags_t const flags, int const num_files)
 {
-	lt::file_storage fs;
-	int const total_size = piece_size * num_pieces;
-	fs.add_file("file-1", total_size);
-	lt::create_torrent t(fs, piece_size, flags);
+	lt::file_storage ifs;
+	int total_size = num_files * piece_size * num_pieces + 1234;
+	if (num_files == 1)
+	{
+		ifs.add_file("file-1", total_size);
+	}
+	else
+	{
+		int const file_size = total_size / num_files + 10;
+		for (int i = 0; i < num_files; ++i)
+		{
+			int const this_size = std::min(file_size, total_size);
+			ifs.add_file("test-torrent/file-" + std::to_string(i + 1), this_size);
+			total_size -= this_size;
+		}
+	}
+	lt::create_torrent t(ifs, piece_size, flags);
+
+	lt::file_storage const& fs = t.files();
+
+	auto const pad_bytes = compute_pad_bytes(fs);
 
 	if (flags & lt::create_torrent::v1_only)
 	{
 		for (auto const i : fs.piece_range())
-			t.set_hash(i, generate_hash1(i, fs));
+			t.set_hash(i, generate_hash1(i, fs, pads_in_piece(pad_bytes, i)));
 	}
 	else
 	{
-		int const blocks_in_piece = piece_size / lt::default_block_size;
-		TORRENT_ASSERT(blocks_in_piece * lt::default_block_size == piece_size);
+		int const blocks_per_piece = piece_size / lt::default_block_size;
+		TORRENT_ASSERT(blocks_per_piece * lt::default_block_size == piece_size);
+		// blocks per piece must be a power of two
+		TORRENT_ASSERT((blocks_per_piece & (blocks_per_piece - 1)) == 0);
 
-		int const num_leafs = lt::merkle_num_leafs(blocks_in_piece);
-		lt::aux::vector<lt::sha256_hash> v2tree(lt::merkle_num_nodes(num_leafs));
-		auto const blocks = lt::span<lt::sha256_hash>(v2tree).subspan(lt::merkle_first_leaf(num_leafs));
+		lt::aux::vector<lt::sha256_hash> blocks(blocks_per_piece);
+		std::vector<lt::sha256_hash> scratch_space;
 
-		for (auto const i : fs.piece_range())
+		for (auto const f : fs.file_range())
 		{
-			auto const hash = generate_hash2(i, fs, blocks);
-			lt::merkle_fill_tree(v2tree, num_leafs);
-			t.set_hash2(lt::file_index_t{0}, i - 0_piece, v2tree[0]);
+			if (fs.pad_file_at(f)) continue;
+			auto const file_piece = fs.piece_index_at_file(f);
+			for (auto const p : fs.file_piece_range(f))
+			{
+				auto const piece = file_piece + p;
+				auto const blocks_in_piece = fs.blocks_in_piece2(piece);
+				TORRENT_ASSERT(blocks_in_piece > 0);
+				TORRENT_ASSERT(blocks_in_piece <= int(blocks.size()));
+				auto const hash = generate_hash2(piece, fs, blocks, pads_in_piece(pad_bytes, piece));
+				auto const piece_layer_hash = lt::merkle_root_scratch(
+					lt::span<lt::sha256_hash const>(blocks).first(blocks_in_piece)
+					, blocks_per_piece
+					, {}
+					, scratch_space);
+				t.set_hash2(f, p, piece_layer_hash);
 
-			if (!(flags & lt::create_torrent::v2_only))
-				t.set_hash(i, hash);
+				if (!(flags & lt::create_torrent::v2_only))
+					t.set_hash(file_piece + p, hash);
+			}
 		}
 	}
 
@@ -173,10 +273,11 @@ std::shared_ptr<lt::torrent_info> create_test_torrent(int const piece_size
 }
 
 lt::add_torrent_params create_test_torrent(
-	int const num_pieces, lt::create_flags_t const flags)
+	int const num_pieces, lt::create_flags_t const flags, int const blocks_per_piece
+	, int const num_files)
 {
 	lt::add_torrent_params params;
-	params.ti = ::create_test_torrent(lt::default_block_size * 2, num_pieces, flags);
+	params.ti = ::create_test_torrent(lt::default_block_size * blocks_per_piece, num_pieces, flags, num_files);
 	// this is unused by the test disk I/O
 	params.save_path = ".";
 	return params;
@@ -211,7 +312,16 @@ struct test_disk_io final : lt::disk_interface
 		lt::file_storage const& fs = params.files;
 		m_files = &fs;
 		m_blocks_per_piece = fs.piece_length() / lt::default_block_size;
-		m_have.resize(m_files->num_pieces() * m_blocks_per_piece, m_state.seed);
+		m_have.resize(m_files->num_pieces() * m_blocks_per_piece, m_state.files == existing_files_mode::full_valid);
+		m_pad_bytes = compute_pad_bytes(fs);
+
+		if (m_state.files == existing_files_mode::partial_valid)
+		{
+			// we have the first half of the blocks
+			for (std::size_t i = 0; i < m_have.size() / 2u; ++i)
+				m_have.set_bit(i);
+		}
+
 		return lt::storage_holder(lt::storage_index_t{0}, *this);
 	}
 
@@ -241,15 +351,18 @@ struct test_disk_io final : lt::disk_interface
 		TORRENT_ASSERT((r.start % lt::default_block_size) == 0);
 		TORRENT_ASSERT((r.length <= lt::default_block_size));
 
-		// this block should have been written
-		TORRENT_ASSERT(m_have.get_bit(block_index(r)));
-
-		auto const seek_time = disk_seek(std::int64_t(static_cast<int>(r.piece)) * m_files->piece_length() + r.start
-			, lt::default_block_size);
+		auto const seek_time = disk_seek(r.piece, r.start, lt::default_block_size);
 
 		queue_event(seek_time + m_state.read_time, [this,r, h=std::move(h)] () mutable {
 			lt::disk_buffer_holder buf(*this, new char[lt::default_block_size], r.length);
-			generate_block(buf.data(), r);
+
+			if (m_have.get_bit(block_index(r)))
+			{
+				if (m_state.corrupt_data_in-- <= 0)
+					lt::aux::random_bytes(buf);
+				else
+					generate_block(buf.data(), r, pads_in_req(m_pad_bytes, r, m_files->piece_size(r.piece)));
+			}
 
 			post(m_ioc, [h=std::move(h), b=std::move(buf)] () mutable { h(std::move(b), lt::storage_error{}); });
 		});
@@ -282,10 +395,9 @@ struct test_disk_io final : lt::disk_interface
 			return false;
 		}
 
-		bool const valid = validate_block(buf, r);
+		bool const valid = validate_block(*m_files, buf, r);
 
-		auto const seek_time = disk_seek(std::int64_t(static_cast<int>(r.piece)) * m_files->piece_length() + r.start
-			, lt::default_block_size);
+		auto const seek_time = disk_seek(r.piece, r.start, lt::default_block_size);
 
 		queue_event(seek_time + m_state.write_time, [this,valid,r,h=std::move(handler)] () mutable {
 
@@ -319,8 +431,7 @@ struct test_disk_io final : lt::disk_interface
 	{
 		TORRENT_ASSERT(m_files);
 
-		auto const seek_time = disk_seek(std::int64_t(static_cast<int>(piece)) * m_files->piece_length()
-			, m_blocks_per_piece * lt::default_block_size);
+		auto const seek_time = disk_seek(piece, 0, m_blocks_per_piece * lt::default_block_size);
 
 		auto const delay = seek_time
 			+ m_state.read_time * m_blocks_per_piece
@@ -329,22 +440,32 @@ struct test_disk_io final : lt::disk_interface
 
 		queue_event(delay, [this, piece, block_hashes, h=std::move(handler)] () mutable {
 
+			int const piece_size = m_files->piece_size(piece);
+			int const pad_bytes = pads_in_piece(m_pad_bytes, piece);
+			int const payload_blocks = piece_size / lt::default_block_size - pad_bytes / lt::default_block_size;
 			int const block_idx = piece * m_blocks_per_piece;
-			for (int i = 0; i < m_blocks_per_piece; ++i)
+			for (int i = 0; i < payload_blocks; ++i)
 			{
 				if (!m_have.get_bit(block_idx + i))
 				{
-					// If we're missing a block, return an invalida hash
-					post(m_ioc, [h=std::move(h), piece]{ h(piece, lt::sha1_hash{}, lt::storage_error{}); });
+					lt::sha1_hash ph{};
+					if (m_state.files == existing_files_mode::full_invalid)
+					{
+						for (auto& h : block_hashes)
+							h = rand_sha256();
+						ph = rand_sha1();
+					}
+					// If we're missing a block, return an invalid hash
+					post(m_ioc, [h=std::move(h), piece, ph]{ h(piece, ph, lt::storage_error{}); });
 					return;
 				}
 			}
 
 			lt::sha1_hash hash;
 			if (block_hashes.empty())
-				hash = generate_hash1(piece, *m_files);
+				hash = generate_hash1(piece, *m_files, pads_in_piece(m_pad_bytes, piece));
 			else
-				hash = generate_hash2(piece, *m_files, block_hashes);
+				hash = generate_hash2(piece, *m_files, block_hashes, pads_in_piece(m_pad_bytes, piece));
 			post(m_ioc, [h=std::move(h), piece, hash]{ h(piece, hash, lt::storage_error{}); });
 		});
 	}
@@ -355,21 +476,22 @@ struct test_disk_io final : lt::disk_interface
 	{
 		TORRENT_ASSERT(m_files);
 
-		auto const seek_time = disk_seek(std::int64_t(static_cast<int>(piece)) * m_files->piece_length() + offset
-			, m_blocks_per_piece * lt::default_block_size);
+		auto const seek_time = disk_seek(piece, offset, m_blocks_per_piece * lt::default_block_size);
 
 		auto const delay = seek_time + m_state.hash_time + m_state.read_time;
 		queue_event(delay, [this, piece, offset, h=std::move(handler)] () mutable {
 			int const block_idx = piece * m_blocks_per_piece + offset / lt::default_block_size;
+			lt::sha256_hash hash;
 			if (!m_have.get_bit(block_idx))
 			{
-				post(m_ioc, [h=std::move(h),piece] { h(piece, lt::sha256_hash{}, lt::storage_error{}); });
+				if (m_state.files == existing_files_mode::full_invalid)
+					hash = rand_sha256();
 			}
 			else
 			{
-				lt::sha256_hash const hash = generate_block_hash(piece, offset);
-				post(m_ioc, [h=std::move(h),piece, hash] { h(piece, hash, lt::storage_error{}); });
+				hash = generate_block_hash(piece, offset);
 			}
+			post(m_ioc, [h=std::move(h),piece, hash] { h(piece, hash, lt::storage_error{}); });
 		});
 	}
 
@@ -406,9 +528,13 @@ struct test_disk_io final : lt::disk_interface
 		, std::function<void(lt::status_t, lt::storage_error const&)> handler) override
 	{
 		TORRENT_ASSERT(m_files);
-		auto const ret = (!m_state.seed || (p->flags & lt::torrent_flags::seed_mode))
-			? lt::status_t::no_error
-			: lt::status_t::need_full_check;
+
+		auto ret = lt::status_t::need_full_check;
+		if (p && p->flags & lt::torrent_flags::seed_mode)
+			ret = lt::status_t::no_error;
+		else if (m_state.files == existing_files_mode::no_files)
+			ret = lt::status_t::no_error;
+
 		queue_event(lt::microseconds(1), [this,ret,h=std::move(handler)] () mutable {
 			post(m_ioc, [ret,h=std::move(h)] { h(ret, lt::storage_error()); });
 		});
@@ -463,8 +589,9 @@ struct test_disk_io final : lt::disk_interface
 
 private:
 
-	lt::time_duration disk_seek(std::int64_t const offset, int const size = lt::default_block_size)
+	lt::time_duration disk_seek(lt::piece_index_t const piece, int start, int const size = lt::default_block_size)
 	{
+		std::int64_t const offset = std::int64_t(static_cast<int>(piece)) * m_files->piece_length() + start;
 		return (std::exchange(m_last_disk_offset, offset + size) == offset)
 			? lt::milliseconds(0) : m_state.seek_time;
 	}
@@ -474,13 +601,24 @@ private:
 		return r.piece * m_blocks_per_piece + r.start / lt::default_block_size;
 	}
 
-	bool validate_block(char const* b, lt::peer_request const& r) const
+	bool validate_block(lt::file_storage const& fs, char const* b, lt::peer_request const& r) const
 	{
 		auto const fill = generate_block_fill(r.piece, r.start / lt::default_block_size);
-		for (int i = 0; i < lt::default_block_size; i += fill.size())
+		int const piece_size = fs.piece_size(r.piece);
+		int payload_bytes = (piece_size - pads_in_piece(m_pad_bytes, r.piece)) - r.start;
+		int offset = 0;
+		while (offset < r.length && payload_bytes > 0)
 		{
-			if (std::memcmp(b, fill.data(), fill.size()) != 0) return false;
-			b += fill.size();
+			int const to_compare = std::min(payload_bytes, int(fill.size()));
+			if (std::memcmp(b, fill.data(), to_compare) != 0) return false;
+			b += to_compare;
+			offset += to_compare;
+			payload_bytes -= to_compare;
+		}
+		if (offset < r.length)
+		{
+			// the pad bytes must be zero
+			return std::all_of(b, b + r.length - offset, [](char const c) { return c == 0; });
 		}
 		return true;
 	}
@@ -508,7 +646,7 @@ private:
 		{
 			auto f = std::move(m_event_queue.front().second);
 			m_event_queue.pop_front();
-			f();
+			if (f) f();
 		}
 
 		if (m_event_queue.empty())
@@ -534,7 +672,7 @@ private:
 	int m_write_queue = 0;
 	bool m_exceeded_max_size = false;
 
-	// events that is supposed to trigger in the future are put in this queue
+	// events that are supposed to trigger in the future are put in this queue
 	std::deque<std::pair<lt::time_point, std::function<void()>>> m_event_queue;
 	lt::deadline_timer m_timer;
 
@@ -557,6 +695,8 @@ private:
 
 	// callbacks are posted on this
 	lt::io_context& m_ioc;
+
+	std::unordered_map<lt::piece_index_t, int> m_pad_bytes;
 };
 
 std::unique_ptr<lt::disk_interface> test_disk::operator()(
