@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2009-2018, Arvid Norberg
+Copyright (c) 2009, Georg Rudoy
+Copyright (c) 2009, 2012, 2016-2017, 2019-2020, Arvid Norberg
+Copyright (c) 2018, 2020, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,28 +32,23 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/aux_/disable_warnings_push.hpp"
-
-#include <boost/cstdint.hpp>
-
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
-
-#include "libtorrent/bandwidth_queue_entry.hpp"
-#include <cstring>
+#include <cstdint>
 #include <algorithm>
 
-namespace libtorrent
-{
-	bw_request::bw_request(boost::shared_ptr<bandwidth_socket> const& pe
+#include "libtorrent/aux_/bandwidth_queue_entry.hpp"
+
+namespace libtorrent {
+namespace aux {
+
+	bw_request::bw_request(std::shared_ptr<bandwidth_socket> pe
 		, int blk, int prio)
-		: peer(pe)
+		: peer(std::move(pe))
 		, priority(prio)
 		, assigned(0)
 		, request_size(blk)
 		, ttl(20)
 	{
 		TORRENT_ASSERT(priority > 0);
-		std::memset(channel, 0, sizeof(channel));
 	}
 
 	int bw_request::assign_bandwidth()
@@ -66,7 +63,7 @@ namespace libtorrent
 		{
 			if (channel[j]->throttle() == 0) continue;
 			if (channel[j]->tmp == 0) continue;
-			quota = (std::min)(int(boost::int64_t(channel[j]->distribute_quota)
+			quota = std::min(int(std::int64_t(channel[j]->distribute_quota)
 				* priority / channel[j]->tmp), quota);
 		}
 		assigned += quota;
@@ -75,5 +72,10 @@ namespace libtorrent
 		TORRENT_ASSERT(assigned <= request_size);
 		return quota;
 	}
-}
 
+	static_assert(std::is_nothrow_move_constructible<bw_request>::value
+		, "should be nothrow move constructible");
+	static_assert(std::is_nothrow_move_assignable<bw_request>::value
+		, "should be nothrow move assignable");
+}
+}
