@@ -1,6 +1,10 @@
 /*
 
-Copyright (c) 2015-2018, Arvid Norberg
+Copyright (c) 2015-2020, Arvid Norberg
+Copyright (c) 2016-2017, Steven Siloti
+Copyright (c) 2016-2017, Andrei Kurushin
+Copyright (c) 2016-2017, 2019, Alden Torres
+Copyright (c) 2017, Pavel Pimenov
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -38,6 +42,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <cstring> // for memset
 #include <cstdio> // for snprintf
 #include <cinttypes> // for PRId64 et.al.
+#include <algorithm> // for any_of
 
 #ifndef BOOST_SYSTEM_NOEXCEPT
 #define BOOST_SYSTEM_NOEXCEPT throw()
@@ -45,7 +50,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 namespace libtorrent {
 
-	using detail::bdecode_token;
+	using aux::bdecode_token;
 
 namespace {
 
@@ -124,22 +129,22 @@ namespace {
 
 } // anonymous namespace
 
-namespace detail {
+namespace aux {
 	void escape_string(std::string& ret, char const* str, int len)
 	{
-		for (int i = 0; i < len; ++i)
+		if (std::any_of(str, str + len, [](char const c) { return c < 32 || c >= 127; } ))
 		{
-			if (str[i] >= 32 && str[i] < 127)
+			for (int i = 0; i < len; ++i)
 			{
-				ret += str[i];
-			}
-			else
-			{
-				char tmp[5];
-				std::snprintf(tmp, sizeof(tmp), "\\x%02x", std::uint8_t(str[i]));
+				char tmp[3];
+				std::snprintf(tmp, sizeof(tmp), "%02x", std::uint8_t(str[i]));
 				ret += tmp;
 			}
 		}
+		else
+		{
+			ret.assign(str, std::size_t(len));
+		}
 	}
 }
 
@@ -243,7 +248,7 @@ namespace detail {
 		(*this) = n;
 	}
 
-	bdecode_node& bdecode_node::operator=(bdecode_node const& n)
+	bdecode_node& bdecode_node::operator=(bdecode_node const& n) &
 	{
 		if (&n == this) return *this;
 		m_tokens = n.m_tokens;
@@ -271,9 +276,6 @@ namespace detail {
 		, m_buffer(buf)
 		, m_buffer_size(len)
 		, m_token_idx(idx)
-		, m_last_index(-1)
-		, m_last_token(-1)
-		, m_size(-1)
 	{
 		TORRENT_ASSERT(tokens != nullptr);
 		TORRENT_ASSERT(idx >= 0);
@@ -333,7 +335,6 @@ namespace detail {
 				}
 				break;
 			case bdecode_token::string:
-			case bdecode_token::long_string:
 				if (m_buffer[tokens[token].offset] == '0'
 					&& m_buffer[tokens[token].offset + 1] != ':')
 				{
@@ -400,8 +401,6 @@ namespace detail {
 	bdecode_node::type_t bdecode_node::type() const noexcept
 	{
 		if (m_token_idx == -1) return none_t;
-		if (m_root_tokens[m_token_idx].type == bdecode_token::long_string)
-			return bdecode_node::string_t;
 		return static_cast<bdecode_node::type_t>(m_root_tokens[m_token_idx].type);
 	}
 
@@ -418,6 +417,14 @@ namespace detail {
 		return {m_buffer + t.offset, static_cast<std::ptrdiff_t>(next.offset - t.offset)};
 	}
 
+	std::ptrdiff_t bdecode_node::data_offset() const noexcept
+	{
+		TORRENT_ASSERT_PRECOND(m_token_idx != -1);
+		if (m_token_idx == -1) return -1;
+		bdecode_token const& t = m_root_tokens[m_token_idx];
+		return t.offset;
+	}
+
 	bdecode_node bdecode_node::list_at(int i) const
 	{
 		TORRENT_ASSERT(type() == list_t);
@@ -499,7 +506,7 @@ namespace detail {
 		return ret;
 	}
 
-	std::pair<string_view, bdecode_node> bdecode_node::dict_at(int i) const
+	std::pair<bdecode_node, bdecode_node> bdecode_node::dict_at_node(int i) const
 	{
 		TORRENT_ASSERT(type() == dict_t);
 		TORRENT_ASSERT(m_token_idx != -1);
@@ -519,8 +526,7 @@ namespace detail {
 
 		while (item < i)
 		{
-			TORRENT_ASSERT(tokens[token].type == bdecode_token::string
-				|| tokens[token].type == bdecode_token::long_string);
+			TORRENT_ASSERT(tokens[token].type == bdecode_token::string);
 
 			// skip the key
 			token += tokens[token].next_item;
@@ -546,10 +552,18 @@ namespace detail {
 		TORRENT_ASSERT(tokens[token].type != bdecode_token::end);
 
 		return std::make_pair(
-			bdecode_node(tokens, m_buffer, m_buffer_size, token).string_value()
+			bdecode_node(tokens, m_buffer, m_buffer_size, token)
 			, bdecode_node(tokens, m_buffer, m_buffer_size, value_token));
 	}
 
+	std::pair<string_view, bdecode_node> bdecode_node::dict_at(int const i) const
+	{
+		bdecode_node key;
+		bdecode_node value;
+		std::tie(key, value) = dict_at_node(i);
+		return {key.string_value(), value};
+	}
+
 	int bdecode_node::dict_size() const
 	{
 		TORRENT_ASSERT(type() == dict_t);
@@ -600,8 +614,7 @@ namespace detail {
 		while (tokens[token].type != bdecode_token::end)
 		{
 			bdecode_token const& t = tokens[token];
-			TORRENT_ASSERT(t.type == bdecode_token::string
-				|| t.type == bdecode_token::long_string);
+			TORRENT_ASSERT(t.type == bdecode_token::string);
 			int const size = token_source_span(t) - t.start_offset();
 			if (int(key.size()) == size
 				&& std::equal(key.data(), key.data() + size, m_buffer
@@ -698,19 +711,25 @@ namespace detail {
 	{
 		TORRENT_ASSERT(type() == string_t);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
-		std::size_t const size = aux::numeric_cast<std::size_t>(token_source_span(t) - t.start_offset());
-		TORRENT_ASSERT(t.type == bdecode_token::string
-			|| t.type == bdecode_token::long_string);
+		auto const size = aux::numeric_cast<std::size_t>(token_source_span(t) - t.start_offset());
+		TORRENT_ASSERT(t.type == bdecode_token::string);
 
 		return string_view(m_buffer + t.offset + t.start_offset(), size);
 	}
 
+	std::ptrdiff_t bdecode_node::string_offset() const
+	{
+		TORRENT_ASSERT(type() == string_t);
+		bdecode_token const& t = m_root_tokens[m_token_idx];
+		TORRENT_ASSERT(t.type == bdecode_token::string);
+		return t.offset + t.start_offset();
+	}
+
 	char const* bdecode_node::string_ptr() const
 	{
 		TORRENT_ASSERT(type() == string_t);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
-		TORRENT_ASSERT(t.type == bdecode_token::string
-			|| t.type == bdecode_token::long_string);
+		TORRENT_ASSERT(t.type == bdecode_token::string);
 		return m_buffer + t.offset + t.start_offset();
 	}
 
@@ -718,8 +737,7 @@ namespace detail {
 	{
 		TORRENT_ASSERT(type() == string_t);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
-		TORRENT_ASSERT(t.type == bdecode_token::string
-			|| t.type == bdecode_token::long_string);
+		TORRENT_ASSERT(t.type == bdecode_token::string);
 		return token_source_span(t) - t.start_offset();
 	}
 
@@ -941,16 +959,13 @@ namespace detail {
 					// the check above ensures that the buffer is long enough to hold
 					// the string's length which guarantees that start <= end
 
-					int const header = int(start - str_start);
-					TORRENT_ASSERT(header >= 0);
-
 					// the bdecode_token only has 8 bits to keep the header size
 					// in. If it overflows, fail!
-					if (header > detail::bdecode_token::long_string_max_header)
+					if (start - str_start - 2 > aux::bdecode_token::max_header)
 						TORRENT_FAIL_BDECODE(bdecode_errors::limit_exceeded);
 
 					ret.m_tokens.push_back({str_start - orig_start
-						, 1, bdecode_token::string, std::uint32_t(header)});
+						, 1, bdecode_token::string, std::uint8_t(start - str_start)});
 					start += len;
 					break;
 				}
@@ -1080,15 +1095,15 @@ done:
 			ret += "'";
 			return;
 		}
-		if (single_line && len > 20)
+		if (single_line && len > 32)
 		{
-			detail::escape_string(ret, str.data(), 9);
+			aux::escape_string(ret, str.data(), 25);
 			ret += "...";
-			detail::escape_string(ret, str.data() + len - 9, 9);
+			aux::escape_string(ret, str.data() + len - 4, 4);
 		}
 		else
 		{
-			detail::escape_string(ret, str.data(), len);
+			aux::escape_string(ret, str.data(), len);
 		}
 		ret += "'";
 	}
