@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2008-2018, Arvid Norberg
+Copyright (c) 2016-2020, Arvid Norberg
+Copyright (c) 2016-2018, Alden Torres
+Copyright (c) 2017, Falcosc
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -109,6 +111,9 @@ namespace libtorrent {
 
 		for (int i = 1; i < words + 1; ++i)
 		{
+#if defined __GNUC__ || defined __clang__
+			ret += __builtin_popcountl(m_buf[i]);
+#else
 			std::uint32_t const v = m_buf[i];
 			// from:
 			// http://graphics.stanford.edu/~seander/bithacks.html#CountBitsSetParallel
@@ -122,6 +127,7 @@ namespace libtorrent {
 			c = ((c >> S[4]) + c) & B[4];
 			ret += c;
 			TORRENT_ASSERT(ret <= size());
+#endif
 		}
 
 		TORRENT_ASSERT(ret <= size());
