@@ -1,6 +1,10 @@
 /*
 
-Copyright (c) 2014-2018, Arvid Norberg
+Copyright (c) 2019, Amir Abrams
+Copyright (c) 2014-2020, Arvid Norberg
+Copyright (c) 2016, Steven Siloti
+Copyright (c) 2016, 2018, Alden Torres
+Copyright (c) 2019, Monson Shao
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -197,35 +201,9 @@ namespace {
 		return c1 > c2;
 	}
 
-#if TORRENT_ABI_VERSION == 1
-	bool bittyrant_unchoke_compare(peer_connection const* lhs
-		, peer_connection const* rhs)
-	{
-		// first compare how many bytes they've sent us
-		std::int64_t d1 = lhs->downloaded_in_last_round();
-		std::int64_t d2 = rhs->downloaded_in_last_round();
-		// divided by the number of bytes we've sent them
-		std::int64_t const u1 = lhs->uploaded_in_last_round();
-		std::int64_t const u2 = rhs->uploaded_in_last_round();
-
-		// take torrent priority into account
-		d1 *= lhs->get_priority(peer_connection::upload_channel);
-		d2 *= rhs->get_priority(peer_connection::upload_channel);
-
-		d1 = d1 * 1000 / std::max(std::int64_t(1), u1);
-		d2 = d2 * 1000 / std::max(std::int64_t(1), u2);
-		if (d1 != d2) return d1 > d2;
-
-		// if both peers are still in their send quota or not in their send quota
-		// prioritize the one that has waited the longest to be unchoked
-		return lhs->time_of_last_unchoke() < rhs->time_of_last_unchoke();
-	}
-#endif
-
 	} // anonymous namespace
 
 	int unchoke_sort(std::vector<peer_connection*>& peers
-		, int const max_upload_rate
 		, time_duration const unchoke_interval
 		, aux::session_settings const& sett)
 	{
@@ -237,65 +215,6 @@ namespace {
 		}
 #endif
 
-#if TORRENT_ABI_VERSION == 1
-		// ==== BitTyrant ====
-		//
-		// if we're using the bittyrant unchoker, go through all peers that
-		// we have unchoked already, and adjust our estimated reciprocation
-		// rate. If the peer has reciprocated, lower the estimate, if it hasn't,
-		// increase the estimate (this attempts to optimize "ROI" of upload
-		// capacity, by sending just enough to be reciprocated).
-		// For more information, see: http://bittyrant.cs.washington.edu/
-		if (sett.get_int(settings_pack::choking_algorithm)
-			== settings_pack::bittyrant_choker)
-		{
-			for (auto const p : peers)
-			{
-				if (p->is_choked() || !p->is_interesting()) continue;
-
-				if (!p->has_peer_choked())
-				{
-					// we're unchoked, we may want to lower our estimated
-					// reciprocation rate
-					p->decrease_est_reciprocation_rate();
-				}
-				else
-				{
-					// we've unchoked this peer, and it hasn't reciprocated
-					// we may want to increase our estimated reciprocation rate
-					p->increase_est_reciprocation_rate();
-				}
-			}
-
-			// if we're using the bittyrant choker, sort peers by their return
-			// on investment. i.e. download rate / upload rate
-			// TODO: use an incremental partial_sort() here
-			std::sort(peers.begin(), peers.end()
-				, std::bind(&bittyrant_unchoke_compare, _1, _2));
-
-			int upload_capacity_left = max_upload_rate;
-
-			// now, figure out how many peers should be unchoked. We deduct the
-			// estimated reciprocation rate from our upload_capacity estimate
-			// until there none left
-			int upload_slots = 0;
-
-			for (auto const p : peers)
-			{
-				TORRENT_ASSERT(p != nullptr);
-
-				if (p->est_reciprocation_rate() > upload_capacity_left) break;
-
-				++upload_slots;
-				upload_capacity_left -= p->est_reciprocation_rate();
-			}
-
-			return upload_slots;
-		}
-#else
-		TORRENT_UNUSED(max_upload_rate);
-#endif
-
 		int upload_slots = sett.get_int(settings_pack::unchoke_slots_limit);
 		if (upload_slots < 0)
 			upload_slots = std::numeric_limits<int>::max();
@@ -320,7 +239,8 @@ namespace {
 			int rate_threshold = sett.get_int(settings_pack::rate_choker_initial_threshold);
 
 			std::sort(peers.begin(), peers.end()
-				, std::bind(&upload_rate_compare, _1, _2));
+				, [](peer_connection const* lhs, peer_connection const* rhs)
+				{ return upload_rate_compare(lhs, rhs); });
 
 			for (auto const* p : peers)
 			{
@@ -355,28 +275,32 @@ namespace {
 
 			std::nth_element(peers.begin(), peers.begin()
 				+ slots, peers.end()
-				, std::bind(&unchoke_compare_rr, _1, _2, pieces));
+				, [pieces](peer_connection const* lhs, peer_connection const* rhs)
+				{ return unchoke_compare_rr(lhs, rhs, pieces); });
 		}
 		else if (sett.get_int(settings_pack::seed_choking_algorithm)
 			== settings_pack::fastest_upload)
 		{
 			std::nth_element(peers.begin(), peers.begin()
 				+ slots, peers.end()
-				, std::bind(&unchoke_compare_fastest_upload, _1, _2));
+				, [](peer_connection const* lhs, peer_connection const* rhs)
+				{ return unchoke_compare_fastest_upload(lhs, rhs); });
 		}
 		else if (sett.get_int(settings_pack::seed_choking_algorithm)
 			== settings_pack::anti_leech)
 		{
 			std::nth_element(peers.begin(), peers.begin()
 				+ slots, peers.end()
-				, std::bind(&unchoke_compare_anti_leech, _1, _2));
+				, [](peer_connection const* lhs, peer_connection const* rhs)
+				{ return unchoke_compare_anti_leech(lhs, rhs); });
 		}
 		else
 		{
 			int const pieces = sett.get_int(settings_pack::seeding_piece_quota);
 			std::nth_element(peers.begin(), peers.begin()
 				+ slots, peers.end()
-				, std::bind(&unchoke_compare_rr, _1, _2, pieces));
+				, [pieces](peer_connection const* lhs, peer_connection const* rhs)
+				{ return unchoke_compare_rr(lhs, rhs, pieces); } );
 
 			TORRENT_ASSERT_FAIL();
 		}
