@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2008-2018, Arvid Norberg
+Copyright (c) 2008, 2014, 2016-2019, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -31,14 +31,22 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/disk_buffer_holder.hpp"
+#include <utility>
 
 namespace libtorrent {
 
 	disk_buffer_holder::disk_buffer_holder(buffer_allocator_interface& alloc
-		, char* buf, std::size_t sz) noexcept
-		: m_allocator(&alloc), m_buf(buf), m_size(sz), m_ref()
+		, char* const buf, int const sz) noexcept
+		: m_allocator(&alloc), m_buf(buf), m_size(sz)
 	{}
 
+	disk_buffer_holder::disk_buffer_holder(disk_buffer_holder&& h) noexcept
+		: m_allocator(h.m_allocator), m_buf(h.m_buf), m_size(h.m_size)
+	{
+		h.m_buf = nullptr;
+		h.m_size = 0;
+	}
+
 	disk_buffer_holder& disk_buffer_holder::operator=(disk_buffer_holder&& h) & noexcept
 	{
 		if (&h == this) return *this;
@@ -46,46 +54,11 @@ namespace libtorrent {
 		return *this;
 	}
 
-	disk_buffer_holder::disk_buffer_holder(disk_buffer_holder&& h) noexcept
-		: m_allocator(h.m_allocator), m_buf(h.m_buf), m_size(h.m_size), m_ref(h.m_ref)
+	void disk_buffer_holder::reset()
 	{
-		// we own this buffer now
-		h.m_buf = nullptr;
-		h.m_ref = aux::block_cache_reference();
-	}
-
-	disk_buffer_holder::disk_buffer_holder(buffer_allocator_interface& alloc
-		, aux::block_cache_reference const& ref, char* buf
-		, std::size_t sz) noexcept
-		: m_allocator(&alloc), m_buf(buf), m_size(sz), m_ref(ref)
-	{}
-
-	void disk_buffer_holder::reset(aux::block_cache_reference const& ref, char* buf, std::size_t const sz)
-	{
-		if (m_ref.cookie != aux::block_cache_reference::none) m_allocator->reclaim_blocks(m_ref);
-		else if (m_buf) m_allocator->free_disk_buffer(m_buf);
-		m_buf = buf;
-		m_size = sz;
-		m_ref = ref;
-	}
-
-	void disk_buffer_holder::reset(char* const buf, std::size_t const sz)
-	{
-		if (m_ref.cookie != aux::block_cache_reference::none) m_allocator->reclaim_blocks(m_ref);
-		else if (m_buf) m_allocator->free_disk_buffer(m_buf);
-		m_buf = buf;
-		m_size = sz;
-		m_ref = aux::block_cache_reference();
-	}
-
-	char* disk_buffer_holder::release() noexcept
-	{
-		TORRENT_ASSERT(m_ref.cookie == aux::block_cache_reference::none);
-		char* ret = m_buf;
+		if (m_buf) m_allocator->free_disk_buffer(m_buf);
 		m_buf = nullptr;
 		m_size = 0;
-		m_ref = aux::block_cache_reference();
-		return ret;
 	}
 
 	disk_buffer_holder::~disk_buffer_holder() { reset(); }
