@@ -1,6 +1,9 @@
 /*
 
-Copyright (c) 2008-2018, Arvid Norberg
+Copyright (c) 2008-2020, Arvid Norberg
+Copyright (c) 2016-2017, 2019, Steven Siloti
+Copyright (c) 2017, Pavel Pimenov
+Copyright (c) 2019, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -43,7 +46,7 @@ namespace libtorrent {
 		const char* name() const BOOST_SYSTEM_NOEXCEPT override;
 		std::string message(int ev) const override;
 		boost::system::error_condition default_error_condition(int ev) const BOOST_SYSTEM_NOEXCEPT override
-		{ return boost::system::error_condition(ev, *this); }
+		{ return {ev, *this}; }
 	};
 
 	const char* libtorrent_error_category::name() const BOOST_SYSTEM_NOEXCEPT
@@ -172,9 +175,9 @@ namespace libtorrent {
 			"banned by port filter",
 			"invalid session handle used",
 			"listen socket has been closed",
-			"blocked by SSRF mitigation",
-			"blocked by IDNA ban",
-			"",
+			"invalid hash request",
+			"invalid hashes",
+			"invalid hash reject",
 
 // natpmp errors
 			"unsupported protocol version",
@@ -203,7 +206,7 @@ namespace libtorrent {
 			"invalid piece index in slot list",
 			"pieces needs to be reordered",
 			"fastresume not modified since last save",
-			"invalid save_path",
+			"",
 			"",
 			"",
 			"",
@@ -273,6 +276,25 @@ namespace libtorrent {
 			"", "", "", "", "",
 #endif
 			"random number generator failed",
+			"blocked by SSRF mitigation",
+			"blocked by IDNA ban",
+			"",
+			"",
+			"",
+			"",
+			"",
+			"",
+			"",
+
+			"the torrent file has an unknown meta version",
+			"the v2 torrent file has no file tree",
+			"the torrent contains v2 keys but does not specify meta version 2",
+			"the v1 and v2 file metadata does not match",
+			"one or more files are missing piece layer hashes",
+			"a piece layer is invalid",
+			"a v2 file entry has no root hash",
+			"v1 and v2 hashes do not describe the same data",
+			"a file in the v2 metadata has the pad attribute set"
 		};
 		if (ev < 0 || ev >= int(sizeof(msgs)/sizeof(msgs[0])))
 			return "Unknown error";
@@ -319,7 +341,7 @@ namespace libtorrent {
 		}
 		boost::system::error_condition default_error_condition(
 			int ev) const BOOST_SYSTEM_NOEXCEPT override
-		{ return boost::system::error_condition(ev, *this); }
+		{ return {ev, *this}; }
 	};
 
 	boost::system::error_category& http_category()
@@ -333,7 +355,7 @@ namespace libtorrent {
 		// hidden
 		boost::system::error_code make_error_code(error_code_enum e)
 		{
-			return boost::system::error_code(e, libtorrent_category());
+			return {e, libtorrent_category()};
 		}
 	}
 
