@@ -1,6 +1,13 @@
 /*
 
-Copyright (c) 2007-2018, Arvid Norberg
+Copyright (c) 2007-2020, Arvid Norberg
+Copyright (c) 2015, Mikhail Titov
+Copyright (c) 2016-2018, 2020, Alden Torres
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2017, Jan Berkel
+Copyright (c) 2017, Steven Siloti
+Copyright (c) 2019, patch3proxyheaders915360
+Copyright (c) 2020, Paul-Louis Ageneau
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,54 +44,51 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/parse_url.hpp"
 #include "libtorrent/socket.hpp"
 #include "libtorrent/aux_/socket_type.hpp" // for async_shutdown
-#include "libtorrent/aux_/bind_to_device.hpp"
-#include "libtorrent/resolver_interface.hpp"
+#include "libtorrent/aux_/resolver_interface.hpp"
 #include "libtorrent/settings_pack.hpp"
 #include "libtorrent/aux_/time.hpp"
 #include "libtorrent/random.hpp"
 #include "libtorrent/debug.hpp"
 #include "libtorrent/time.hpp"
+#include "libtorrent/io_context.hpp"
+#include "libtorrent/i2p_stream.hpp"
+#include "libtorrent/aux_/ip_helpers.hpp"
+#include "libtorrent/ssl.hpp"
 
 #include <functional>
 #include <string>
 #include <algorithm>
 #include <sstream>
 
-#ifdef TORRENT_USE_OPENSSL
-#include "libtorrent/aux_/disable_warnings_push.hpp"
-#include <boost/asio/ssl/context.hpp>
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
-#endif
-
 using namespace std::placeholders;
 
 namespace libtorrent {
 
-http_connection::http_connection(io_service& ios
-	, resolver_interface& resolver
-	, http_handler const& handler
+http_connection::http_connection(io_context& ios
+	, aux::resolver_interface& resolver
+	, http_handler handler
 	, bool bottled
 	, int max_bottled_buffer_size
-	, http_connect_handler const& ch
-	, http_filter_handler const& fh
-	, hostname_filter_handler const& hfh
-#ifdef TORRENT_USE_OPENSSL
+	, http_connect_handler ch
+	, http_filter_handler fh
+	, hostname_filter_handler hfh
+#if TORRENT_USE_SSL
 	, ssl::context* ssl_ctx
 #endif
 	)
-	: m_next_ep(0)
-	, m_sock(ios)
-#ifdef TORRENT_USE_OPENSSL
+	: m_ios(ios)
+	, m_next_ep(0)
+#if TORRENT_USE_SSL
 	, m_ssl_ctx(ssl_ctx)
 #endif
 #if TORRENT_USE_I2P
 	, m_i2p_conn(nullptr)
 #endif
 	, m_resolver(resolver)
-	, m_handler(handler)
-	, m_connect_handler(ch)
-	, m_filter_handler(fh)
-	, m_hostname_filter_handler(hfh)
+	, m_handler(std::move(handler))
+	, m_connect_handler(std::move(ch))
+	, m_filter_handler(std::move(fh))
+	, m_hostname_filter_handler(std::move(hfh))
 	, m_timer(ios)
 	, m_completion_timeout(seconds(5))
 	, m_limiter_timer(ios)
@@ -95,6 +99,7 @@ http_connection::http_connection(io_service& ios
 	, m_max_bottled_buffer_size(max_bottled_buffer_size)
 	, m_rate_limit(0)
 	, m_download_quota(0)
+	, m_priority(0)
 	, m_resolve_flags{}
 	, m_port(0)
 	, m_bottled(bottled)
@@ -104,10 +109,9 @@ http_connection::http_connection(io_service& ios
 
 http_connection::~http_connection() = default;
 
-void http_connection::get(std::string const& url, time_duration timeout
+void http_connection::get(std::string const& url, time_duration timeout, int prio
 	, aux::proxy_settings const* ps, int handle_redirects, std::string const& user_agent
-	, boost::optional<bind_info_t> const& bind_addr, resolver_flags const resolve_flags
-	, std::string const& auth_
+	, boost::optional<address> const& bind_addr, aux::resolver_flags const resolve_flags, std::string const& auth_
 #if TORRENT_USE_I2P
 	, i2p_connection* i2p_conn
 #endif
@@ -139,31 +143,33 @@ void http_connection::get(std::string const& url, time_duration timeout
 
 	if (ec)
 	{
-		lt::get_io_service(m_timer).post(std::bind(&http_connection::callback
+		post(m_ios, std::bind(&http_connection::callback
 			, me, ec, span<char>{}));
 		return;
 	}
 
 	if (m_hostname_filter_handler && !m_hostname_filter_handler(*this, hostname))
 	{
-		ec.assign(errors::blocked_by_idna, libtorrent_category());
-		lt::get_io_service(m_timer).post(std::bind(&http_connection::callback
-			, me, ec, span<char>{}));
+		error_code err(errors::blocked_by_idna);
+		post(m_ios, std::bind(&http_connection::callback
+			, me, err, span<char>{}));
 		return;
 	}
 
 	if (protocol != "http"
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 		&& protocol != "https"
 #endif
 		)
 	{
 		error_code err(errors::unsupported_url_protocol);
-		lt::get_io_service(m_timer).post(std::bind(&http_connection::callback
+		post(m_ios, std::bind(&http_connection::callback
 			, me, err, span<char>{}));
 		return;
 	}
 
+	TORRENT_ASSERT(prio >= 0 && prio < 3);
+
 	bool const ssl = (protocol == "https");
 
 	std::stringstream request;
@@ -210,7 +216,7 @@ void http_connection::get(std::string const& url, time_duration timeout
 
 	m_sendbuffer.assign(request.str());
 	m_url = url;
-	start(hostname, port, timeout
+	start(hostname, port, timeout, prio
 		, ps, ssl, handle_redirects, bind_addr, m_resolve_flags
 #if TORRENT_USE_I2P
 		, i2p_conn
@@ -219,15 +225,17 @@ void http_connection::get(std::string const& url, time_duration timeout
 }
 
 void http_connection::start(std::string const& hostname, int port
-	, time_duration timeout, aux::proxy_settings const* ps, bool ssl
+	, time_duration timeout, int prio, aux::proxy_settings const* ps, bool ssl
 	, int handle_redirects
-	, boost::optional<bind_info_t> const& bind_addr
-	, resolver_flags const resolve_flags
+	, boost::optional<address> const& bind_addr
+	, aux::resolver_flags const resolve_flags
 #if TORRENT_USE_I2P
 	, i2p_connection* i2p_conn
 #endif
 	)
 {
+	TORRENT_ASSERT(prio >= 0 && prio < 3);
+
 	m_redirects = handle_redirects;
 	m_resolve_flags = resolve_flags;
 	if (ps) m_proxy = *ps;
@@ -237,8 +245,7 @@ void http_connection::start(std::string const& hostname, int port
 	std::shared_ptr<http_connection> me(shared_from_this());
 
 	m_completion_timeout = timeout;
-	error_code ec;
-	m_timer.expires_from_now(m_completion_timeout, ec);
+	m_timer.expires_after(m_completion_timeout);
 	ADD_OUTSTANDING_ASYNC("http_connection::on_timeout");
 	m_timer.async_wait(std::bind(&http_connection::on_timeout
 		, std::weak_ptr<http_connection>(me), _1));
@@ -246,23 +253,17 @@ void http_connection::start(std::string const& hostname, int port
 	m_parser.reset();
 	m_recvbuffer.clear();
 	m_read_pos = 0;
+	m_priority = prio;
 
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 	TORRENT_ASSERT(!ssl || m_ssl_ctx != nullptr);
 #endif
 
-	if (ec)
-	{
-		lt::get_io_service(m_timer).post(std::bind(&http_connection::callback
-			, me, ec, span<char>{}));
-		return;
-	}
-
-	if (m_sock.is_open() && m_hostname == hostname && m_port == port
+	if (m_sock && m_sock->is_open() && m_hostname == hostname && m_port == port
 		&& m_ssl == ssl && m_bind_addr == bind_addr)
 	{
 		ADD_OUTSTANDING_ASYNC("http_connection::on_write");
-		async_write(m_sock, boost::asio::buffer(m_sendbuffer)
+		async_write(*m_sock, boost::asio::buffer(m_sendbuffer)
 			, std::bind(&http_connection::on_write, me, _1));
 	}
 	else
@@ -270,7 +271,7 @@ void http_connection::start(std::string const& hostname, int port
 		m_ssl = ssl;
 		m_bind_addr = bind_addr;
 		error_code err;
-		if (m_sock.is_open()) m_sock.close(err);
+		if (m_sock && m_sock->is_open()) m_sock->close(err);
 
 		aux::proxy_settings const* proxy = ps;
 
@@ -291,7 +292,7 @@ void http_connection::start(std::string const& hostname, int port
 #if TORRENT_USE_I2P
 			if (i2p_conn->proxy().type != settings_pack::i2p_proxy)
 			{
-				lt::get_io_service(m_timer).post(std::bind(&http_connection::callback
+				post(m_ios, std::bind(&http_connection::callback
 					, me, error_code(errors::no_i2p_router), span<char>{}));
 				return;
 			}
@@ -314,7 +315,7 @@ void http_connection::start(std::string const& hostname, int port
 		aux::proxy_settings null_proxy;
 
 		void* userdata = nullptr;
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 		if (m_ssl)
 		{
 			TORRENT_ASSERT(m_ssl_ctx != nullptr);
@@ -324,29 +325,27 @@ void http_connection::start(std::string const& hostname, int port
 		// assume this is not a tracker connection. Tracker connections that
 		// shouldn't be subject to the proxy should pass in nullptr as the proxy
 		// pointer.
-		instantiate_connection(lt::get_io_service(m_timer)
-			, proxy ? *proxy : null_proxy, m_sock, userdata, nullptr, false, false);
+		m_sock.emplace(instantiate_connection(m_ios
+			, proxy ? *proxy : null_proxy, userdata, nullptr, false, false));
 
 		if (m_bind_addr)
 		{
-			m_sock.open(m_bind_addr->ip.is_v4() ? tcp::v4() : tcp::v6(), ec);
-#if TORRENT_HAS_BINDTODEVICE
-			error_code ignore;
-			bind_device(m_sock, m_bind_addr->device.c_str(), ignore);
-#endif
-			m_sock.bind(tcp::endpoint(m_bind_addr->ip, 0), ec);
+			error_code ec;
+			m_sock->open(m_bind_addr->is_v4() ? tcp::v4() : tcp::v6(), ec);
+			m_sock->bind(tcp::endpoint(*m_bind_addr, 0), ec);
 			if (ec)
 			{
-				lt::get_io_service(m_timer).post(std::bind(&http_connection::callback
+				post(m_ios, std::bind(&http_connection::callback
 					, me, ec, span<char>{}));
 				return;
 			}
 		}
 
-		setup_ssl_hostname(m_sock, hostname, ec);
+		error_code ec;
+		setup_ssl_hostname(*m_sock, hostname, ec);
 		if (ec)
 		{
-			lt::get_io_service(m_timer).post(std::bind(&http_connection::callback
+			post(m_ios, std::bind(&http_connection::callback
 				, me, ec, span<char>{}));
 			return;
 		}
@@ -404,17 +403,7 @@ void http_connection::on_timeout(std::weak_ptr<http_connection> p
 
 	// be forgiving of timeout while we're still resolving the hostname
 	// it may be delayed because we're queued up behind another slow lookup
-	if (c->m_resolving_host
-		&& (c->m_start_time + (c->m_completion_timeout * 2) > now))
-	{
-		ADD_OUTSTANDING_ASYNC("http_connection::on_timeout");
-		error_code ec;
-		c->m_timer.expires_at(c->m_start_time + c->m_completion_timeout * 2, ec);
-		c->m_timer.async_wait(std::bind(&http_connection::on_timeout, p, _1));
-		return;
-	}
-
-	if (c->m_start_time + c->m_completion_timeout <= now)
+	if (c->m_start_time + (c->m_completion_timeout * (int(c->m_resolving_host) + 1)) <= now)
 	{
 		// the connection timed out. If we have more endpoints to try, just
 		// close this connection. The on_connect handler will try the next
@@ -422,7 +411,7 @@ void http_connection::on_timeout(std::weak_ptr<http_connection> p
 		if (c->m_next_ep < int(c->m_endpoints.size()))
 		{
 			error_code ec;
-			c->m_sock.close(ec);
+			c->m_sock->close(ec);
 			if (!c->m_connecting) c->connect();
 			c->m_last_receive = now;
 			c->m_start_time = c->m_last_receive;
@@ -432,15 +421,14 @@ void http_connection::on_timeout(std::weak_ptr<http_connection> p
 			// the socket may have an outstanding operation, that keeps the
 			// http_connection object alive. We want to cancel all that.
 			error_code ec;
-			c->m_sock.close(ec);
+			c->m_sock->close(ec);
 			c->callback(lt::errors::timed_out);
 			return;
 		}
 	}
 
 	ADD_OUTSTANDING_ASYNC("http_connection::on_timeout");
-	error_code ec;
-	c->m_timer.expires_at(c->m_start_time + c->m_completion_timeout, ec);
+	c->m_timer.expires_at(c->m_start_time + c->m_completion_timeout);
 	c->m_timer.async_wait(std::bind(&http_connection::on_timeout, p, _1));
 }
 
@@ -448,19 +436,23 @@ void http_connection::close(bool force)
 {
 	if (m_abort) return;
 
-	error_code ec;
-	if (force)
+	if (m_sock)
 	{
-		m_sock.close(ec);
-		m_timer.cancel(ec);
+		error_code ec;
+		if (force)
+		{
+			m_sock->close(ec);
+			m_timer.cancel();
+		}
+		else
+		{
+			async_shutdown(*m_sock, shared_from_this());
+		}
 	}
 	else
-	{
-		async_shutdown(m_sock, shared_from_this());
-		m_timer.cancel(ec);
-	}
+		m_timer.cancel();
 
-	m_limiter_timer.cancel(ec);
+	m_limiter_timer.cancel();
 
 	m_hostname.clear();
 	m_port = 0;
@@ -471,17 +463,17 @@ void http_connection::close(bool force)
 #if TORRENT_USE_I2P
 void http_connection::connect_i2p_tracker(char const* destination)
 {
-	TORRENT_ASSERT(m_sock.get<i2p_stream>());
-#ifdef TORRENT_USE_OPENSSL
+	TORRENT_ASSERT(boost::get<i2p_stream>(m_sock.get_ptr()));
+#if TORRENT_USE_SSL
 	TORRENT_ASSERT(m_ssl == false);
 #endif
-	m_sock.get<i2p_stream>()->set_destination(destination);
-	m_sock.get<i2p_stream>()->set_command(i2p_stream::cmd_connect);
-	m_sock.get<i2p_stream>()->set_session_id(m_i2p_conn->session_id());
+	boost::get<i2p_stream>(*m_sock).set_destination(destination);
+	boost::get<i2p_stream>(*m_sock).set_command(i2p_stream::cmd_connect);
+	boost::get<i2p_stream>(*m_sock).set_session_id(m_i2p_conn->session_id());
 	ADD_OUTSTANDING_ASYNC("http_connection::on_connect");
 	TORRENT_ASSERT(!m_connecting);
 	m_connecting = true;
-	m_sock.async_connect(tcp::endpoint(), std::bind(&http_connection::on_connect
+	m_sock->async_connect(tcp::endpoint(), std::bind(&http_connection::on_connect
 		, shared_from_this(), _1));
 }
 
@@ -529,7 +521,7 @@ void http_connection::on_resolve(error_code const& e
 	if (m_bind_addr)
 	{
 		auto const new_end = std::remove_if(m_endpoints.begin(), m_endpoints.end()
-			, [&](tcp::endpoint const& ep) { return is_v4(ep) != m_bind_addr->ip.is_v4(); });
+			, [&](tcp::endpoint const& ep) { return aux::is_v4(ep) != m_bind_addr->is_v4(); });
 
 		m_endpoints.erase(new_end, m_endpoints.end());
 		if (m_endpoints.empty())
@@ -563,17 +555,17 @@ void http_connection::connect()
 		{
 			// we're using a socks proxy and we're resolving
 			// hostnames through it
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 			if (m_ssl)
 			{
-				TORRENT_ASSERT(m_sock.get<ssl_stream<socks5_stream>>());
-				m_sock.get<ssl_stream<socks5_stream>>()->next_layer().set_dst_name(m_hostname);
+				TORRENT_ASSERT(boost::get<ssl_stream<socks5_stream>>(m_sock.get_ptr()));
+				boost::get<ssl_stream<socks5_stream>>(*m_sock).next_layer().set_dst_name(m_hostname);
 			}
 			else
 #endif
 			{
-				TORRENT_ASSERT(m_sock.get<socks5_stream>());
-				m_sock.get<socks5_stream>()->set_dst_name(m_hostname);
+				TORRENT_ASSERT(boost::get<socks5_stream>(m_sock.get_ptr()));
+				boost::get<socks5_stream>(*m_sock).set_dst_name(m_hostname);
 			}
 		}
 		else
@@ -591,7 +583,7 @@ void http_connection::connect()
 	ADD_OUTSTANDING_ASYNC("http_connection::on_connect");
 	TORRENT_ASSERT(!m_connecting);
 	m_connecting = true;
-	m_sock.async_connect(target_address, std::bind(&http_connection::on_connect
+	m_sock->async_connect(target_address, std::bind(&http_connection::on_connect
 		, me, _1));
 }
 
@@ -607,20 +599,20 @@ void http_connection::on_connect(error_code const& e)
 	{
 		if (m_connect_handler) m_connect_handler(*this);
 		ADD_OUTSTANDING_ASYNC("http_connection::on_write");
-		async_write(m_sock, boost::asio::buffer(m_sendbuffer)
+		async_write(*m_sock, boost::asio::buffer(m_sendbuffer)
 			, std::bind(&http_connection::on_write, shared_from_this(), _1));
 	}
 	else if (m_next_ep < int(m_endpoints.size()) && !m_abort)
 	{
 		// The connection failed. Try the next endpoint in the list.
 		error_code ec;
-		m_sock.close(ec);
+		m_sock->close(ec);
 		connect();
 	}
 	else
 	{
 		error_code ec;
-		m_sock.close(ec);
+		m_sock->close(ec);
 		callback(e);
 	}
 }
@@ -654,8 +646,7 @@ void http_connection::callback(error_code e, span<char> data)
 		if (m_parser.finished()) e.clear();
 	}
 	m_called = true;
-	error_code ec;
-	m_timer.cancel(ec);
+	m_timer.cancel();
 	if (m_handler) m_handler(e, m_parser, data, *this);
 }
 
@@ -691,7 +682,7 @@ void http_connection::on_write(error_code const& e)
 		}
 	}
 	ADD_OUTSTANDING_ASYNC("http_connection::on_read");
-	m_sock.async_read_some(boost::asio::buffer(m_recvbuffer.data() + m_read_pos
+	m_sock->async_read_some(boost::asio::buffer(m_recvbuffer.data() + m_read_pos
 		, std::size_t(amount_to_read))
 		, std::bind(&http_connection::on_read
 			, shared_from_this(), _1, _2));
@@ -776,10 +767,10 @@ void http_connection::on_read(error_code const& e
 				// but then we'd have to do all the reconnect logic
 				// in its handler. For now, just kill the connection.
 //				async_shutdown(m_sock, me);
-				m_sock.close(ec);
+				m_sock->close(ec);
 
 				std::string url = resolve_redirect_location(m_url, location);
-				get(url, m_completion_timeout, &m_proxy, m_redirects - 1
+				get(url, m_completion_timeout, m_priority, &m_proxy, m_redirects - 1
 					, m_user_agent, m_bind_addr, m_resolve_flags, m_auth
 #if TORRENT_USE_I2P
 					, m_i2p_conn
@@ -804,8 +795,7 @@ void http_connection::on_read(error_code const& e
 		}
 		else if (m_bottled && m_parser.finished())
 		{
-			error_code ec;
-			m_timer.cancel(ec);
+			m_timer.cancel();
 			callback(e, span<char>(m_recvbuffer)
 				.first(m_read_pos)
 				.subspan(m_parser.body_start()));
@@ -845,7 +835,7 @@ void http_connection::on_read(error_code const& e
 		}
 	}
 	ADD_OUTSTANDING_ASYNC("http_connection::on_read");
-	m_sock.async_read_some(boost::asio::buffer(m_recvbuffer.data() + m_read_pos
+	m_sock->async_read_some(boost::asio::buffer(m_recvbuffer.data() + m_read_pos
 		, std::size_t(amount_to_read))
 		, std::bind(&http_connection::on_read
 			, me, _1, _2));
@@ -856,7 +846,7 @@ void http_connection::on_assign_bandwidth(error_code const& e)
 	COMPLETE_ASYNC("http_connection::on_assign_bandwidth");
 	if ((e == boost::asio::error::operation_aborted
 		&& m_limiter_timer_active)
-		|| !m_sock.is_open())
+		|| !m_sock->is_open())
 	{
 		callback(boost::asio::error::eof);
 		return;
@@ -874,17 +864,16 @@ void http_connection::on_assign_bandwidth(error_code const& e)
 	if (amount_to_read > m_download_quota)
 		amount_to_read = m_download_quota;
 
-	if (!m_sock.is_open()) return;
+	if (!m_sock->is_open()) return;
 
 	ADD_OUTSTANDING_ASYNC("http_connection::on_read");
-	m_sock.async_read_some(boost::asio::buffer(m_recvbuffer.data() + m_read_pos
+	m_sock->async_read_some(boost::asio::buffer(m_recvbuffer.data() + m_read_pos
 		, std::size_t(amount_to_read))
 		, std::bind(&http_connection::on_read
 			, shared_from_this(), _1, _2));
 
-	error_code ec;
 	m_limiter_timer_active = true;
-	m_limiter_timer.expires_from_now(milliseconds(250), ec);
+	m_limiter_timer.expires_after(milliseconds(250));
 	ADD_OUTSTANDING_ASYNC("http_connection::on_assign_bandwidth");
 	m_limiter_timer.async_wait(std::bind(&http_connection::on_assign_bandwidth
 		, shared_from_this(), _1));
@@ -892,13 +881,12 @@ void http_connection::on_assign_bandwidth(error_code const& e)
 
 void http_connection::rate_limit(int limit)
 {
-	if (!m_sock.is_open()) return;
+	if (!m_sock->is_open()) return;
 
 	if (!m_limiter_timer_active)
 	{
-		error_code ec;
 		m_limiter_timer_active = true;
-		m_limiter_timer.expires_from_now(milliseconds(250), ec);
+		m_limiter_timer.expires_after(milliseconds(250));
 		ADD_OUTSTANDING_ASYNC("http_connection::on_assign_bandwidth");
 		m_limiter_timer.async_wait(std::bind(&http_connection::on_assign_bandwidth
 			, shared_from_this(), _1));
