@@ -1,6 +1,11 @@
 /*
 
-Copyright (c) 2006-2018, Arvid Norberg
+Copyright (c) 2006-2017, 2019-2020, Arvid Norberg
+Copyright (c) 2015, Thomas
+Copyright (c) 2015, 2017, Steven Siloti
+Copyright (c) 2016-2018, Alden Torres
+Copyright (c) 2016-2017, Andrei Kurushin
+Copyright (c) 2017, Pavel Pimenov
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,7 +38,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <libtorrent/config.hpp>
 #include <libtorrent/io.hpp>
 #include <libtorrent/random.hpp>
-#include <libtorrent/invariant_check.hpp>
+#include <libtorrent/aux_/invariant_check.hpp>
 #include <libtorrent/kademlia/rpc_manager.hpp>
 #include <libtorrent/kademlia/routing_table.hpp>
 #include <libtorrent/kademlia/find_data.hpp>
@@ -44,12 +49,12 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <libtorrent/kademlia/direct_request.hpp>
 #include <libtorrent/kademlia/get_item.hpp>
 #include <libtorrent/kademlia/sample_infohashes.hpp>
-#include <libtorrent/kademlia/dht_settings.hpp>
+#include <libtorrent/aux_/session_settings.hpp>
 
 #include <libtorrent/socket_io.hpp> // for print_endpoint
 #include <libtorrent/aux_/time.hpp> // for aux::time_now
 #include <libtorrent/aux_/aligned_union.hpp>
-#include <libtorrent/broadcast_socket.hpp> // for is_v6
+#include <libtorrent/aux_/ip_helpers.hpp> // for is_v6
 
 #include <type_traits>
 #include <functional>
@@ -83,7 +88,7 @@ void observer::set_target(udp::endpoint const& ep)
 	m_sent = clock_type::now();
 
 	m_port = ep.port();
-	if (is_v6(ep))
+	if (aux::is_v6(ep))
 	{
 		flags |= flag_ipv6_address;
 		m_addr.v6 = ep.address().to_v6().to_bytes();
@@ -105,7 +110,7 @@ address observer::target_addr() const
 
 udp::endpoint observer::target_ep() const
 {
-	return udp::endpoint(target_addr(), m_port);
+	return {target_addr(), m_port};
 }
 
 void observer::abort()
@@ -155,13 +160,13 @@ using observer_storage = aux::aligned_union<1
 	, traversal_observer>::type;
 
 rpc_manager::rpc_manager(node_id const& our_id
-	, dht_settings const& settings
+	, aux::session_settings const& settings
 	, routing_table& table
-	, aux::listen_socket_handle const& sock
+	, aux::listen_socket_handle sock
 	, socket_manager* sock_man
 	, dht_logger* log)
 	: m_pool_allocator(sizeof(observer_storage), 10)
-	, m_sock(sock)
+	, m_sock(std::move(sock))
 	, m_sock_man(sock_man)
 #ifndef TORRENT_DISABLE_LOGGING
 	, m_log(log)
@@ -261,7 +266,7 @@ bool rpc_manager::incoming(msg const& m, node_id* id)
 	if (transaction_id.empty()) return false;
 
 	auto ptr = transaction_id.begin();
-	std::uint16_t tid = transaction_id.size() != 2 ? std::uint64_t(0xffff) : detail::read_uint16(ptr);
+	std::uint16_t const tid = transaction_id.size() != 2 ? std::uint64_t(0xffff) : aux::read_uint16(ptr);
 
 	observer_ptr o;
 	auto range = m_transactions.equal_range(tid);
@@ -356,7 +361,7 @@ bool rpc_manager::incoming(msg const& m, node_id* id)
 	}
 
 	node_id const nid = node_id(node_id_ent.string_ptr());
-	if (m_settings.enforce_node_id && !verify_id(nid, m.addr.address()))
+	if (m_settings.get_bool(settings_pack::dht_enforce_node_id) && !verify_id(nid, m.addr.address()))
 	{
 		o->timeout();
 		return false;
@@ -384,17 +389,17 @@ time_duration rpc_manager::tick()
 {
 	INVARIANT_CHECK;
 
-	constexpr int short_timeout = 1;
-	constexpr int timeout = 15;
+	constexpr auto short_timeout = seconds(1);
+	constexpr auto timeout = seconds(15);
 
 	// look for observers that have timed out
 
-	if (m_transactions.empty()) return seconds(short_timeout);
+	if (m_transactions.empty()) return short_timeout;
 
 	std::vector<observer_ptr> timeouts;
 	std::vector<observer_ptr> short_timeouts;
 
-	time_duration ret = seconds(short_timeout);
+	time_duration ret = short_timeout;
 	time_point now = aux::time_now();
 
 	for (auto i = m_transactions.begin(); i != m_transactions.end();)
@@ -402,7 +407,7 @@ time_duration rpc_manager::tick()
 		observer_ptr o = i->second;
 
 		time_duration diff = now - o->sent();
-		if (diff >= seconds(timeout))
+		if (diff >= timeout)
 		{
 #ifndef TORRENT_DISABLE_LOGGING
 			if (m_log->should_log(dht_logger::rpc_manager))
@@ -419,7 +424,7 @@ time_duration rpc_manager::tick()
 
 		// don't call short_timeout() again if we've
 		// already called it once
-		if (diff >= seconds(short_timeout) && !o->has_short_timeout())
+		if (diff >= short_timeout && !o->has_short_timeout())
 		{
 #ifndef TORRENT_DISABLE_LOGGING
 			if (m_log->should_log(dht_logger::rpc_manager))
@@ -435,7 +440,7 @@ time_duration rpc_manager::tick()
 			continue;
 		}
 
-		ret = std::min(seconds(timeout) - diff, ret);
+		ret = std::min(duration_cast<time_duration>(timeout - diff), ret);
 		++i;
 	}
 
@@ -465,12 +470,12 @@ bool rpc_manager::invoke(entry& e, udp::endpoint const& target_addr
 	transaction_id.resize(2);
 	char* out = &transaction_id[0];
 	std::uint16_t const tid = std::uint16_t(random(0xffff));
-	detail::write_uint16(tid, out);
+	aux::write_uint16(tid, out);
 	e["t"] = transaction_id;
 
 	// When a DHT node enters the read-only state, in each outgoing query message,
 	// places a 'ro' key in the top-level message dictionary and sets its value to 1.
-	if (m_settings.read_only) e["ro"] = 1;
+	if (m_settings.get_bool(settings_pack::dht_read_only)) e["ro"] = 1;
 
 	node& n = o->algorithm()->get_node();
 	if (!n.native_address(o->target_addr()))
