@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2007-2018, Arvid Norberg
+Copyright (c) 2007-2012, 2014-2020, Arvid Norberg
+Copyright (c) 2016, Pavel Pimenov
+Copyright (c) 2016-2017, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -44,7 +46,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/hex.hpp" // to_hex, from_hex
 #include "libtorrent/aux_/numeric_cast.hpp"
 #include "libtorrent/enum_net.hpp"
-#include "libtorrent/aux_/scope_end.hpp"
 
 #include "libtorrent/aux_/disable_warnings_push.hpp"
 #include <boost/asio/ip/multicast.hpp>
@@ -70,8 +71,8 @@ int render_lsd_packet(char* dst, int const len, int const listen_port
 }
 } // anonymous namespace
 
-lsd::lsd(io_service& ios, aux::lsd_callback& cb
-	, address const& listen_address, address const& netmask)
+lsd::lsd(io_context& ios, aux::lsd_callback& cb
+	, address const listen_address, address const netmask)
 	: m_callback(cb)
 	, m_listen_address(listen_address)
 	, m_netmask(netmask)
@@ -135,8 +136,8 @@ void lsd::start(error_code& ec)
 	}
 
 	ADD_OUTSTANDING_ASYNC("lsd::on_announce");
-	m_socket.async_receive_from(boost::asio::buffer(m_buffer), m_remote
-		, std::bind(&lsd::on_announce, self(), _1, _2));
+	m_socket.async_wait(udp::socket::wait_read
+		, std::bind(&lsd::on_announce, self(), _1));
 }
 
 lsd::~lsd() = default;
@@ -192,7 +193,7 @@ void lsd::announce_impl(sha1_hash const& ih, int const listen_port
 	if (m_disabled) return;
 
 	ADD_OUTSTANDING_ASYNC("lsd::resend_announce");
-	m_broadcast_timer.expires_from_now(seconds(2 * retry_count), ec);
+	m_broadcast_timer.expires_after(seconds(2 * retry_count));
 	m_broadcast_timer.async_wait(std::bind(&lsd::resend_announce, self(), _1
 		, ih, listen_port, retry_count));
 }
@@ -206,26 +207,20 @@ void lsd::resend_announce(error_code const& e, sha1_hash const& info_hash
 	announce_impl(info_hash, listen_port, retry_count);
 }
 
-void lsd::on_announce(error_code const& ec, std::size_t len)
+void lsd::on_announce(error_code const& ec)
 {
 	COMPLETE_ASYNC("lsd::on_announce");
-	if (ec)
-	{
-#ifndef TORRENT_DISABLE_LOGGING
-		debug_log("<== LSD: receive error: %s", ec.message().c_str());
-#endif
-		return;
-	}
+	if (ec) return;
 
-	udp::endpoint const from = m_remote;
+	std::array<char, 1500> buffer;
+	udp::endpoint from;
+	error_code err;
+	int const len = static_cast<int>(m_socket.receive_from(
+		boost::asio::buffer(buffer), from, {}, err));
 
-	// reissue the async receive as we exit the function. We can't do this
-	// earlier because we still need to use m_buffer
-	auto reissue_receive = aux::scope_end([&] {
-		ADD_OUTSTANDING_ASYNC("lsd::on_announce");
-		m_socket.async_receive_from(boost::asio::buffer(m_buffer), m_remote
-			, std::bind(&lsd::on_announce, self(), _1, _2));
-	});
+	ADD_OUTSTANDING_ASYNC("lsd::on_announce");
+	m_socket.async_wait(udp::socket::wait_read
+		, std::bind(&lsd::on_announce, self(), _1));
 
 	if (!match_addr_mask(from.address(), m_listen_address, m_netmask))
 	{
@@ -237,10 +232,18 @@ void lsd::on_announce(error_code const& ec, std::size_t len)
 		return;
 	}
 
+	if (err)
+	{
+#ifndef TORRENT_DISABLE_LOGGING
+		debug_log("<== LSD: receive error: %s", err.message().c_str());
+#endif
+		return;
+	}
+
 	http_parser p;
 
 	bool error = false;
-	p.incoming(span<char const>{m_buffer.data(), std::ptrdiff_t(len)}, error);
+	p.incoming(span<char const>{buffer.data(), len}, error);
 
 	if (!p.header_finished() || error)
 	{
@@ -330,7 +333,7 @@ void lsd::close()
 {
 	error_code ec;
 	m_socket.close(ec);
-	m_broadcast_timer.cancel(ec);
+	m_broadcast_timer.cancel();
 	m_disabled = true;
 }
 
