@@ -399,10 +399,8 @@ namespace {
 		TORRENT_ASSERT_PRECOND(!params.save_path.empty());
 
 #if TORRENT_ABI_VERSION < 3
-		if (params.info_hashes.v1.is_all_zeros())
+		if (!params.info_hashes.has_v1() && !params.info_hashes.has_v2() && !params.ti)
 			params.info_hashes.v1 = params.info_hash;
-		if (params.info_hash.is_all_zeros())
-			params.info_hash = params.info_hashes.v1;
 #endif
 
 		// the internal torrent object keeps and mutates state in the
@@ -431,10 +429,8 @@ namespace {
 		TORRENT_ASSERT_PRECOND(!params.save_path.empty());
 
 #if TORRENT_ABI_VERSION < 3
-		if (params.info_hashes.v1.is_all_zeros())
+		if (!params.info_hashes.has_v1() && !params.info_hashes.has_v2() && !params.ti)
 			params.info_hashes.v1 = params.info_hash;
-		if (params.info_hash.is_all_zeros())
-			params.info_hash = params.info_hashes.v1;
 #endif
 
 		// the internal torrent object keeps and mutates state in the
@@ -465,10 +461,8 @@ namespace {
 		TORRENT_ASSERT_PRECOND(!params.save_path.empty());
 
 #if TORRENT_ABI_VERSION < 3
-		if (params.info_hashes.v1.is_all_zeros())
+		if (!params.info_hashes.has_v1() && !params.info_hashes.has_v2() && !params.ti)
 			params.info_hashes.v1 = params.info_hash;
-		if (params.info_hash.is_all_zeros())
-			params.info_hash = params.info_hashes.v1;
 #endif
 
 		// the internal torrent object keeps and mutates state in the
