@@ -1,6 +1,13 @@
 /*
 
-Copyright (c) 2003-2018, Arvid Norberg
+Copyright (c) 2003-2020, Arvid Norberg
+Copyright (c) 2004, Magnus Jonsson
+Copyright (c) 2016-2017, 2019, Alden Torres
+Copyright (c) 2017, Falcosc
+Copyright (c) 2017, 2020, AllSeeingEyeTolledEweSew
+Copyright (c) 2018, Steven Siloti
+Copyright (c) 2019, ghbplayer
+Copyright (c) 2019, Andrei Kurushin
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -38,27 +45,23 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/torrent_handle.hpp"
 #include "libtorrent/torrent.hpp"
-#include "libtorrent/torrent_info.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/entry.hpp"
 #include "libtorrent/aux_/session_impl.hpp"
 #include "libtorrent/aux_/session_call.hpp"
 #include "libtorrent/aux_/throw.hpp"
-#include "libtorrent/invariant_check.hpp"
+#include "libtorrent/aux_/invariant_check.hpp"
 #include "libtorrent/utf8.hpp"
 #include "libtorrent/announce_entry.hpp"
 #include "libtorrent/write_resume_data.hpp"
 #include "libtorrent/torrent_flags.hpp"
 #include "libtorrent/pex_flags.hpp"
+#include "libtorrent/aux_/ip_helpers.hpp" // for is_v6
 
 #if TORRENT_ABI_VERSION == 1
 #include "libtorrent/peer_info.hpp" // for peer_list_entry
 #endif
 
-#if TORRENT_ABI_VERSION == 1 && defined TORRENT_WINDOWS
-#include "libtorrent/aux_/escape_string.hpp"
-#endif
-
 using libtorrent::aux::session_impl;
 
 namespace libtorrent {
@@ -71,6 +74,7 @@ namespace libtorrent {
 	constexpr pause_flags_t torrent_handle::clear_disk_cache;
 	constexpr deadline_flags_t torrent_handle::alert_when_available;
 	constexpr reannounce_flags_t torrent_handle::ignore_min_interval;
+	constexpr file_progress_flags_t torrent_handle::piece_granularity;
 
 	constexpr status_flags_t torrent_handle::query_distributed_copies;
 	constexpr status_flags_t torrent_handle::query_accurate_download_counters;
@@ -81,6 +85,24 @@ namespace libtorrent {
 	constexpr status_flags_t torrent_handle::query_name;
 	constexpr status_flags_t torrent_handle::query_save_path;
 
+	void block_info::set_peer(tcp::endpoint const& ep)
+	{
+		is_v6_addr = aux::is_v6(ep);
+		if (is_v6_addr)
+			addr.v6 = ep.address().to_v6().to_bytes();
+		else
+			addr.v4 = ep.address().to_v4().to_bytes();
+		port = ep.port();
+	}
+
+	tcp::endpoint block_info::peer() const
+	{
+		if (is_v6_addr)
+			return {address_v6(addr.v6), port};
+		else
+			return {address_v4(addr.v4), port};
+	}
+
 #ifndef BOOST_NO_EXCEPTIONS
 	[[noreturn]] void throw_invalid_handle()
 	{
@@ -94,21 +116,21 @@ namespace libtorrent {
 		std::shared_ptr<torrent> t = m_torrent.lock();
 		if (!t) aux::throw_ex<system_error>(errors::invalid_torrent_handle);
 		auto& ses = static_cast<session_impl&>(t->session());
-		ses.get_io_service().dispatch([=,&ses] ()
+		dispatch(ses.get_context(), [=,&ses] ()
 		{
 #ifndef BOOST_NO_EXCEPTIONS
 			try {
 #endif
-				(t.get()->*f)(std::move(a)...);
+				(t.get()->*f)(a...);
 #ifndef BOOST_NO_EXCEPTIONS
 			} catch (system_error const& e) {
-				ses.alerts().emplace_alert<torrent_error_alert>(torrent_handle(t)
+				ses.alerts().emplace_alert<torrent_error_alert>(torrent_handle(m_torrent)
 					, e.code(), e.what());
 			} catch (std::exception const& e) {
-				ses.alerts().emplace_alert<torrent_error_alert>(torrent_handle(t)
+				ses.alerts().emplace_alert<torrent_error_alert>(torrent_handle(m_torrent)
 					, error_code(), e.what());
 			} catch (...) {
-				ses.alerts().emplace_alert<torrent_error_alert>(torrent_handle(t)
+				ses.alerts().emplace_alert<torrent_error_alert>(torrent_handle(m_torrent)
 					, error_code(), "unknown error");
 			}
 #endif
@@ -126,12 +148,12 @@ namespace libtorrent {
 		bool done = false;
 
 		std::exception_ptr ex;
-		ses.get_io_service().dispatch([=,&done,&ses,&ex] ()
+		dispatch(ses.get_context(), [=,&done,&ses,&ex] ()
 		{
 #ifndef BOOST_NO_EXCEPTIONS
 			try {
 #endif
-				(t.get()->*f)(std::move(a)...);
+				(t.get()->*f)(a...);
 #ifndef BOOST_NO_EXCEPTIONS
 			} catch (...) {
 				ex = std::current_exception();
@@ -162,12 +184,12 @@ namespace libtorrent {
 		bool done = false;
 
 		std::exception_ptr ex;
-		ses.get_io_service().dispatch([=,&r,&done,&ses,&ex] ()
+		dispatch(ses.get_context(), [=,&r,&done,&ses,&ex] ()
 		{
 #ifndef BOOST_NO_EXCEPTIONS
 			try {
 #endif
-				r = (t.get()->*f)(std::move(a)...);
+				r = (t.get()->*f)(a...);
 #ifndef BOOST_NO_EXCEPTIONS
 			} catch (...) {
 				ex = std::current_exception();
@@ -187,7 +209,13 @@ namespace libtorrent {
 	sha1_hash torrent_handle::info_hash() const
 	{
 		std::shared_ptr<torrent> t = m_torrent.lock();
-		return t ? t->info_hash() : sha1_hash();
+		return t ? t->info_hash().get_best() : sha1_hash{};
+	}
+
+	info_hash_t torrent_handle::info_hashes() const
+	{
+		std::shared_ptr<torrent> t = m_torrent.lock();
+		return t ? t->info_hash() : info_hash_t();
 	}
 
 	int torrent_handle::max_uploads() const
@@ -245,19 +273,6 @@ namespace libtorrent {
 	{
 		async_call(&torrent::move_storage, save_path, static_cast<move_flags_t>(flags));
 	}
-
-#if defined TORRENT_WINDOWS
-	void torrent_handle::move_storage(
-		std::wstring const& save_path, int flags) const
-	{
-		async_call(&torrent::move_storage, convert_from_wstring(save_path), static_cast<move_flags_t>(flags));
-	}
-
-	void torrent_handle::rename_file(file_index_t index, std::wstring const& new_name) const
-	{
-		async_call(&torrent::rename_file, index, convert_from_wstring(new_name));
-	}
-#endif // TORRENT_WINDOWS
 #endif // TORRENT_ABI_VERSION
 
 	void torrent_handle::rename_file(file_index_t index, std::string const& new_name) const
@@ -266,8 +281,8 @@ namespace libtorrent {
 	}
 
 	void torrent_handle::add_extension(
-		std::function<std::shared_ptr<torrent_plugin>(torrent_handle const&, void*)> const& ext
-		, void* userdata)
+		std::function<std::shared_ptr<torrent_plugin>(torrent_handle const&, client_data_t)> const& ext
+		, client_data_t userdata)
 	{
 #ifndef TORRENT_DISABLE_EXTENSIONS
 		async_call(&torrent::add_extension_fun, ext, userdata);
@@ -346,7 +361,7 @@ namespace libtorrent {
 		, std::string const& dh_params
 		, std::string const& passphrase)
 	{
-#ifdef TORRENT_USE_OPENSSL
+#ifdef TORRENT_SSL_PEERS
 		async_call(&torrent::set_ssl_cert, certificate, private_key, dh_params, passphrase);
 #else
 		TORRENT_UNUSED(certificate);
@@ -361,7 +376,7 @@ namespace libtorrent {
 		, std::string const& private_key
 		, std::string const& dh_params)
 	{
-#ifdef TORRENT_USE_OPENSSL
+#ifdef TORRENT_SSL_PEERS
 		async_call(&torrent::set_ssl_cert_buffer, certificate, private_key, dh_params);
 #else
 		TORRENT_UNUSED(certificate);
@@ -441,12 +456,19 @@ namespace libtorrent {
 	}
 #endif
 
-	void torrent_handle::file_progress(std::vector<std::int64_t>& progress, int flags) const
+	void torrent_handle::file_progress(std::vector<std::int64_t>& progress, file_progress_flags_t flags) const
 	{
 		auto& arg = static_cast<aux::vector<std::int64_t, file_index_t>&>(progress);
 		sync_call(&torrent::file_progress, std::ref(arg), flags);
 	}
 
+	std::vector<std::int64_t> torrent_handle::file_progress(file_progress_flags_t flags) const
+	{
+		aux::vector<std::int64_t, file_index_t> ret;
+		sync_call(&torrent::file_progress, std::ref(ret), flags);
+		return std::move(ret);
+	}
+
 	torrent_status torrent_handle::status(status_flags_t const flags) const
 	{
 		torrent_status st;
@@ -487,7 +509,7 @@ namespace libtorrent {
 		aux::vector<download_priority_t, piece_index_t> ret;
 		auto retp = &ret;
 		sync_call(&torrent::piece_priorities, retp);
-		return TORRENT_RVO(ret);
+		return std::move(ret);
 	}
 
 #if TORRENT_ABI_VERSION == 1
@@ -543,7 +565,7 @@ namespace libtorrent {
 		aux::vector<download_priority_t, file_index_t> ret;
 		auto retp = &ret;
 		sync_call(&torrent::file_priorities, retp);
-		return TORRENT_RVO(ret);
+		return std::move(ret);
 	}
 
 #if TORRENT_ABI_VERSION == 1
@@ -676,12 +698,6 @@ namespace libtorrent {
 		sync_call(&torrent::add_piece, piece, data, flags);
 	}
 
-	void torrent_handle::add_piece(piece_index_t piece, std::vector<char> data
-		, add_piece_flags_t const flags) const
-	{
-		async_call(&torrent::add_piece_async, piece, std::move(data), flags);
-	}
-
 	void torrent_handle::read_piece(piece_index_t piece) const
 	{
 		async_call(&torrent::read_piece, piece);
@@ -692,11 +708,6 @@ namespace libtorrent {
 		return sync_call_ret<bool>(false, &torrent::user_have_piece, piece);
 	}
 
-	storage_interface* torrent_handle::get_storage_impl() const
-	{
-		return sync_call_ret<storage_interface*>(nullptr, &torrent::get_storage_impl);
-	}
-
 	bool torrent_handle::is_valid() const
 	{
 		return !m_torrent.expired();
@@ -705,7 +716,19 @@ namespace libtorrent {
 	std::shared_ptr<const torrent_info> torrent_handle::torrent_file() const
 	{
 		return sync_call_ret<std::shared_ptr<const torrent_info>>(
-			std::shared_ptr<const torrent_info>(), &torrent::get_torrent_copy);
+			std::shared_ptr<const torrent_info>(), &torrent::get_torrent_file);
+	}
+
+	std::shared_ptr<torrent_info> torrent_handle::torrent_file_with_hashes() const
+	{
+		return sync_call_ret<std::shared_ptr<torrent_info>>(
+			std::shared_ptr<torrent_info>(), &torrent::get_torrent_copy_with_hashes);
+	}
+
+	std::vector<std::vector<sha256_hash>> torrent_handle::piece_layers() const
+	{
+		return sync_call_ret<std::vector<std::vector<sha256_hash>>>({}
+			, &torrent::get_piece_layers);
 	}
 
 #if TORRENT_ABI_VERSION == 1
@@ -783,6 +806,11 @@ namespace libtorrent {
 #endif
 	}
 
+	void torrent_handle::force_lsd_announce() const
+	{
+		async_call(&torrent::lsd_announce);
+	}
+
 	void torrent_handle::force_reannounce(int s, int idx, reannounce_flags_t const flags) const
 	{
 		async_call(&torrent::force_tracker_request, aux::time_now() + seconds(s), idx, flags);
@@ -829,6 +857,13 @@ namespace libtorrent {
 		sync_call(&torrent::get_download_queue, queuep);
 	}
 
+	std::vector<partial_piece_info> torrent_handle::get_download_queue() const
+	{
+		std::vector<partial_piece_info> queue;
+		sync_call(&torrent::get_download_queue, &queue);
+		return queue;
+	}
+
 	void torrent_handle::set_piece_deadline(piece_index_t index, int deadline
 		, deadline_flags_t const flags) const
 	{
@@ -869,10 +904,10 @@ namespace libtorrent {
 		return std::size_t(*reinterpret_cast<void* const*>(&th.m_torrent));
 	}
 
-	void* torrent_handle::userdata() const
+	client_data_t torrent_handle::userdata() const
 	{
 		std::shared_ptr<torrent> t = m_torrent.lock();
-		return t ? t->get_userdata() : nullptr;
+		return t ? t->get_userdata() : client_data_t{};
 	}
 
 	bool torrent_handle::in_session() const
