@@ -1,6 +1,14 @@
 /*
 
-Copyright (c) 2007-2018, Arvid Norberg
+Copyright (c) 2019, Amir Abrams
+Copyright (c) 2007-2020, Arvid Norberg
+Copyright (c) 2009, Andrew Resch
+Copyright (c) 2015, Mike Tzou
+Copyright (c) 2016-2019, Alden Torres
+Copyright (c) 2016-2017, Pavel Pimenov
+Copyright (c) 2016, Steven Siloti
+Copyright (c) 2016-2017, Andrei Kurushin
+Copyright (c) 2020, Paul-Louis Ageneau
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -41,19 +49,16 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/time.hpp" // for aux::time_now()
 #include "libtorrent/aux_/escape_string.hpp" // for convert_from_native
 #include "libtorrent/http_connection.hpp"
-#include "libtorrent/aux_/scope_end.hpp"
+#include "libtorrent/aux_/numeric_cast.hpp"
+#include "libtorrent/ssl.hpp"
 
 #if defined TORRENT_ASIO_DEBUGGING
 #include "libtorrent/debug.hpp"
 #endif
-#include "libtorrent/aux_/numeric_cast.hpp"
 
 #include "libtorrent/aux_/disable_warnings_push.hpp"
 #include <boost/asio/ip/host_name.hpp>
 #include <boost/asio/ip/multicast.hpp>
-#ifdef TORRENT_USE_OPENSSL
-#include <boost/asio/ssl/context.hpp>
-#endif
 #include "libtorrent/aux_/disable_warnings_pop.hpp"
 
 #include <cstdlib>
@@ -82,6 +87,7 @@ namespace upnp_errors
 static error_code ignore_error;
 
 upnp::rootdevice::rootdevice() = default;
+
 #if TORRENT_USE_ASSERTS
 upnp::rootdevice::~rootdevice()
 {
@@ -93,36 +99,36 @@ upnp::rootdevice::~rootdevice() = default;
 #endif
 
 upnp::rootdevice::rootdevice(rootdevice const&) = default;
-upnp::rootdevice& upnp::rootdevice::operator=(rootdevice const&) = default;
-upnp::rootdevice::rootdevice(rootdevice&&) = default;
-upnp::rootdevice& upnp::rootdevice::operator=(rootdevice&&) = default;
+upnp::rootdevice& upnp::rootdevice::operator=(rootdevice const&) & = default;
+upnp::rootdevice::rootdevice(rootdevice&&) noexcept = default;
+upnp::rootdevice& upnp::rootdevice::operator=(rootdevice&&) & = default;
 
-// TODO: 3 bind the broadcast socket. it would probably have to be changed to a vector of interfaces to
-// bind to, since the broadcast socket opens one socket per local
-// interface by default
-upnp::upnp(io_service& ios
+// TODO: 2 use boost::asio::ip::network instead of netmask
+upnp::upnp(io_context& ios
 	, aux::session_settings const& settings
 	, aux::portmap_callback& cb
-	, address_v4 const& listen_address
-	, address_v4 const& netmask
-	, std::string listen_device)
+	, address_v4 const listen_address
+	, address_v4 const netmask
+	, std::string listen_device
+	, listen_socket_handle ls)
 	: m_settings(settings)
 	, m_callback(cb)
 	, m_io_service(ios)
 	, m_resolver(ios)
-	, m_multicast(ios)
-	, m_unicast(ios)
+	, m_multicast_socket(ios)
+	, m_unicast_socket(ios)
 	, m_broadcast_timer(ios)
 	, m_refresh_timer(ios)
 	, m_map_timer(ios)
 	, m_listen_address(listen_address)
 	, m_netmask(netmask)
 	, m_device(std::move(listen_device))
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 	, m_ssl_ctx(ssl::context::sslv23_client)
 #endif
+	, m_listen_handle(std::move(ls))
 {
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 	m_ssl_ctx.set_verify_mode(ssl::context::verify_none);
 #endif
 }
@@ -132,7 +138,7 @@ void upnp::start()
 	TORRENT_ASSERT(is_single_thread());
 
 	error_code ec;
-	open_multicast_socket(m_multicast, ec);
+	open_multicast_socket(m_multicast_socket, ec);
 #ifndef TORRENT_DISABLE_LOGGING
 	if (ec && should_log())
 	{
@@ -143,7 +149,7 @@ void upnp::start()
 	}
 #endif
 
-	open_unicast_socket(m_unicast, ec);
+	open_unicast_socket(m_unicast_socket, ec);
 #ifndef TORRENT_DISABLE_LOGGING
 	if (ec && should_log())
 	{
@@ -165,39 +171,39 @@ namespace {
 
 }
 
-void upnp::open_multicast_socket(aux::socket_package& s, error_code& ec)
+void upnp::open_multicast_socket(udp::socket& s, error_code& ec)
 {
 	using namespace boost::asio::ip::multicast;
-	s.socket.open(udp::v4(), ec);
+	s.open(udp::v4(), ec);
 	if (ec) return;
-	s.socket.set_option(udp::socket::reuse_address(true), ec);
+	s.set_option(udp::socket::reuse_address(true), ec);
 	if (ec) return;
-	s.socket.bind(udp::endpoint(m_listen_address, ssdp_port), ec);
+	s.bind(udp::endpoint(m_listen_address, ssdp_port), ec);
 	if (ec) return;
-	s.socket.set_option(join_group(ssdp_multicast_addr), ec);
+	s.set_option(join_group(ssdp_multicast_addr), ec);
 	if (ec) return;
-	s.socket.set_option(hops(255), ec);
+	s.set_option(hops(255), ec);
 	if (ec) return;
-	s.socket.set_option(enable_loopback(true), ec);
+	s.set_option(enable_loopback(true), ec);
 	if (ec) return;
-	s.socket.set_option(outbound_interface(m_listen_address), ec);
+	s.set_option(outbound_interface(m_listen_address), ec);
 	if (ec) return;
 
 	ADD_OUTSTANDING_ASYNC("upnp::on_reply");
-	s.socket.async_receive_from(boost::asio::buffer(s.buffer), s.remote
-		, std::bind(&upnp::on_reply, self(), std::ref(s), _1, _2));
+	s.async_receive(boost::asio::null_buffers{}
+		, std::bind(&upnp::on_reply, self(), std::ref(s), _1));
 }
 
-void upnp::open_unicast_socket(aux::socket_package& s, error_code& ec)
+void upnp::open_unicast_socket(udp::socket& s, error_code& ec)
 {
-	s.socket.open(udp::v4(), ec);
+	s.open(udp::v4(), ec);
 	if (ec) return;
-	s.socket.bind(udp::endpoint(m_listen_address, 0), ec);
+	s.bind(udp::endpoint(m_listen_address, 0), ec);
 	if (ec) return;
 
 	ADD_OUTSTANDING_ASYNC("upnp::on_reply");
-	s.socket.async_receive_from(boost::asio::buffer(s.buffer), s.remote
-		, std::bind(&upnp::on_reply, self(), std::ref(s), _1, _2));
+	s.async_receive(boost::asio::null_buffers{}
+		, std::bind(&upnp::on_reply, self(), std::ref(s), _1));
 }
 
 upnp::~upnp() = default;
@@ -218,7 +224,7 @@ void upnp::log(char const* fmt, ...) const
 	char msg[1024];
 	std::vsnprintf(msg, sizeof(msg), fmt, v);
 	va_end(v);
-	m_callback.log_portmap(portmap_transport::upnp, msg);
+	m_callback.log_portmap(portmap_transport::upnp, msg, m_listen_handle);
 }
 #endif
 
@@ -233,7 +239,6 @@ void upnp::discover_device_impl()
 		"MX:3\r\n"
 		"\r\n\r\n";
 
-	error_code ec;
 #ifdef TORRENT_DEBUG_UPNP
 	// simulate packet loss
 	if (m_retry_count & 1)
@@ -241,9 +246,9 @@ void upnp::discover_device_impl()
 
 	error_code mcast_ec;
 	error_code ucast_ec;
-	m_multicast.socket.send_to(boost::asio::buffer(msearch, sizeof(msearch) - 1)
+	m_multicast_socket.send_to(boost::asio::buffer(msearch, sizeof(msearch) - 1)
 		, udp::endpoint(ssdp_multicast_addr, ssdp_port), 0, mcast_ec);
-	m_unicast.socket.send_to(boost::asio::buffer(msearch, sizeof(msearch) - 1)
+	m_unicast_socket.send_to(boost::asio::buffer(msearch, sizeof(msearch) - 1)
 		, udp::endpoint(ssdp_multicast_addr, ssdp_port), 0, ucast_ec);
 
 	if (mcast_ec && ucast_ec)
@@ -262,7 +267,7 @@ void upnp::discover_device_impl()
 
 	ADD_OUTSTANDING_ASYNC("upnp::resend_request");
 	++m_retry_count;
-	m_broadcast_timer.expires_from_now(seconds(2 * m_retry_count), ec);
+	m_broadcast_timer.expires_after(seconds(2 * m_retry_count));
 	m_broadcast_timer.async_wait(std::bind(&upnp::resend_request
 		, self(), _1));
 
@@ -273,7 +278,7 @@ void upnp::discover_device_impl()
 
 // returns a reference to a mapping or -1 on failure
 port_mapping_t upnp::add_mapping(portmap_protocol const p, int const external_port
-	, tcp::endpoint const local_ep, std::string const& device)
+	, tcp::endpoint const local_ep)
 {
 	TORRENT_ASSERT(is_single_thread());
 	// external port 0 means _every_ port
@@ -283,10 +288,9 @@ port_mapping_t upnp::add_mapping(portmap_protocol const p, int const external_po
 	if (should_log())
 	{
 		log("adding port map: [ protocol: %s ext_port: %d "
-			"local_ep: %s device: %s] %s", (p == portmap_protocol::tcp?"tcp":"udp")
+			"local_ep: %s ] %s", (p == portmap_protocol::tcp?"tcp":"udp")
 			, external_port
-			, print_endpoint(local_ep).c_str(), device.c_str()
-			, m_disabled ? "DISABLED": "");
+			, print_endpoint(local_ep).c_str(), m_disabled ? "DISABLED": "");
 	}
 #endif
 	if (m_disabled) return port_mapping_t{-1};
@@ -311,13 +315,12 @@ port_mapping_t upnp::add_mapping(portmap_protocol const p, int const external_po
 	mapping_it->protocol = p;
 	mapping_it->external_port = external_port;
 	mapping_it->local_ep = local_ep;
-	mapping_it->device = device;
 
 	port_mapping_t const mapping_index{static_cast<int>(mapping_it - m_mappings.begin())};
 
 	for (auto const& dev : m_devices)
 	{
-		rootdevice& d = const_cast<rootdevice&>(dev);
+		auto& d = const_cast<rootdevice&>(dev);
 		TORRENT_ASSERT(d.magic == 1337);
 		if (d.disabled) continue;
 
@@ -329,7 +332,6 @@ port_mapping_t upnp::add_mapping(portmap_protocol const p, int const external_po
 		m.protocol = p;
 		m.external_port = external_port;
 		m.local_ep = local_ep;
-		m.device = device;
 
 		if (!d.service_namespace.empty()) update_map(d, mapping_index);
 	}
@@ -349,10 +351,8 @@ void upnp::delete_mapping(port_mapping_t const mapping)
 	if (should_log())
 	{
 		log("deleting port map: [ protocol: %s ext_port: %u "
-			"local_ep: %s device: %s]"
-			, (m.protocol == portmap_protocol::tcp?"tcp":"udp"), m.external_port
-			, print_endpoint(m.local_ep).c_str()
-			, m.device.c_str());
+			"local_ep: %s ]", (m.protocol == portmap_protocol::tcp?"tcp":"udp"), m.external_port
+			, print_endpoint(m.local_ep).c_str());
 	}
 #endif
 
@@ -360,7 +360,7 @@ void upnp::delete_mapping(port_mapping_t const mapping)
 
 	for (auto const& dev : m_devices)
 	{
-		rootdevice& d = const_cast<rootdevice&>(dev);
+		auto& d = const_cast<rootdevice&>(dev);
 		TORRENT_ASSERT(d.magic == 1337);
 		if (d.disabled) continue;
 
@@ -441,11 +441,11 @@ void upnp::connect(rootdevice& d)
 			, http_connect_handler()
 			, http_filter_handler()
 			, hostname_filter_handler()
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 			, &m_ssl_ctx
 #endif
 			);
-		d.upnp_connection->get(d.url, seconds(30));
+		d.upnp_connection->get(d.url, seconds(30), 1);
 	}
 	TORRENT_CATCH (std::exception const& exc)
 	{
@@ -458,7 +458,7 @@ void upnp::connect(rootdevice& d)
 	}
 }
 
-void upnp::on_reply(aux::socket_package& s, error_code const& ec, std::size_t const len)
+void upnp::on_reply(udp::socket& s, error_code const& ec)
 {
 	TORRENT_ASSERT(is_single_thread());
 	COMPLETE_ASYNC("upnp::on_reply");
@@ -467,7 +467,13 @@ void upnp::on_reply(aux::socket_package& s, error_code const& ec, std::size_t co
 	if (m_closing) return;
 
 	std::shared_ptr<upnp> me(self());
-	udp::endpoint const from = s.remote;
+
+	std::array<char, 1500> buffer{};
+	udp::endpoint from;
+	error_code err;
+	int const len = static_cast<int>(s.receive_from(boost::asio::buffer(buffer)
+		, from, 0, err));
+
 	// parse out the url for the device
 
 /*
@@ -495,15 +501,11 @@ void upnp::on_reply(aux::socket_package& s, error_code const& ec, std::size_t co
 
 */
 
-	// reissue the async receive as we exit the function. We can't do this
-	// earlier because we still need to use s.buffer
-	auto reissue_receive = scope_end([&] {
-		ADD_OUTSTANDING_ASYNC("upnp::on_reply");
-		s.socket.async_receive_from(boost::asio::buffer(s.buffer), s.remote
-			, std::bind(&upnp::on_reply, self(), std::ref(s), _1, _2));
-	});
+	ADD_OUTSTANDING_ASYNC("upnp::on_reply");
+	s.async_receive(boost::asio::null_buffers{}
+		, std::bind(&upnp::on_reply, self(), std::ref(s), _1));
 
-	if (ec) return;
+	if (err) return;
 
 	if (m_settings.get_bool(settings_pack::upnp_ignore_nonrouters)
 		&& !match_addr_mask(m_listen_address, from.address(), m_netmask))
@@ -522,7 +524,7 @@ void upnp::on_reply(aux::socket_package& s, error_code const& ec, std::size_t co
 
 	http_parser p;
 	bool error = false;
-	p.incoming({s.buffer.data(), std::ptrdiff_t(len)}, error);
+	p.incoming({buffer.data(), len}, error);
 	if (error)
 	{
 #ifndef TORRENT_DISABLE_LOGGING
@@ -589,7 +591,6 @@ void upnp::on_reply(aux::socket_package& s, error_code const& ec, std::size_t co
 	{
 		std::string protocol;
 		std::string auth;
-		error_code err;
 		// we don't have this device in our list. Add it
 		std::tie(protocol, auth, d.hostname, d.port, d.path)
 			= parse_url_components(d.url, err);
@@ -659,7 +660,6 @@ void upnp::on_reply(aux::socket_package& s, error_code const& ec, std::size_t co
 			mapping_t m;
 			m.act = portmap_action::add;
 			m.local_ep = j.local_ep;
-			m.device = j.device;
 			m.external_port = j.external_port;
 			m.protocol = j.protocol;
 			d.mapping.push_back(m);
@@ -674,8 +674,7 @@ void upnp::on_reply(aux::socket_package& s, error_code const& ec, std::size_t co
 	// check back in a little bit to see if we have seen any
 	// devices at one of our default routes. If not, we want to override
 	// ignoring them and use them instead (better than not working).
-	error_code ignore;
-	m_map_timer.expires_from_now(seconds(1), ignore);
+	m_map_timer.expires_after(seconds(1));
 	ADD_OUTSTANDING_ASYNC("upnp::map_timer");
 	m_map_timer.async_wait(std::bind(&upnp::map_timer, self(), _1));
 }
@@ -855,14 +854,13 @@ void upnp::update_map(rootdevice& d, port_mapping_t const i)
 			, std::bind(&upnp::create_port_mapping, self(), _1, std::ref(d), i)
 			, http_filter_handler()
 			, hostname_filter_handler()
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 			, &m_ssl_ctx
 #endif
 			);
 
-		bind_info_t bi{m.device, m.local_ep.address()};
 		d.upnp_connection->start(d.hostname, d.port
-			, seconds(10), nullptr, false, 5, bi);
+			, seconds(10), 1, nullptr, false, 5, m.local_ep.address());
 	}
 	else if (m.act == portmap_action::del)
 	{
@@ -874,13 +872,12 @@ void upnp::update_map(rootdevice& d, port_mapping_t const i)
 			, std::bind(&upnp::delete_port_mapping, self(), std::ref(d), i)
 			, http_filter_handler()
 			, hostname_filter_handler()
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 			, &m_ssl_ctx
 #endif
 			);
-		bind_info_t bi{m.device, m.local_ep.address()};
 		d.upnp_connection->start(d.hostname, d.port
-			, seconds(10), nullptr, false, 5, bi);
+			, seconds(10), 1, nullptr, false, 5, m.local_ep.address());
 	}
 
 	m.act = portmap_action::none;
@@ -939,10 +936,7 @@ void find_control_url(int const type, string_view str, parse_state& state)
 	else if (type == xml_string)
 	{
 		if (state.tag_stack.empty()) return;
-		// default to the first (or only) control url in the router's listing
-		if (!state.in_service
-			&& state.top_tags("service", "servicetype")
-			&& state.service_type.empty())
+		if (!state.in_service && state.top_tags("service", "servicetype"))
 		{
 			if (string_equal_no_case(str, "urn:schemas-upnp-org:service:WANIPConnection:1")
 				|| string_equal_no_case(str, "urn:schemas-upnp-org:service:WANIPConnection:2")
@@ -1097,11 +1091,12 @@ void upnp::on_upnp_xml(error_code const& e
 		, std::bind(&upnp::get_ip_address, self(), std::ref(d))
 		, http_filter_handler()
 		, hostname_filter_handler()
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 		, &m_ssl_ctx
 #endif
 		);
-	d.upnp_connection->start(d.hostname, d.port, seconds(10));
+	d.upnp_connection->start(d.hostname, d.port
+		, seconds(10), 1);
 }
 
 void upnp::get_ip_address(rootdevice& d)
@@ -1144,20 +1139,20 @@ void upnp::disable(error_code const& ec)
 		if (i->protocol == portmap_protocol::none) continue;
 		portmap_protocol const proto = i->protocol;
 		i->protocol = portmap_protocol::none;
-		m_callback.on_port_mapping(port_mapping_t(static_cast<int>(i - m_mappings.begin())), address(), 0, proto, ec
-			, portmap_transport::upnp);
+		m_callback.on_port_mapping(port_mapping_t(static_cast<int>(i - m_mappings.begin()))
+			, address(), 0, proto, ec, portmap_transport::upnp, m_listen_handle);
 	}
 
 	// we cannot clear the devices since there
 	// might be outstanding requests relying on
 	// the device entry being present when they
 	// complete
+	m_broadcast_timer.cancel();
+	m_refresh_timer.cancel();
+	m_map_timer.cancel();
 	error_code e;
-	m_broadcast_timer.cancel(e);
-	m_refresh_timer.cancel(e);
-	m_map_timer.cancel(e);
-	m_unicast.socket.close(e);
-	m_multicast.socket.close(e);
+	m_unicast_socket.close(e);
+	m_multicast_socket.close(e);
 }
 
 void find_error_code(int const type, string_view string, error_code_parse_state& state)
@@ -1477,19 +1472,17 @@ void upnp::on_upnp_map_response(error_code const& e
 	if (s.error_code == -1)
 	{
 		m_callback.on_port_mapping(mapping, d.external_ip, m.external_port, m.protocol, error_code()
-			, portmap_transport::upnp);
+			, portmap_transport::upnp, m_listen_handle);
 		if (d.use_lease_duration && m_settings.get_int(settings_pack::upnp_lease_duration) != 0)
 		{
 			time_point const now = aux::time_now();
 			m.expires = now + seconds(
 				m_settings.get_int(settings_pack::upnp_lease_duration) * 3 / 4);
-
-			time_point next_expire = m_refresh_timer.expires_at();
+			time_point next_expire = m_refresh_timer.expiry();
 			if (next_expire < now || next_expire > m.expires)
 			{
 				ADD_OUTSTANDING_ASYNC("upnp::on_expire");
-				error_code ec;
-				m_refresh_timer.expires_at(m.expires, ec);
+				m_refresh_timer.expires_at(m.expires);
 				m_refresh_timer.async_wait(std::bind(&upnp::on_expire, self(), _1));
 			}
 		}
@@ -1522,7 +1515,7 @@ void upnp::return_error(port_mapping_t const mapping, int const code)
 	}
 	portmap_protocol const proto = m_mappings[mapping].protocol;
 	m_callback.on_port_mapping(mapping, address(), 0, proto, error_code(code, upnp_category())
-		, portmap_transport::upnp);
+		, portmap_transport::upnp, m_listen_handle);
 }
 
 void upnp::on_upnp_unmap_response(error_code const& e
@@ -1590,7 +1583,7 @@ void upnp::on_upnp_unmap_response(error_code const& e
 	m_callback.on_port_mapping(mapping, address(), 0, proto, p.status_code() != 200
 		? error_code(p.status_code(), http_category())
 		: error_code(s.error_code, upnp_category())
-		, portmap_transport::upnp);
+		, portmap_transport::upnp, m_listen_handle);
 
 	d.mapping[mapping].protocol = portmap_protocol::none;
 
@@ -1618,7 +1611,7 @@ void upnp::on_expire(error_code const& ec)
 
 	for (auto& dev : m_devices)
 	{
-		rootdevice& d = const_cast<rootdevice&>(dev);
+		auto& d = const_cast<rootdevice&>(dev);
 		TORRENT_ASSERT(d.magic == 1337);
 		if (d.disabled) continue;
 		for (port_mapping_t m{0}; m < m_mappings.end_index(); ++m)
@@ -1640,8 +1633,7 @@ void upnp::on_expire(error_code const& ec)
 	if (next_expire != max_time())
 	{
 		ADD_OUTSTANDING_ASYNC("upnp::on_expire");
-		error_code e;
-		m_refresh_timer.expires_at(next_expire, e);
+		m_refresh_timer.expires_at(next_expire);
 		m_refresh_timer.async_wait(std::bind(&upnp::on_expire, self(), _1));
 	}
 }
@@ -1650,17 +1642,17 @@ void upnp::close()
 {
 	TORRENT_ASSERT(is_single_thread());
 
-	error_code ec;
-	m_refresh_timer.cancel(ec);
-	m_broadcast_timer.cancel(ec);
-	m_map_timer.cancel(ec);
+	m_refresh_timer.cancel();
+	m_broadcast_timer.cancel();
+	m_map_timer.cancel();
 	m_closing = true;
-	m_unicast.socket.close(ec);
-	m_multicast.socket.close(ec);
+	error_code ec;
+	m_unicast_socket.close(ec);
+	m_multicast_socket.close(ec);
 
 	for (auto& dev : m_devices)
 	{
-		rootdevice& d = const_cast<rootdevice&>(dev);
+		auto& d = const_cast<rootdevice&>(dev);
 		TORRENT_ASSERT(d.magic == 1337);
 		if (d.disabled || d.control_url.empty()) continue;
 		for (auto& m : d.mapping)
