@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2016, Arvid Norberg
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2016-2020, Arvid Norberg
+Copyright (c) 2016, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,12 +35,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <deque>
 
 #include "make_torrent.hpp"
-#include "libtorrent/storage.hpp"
 #include "libtorrent/hasher.hpp"
 #include "libtorrent/entry.hpp"
 #include "libtorrent/bencode.hpp"
-#include "libtorrent/file_pool.hpp"
-#include "libtorrent/storage_defs.hpp"
+#include "libtorrent/aux_/session_settings.hpp"
+#include "libtorrent/aux_/posix_storage.hpp"
 
 using namespace lt;
 
@@ -121,6 +122,12 @@ std::shared_ptr<lt::torrent_info> make_test_torrent(torrent_args const& args)
 		e["httpseeds"] = args.m_http_seed;
 	}
 
+	if (!args.m_collection.empty())
+	{
+		auto& l = info["collections"].list();
+		l.push_back(args.m_collection);
+	}
+
 	std::string piece_hashes;
 
 	int num_pieces = (total_size + piece_length - 1) / piece_length;
@@ -154,8 +161,7 @@ std::shared_ptr<lt::torrent_info> make_test_torrent(torrent_args const& args)
 	info["pieces"] = piece_hashes;
 
 	std::vector<char> tmp;
-	std::back_insert_iterator<std::vector<char>> out(tmp);
-	bencode(out, e);
+	bencode(std::back_inserter(tmp), e);
 
 	FILE* f = fopen("test.torrent", "w+");
 	fwrite(&tmp[0], 1, tmp.size(), f);
@@ -167,8 +173,6 @@ std::shared_ptr<lt::torrent_info> make_test_torrent(torrent_args const& args)
 void generate_files(lt::torrent_info const& ti, std::string const& path
 	, bool alternate_data)
 {
-	file_pool fp;
-
 	aux::vector<download_priority_t, file_index_t> priorities;
 	sha1_hash info_hash;
 	storage_params params{
@@ -180,7 +184,9 @@ void generate_files(lt::torrent_info const& ti, std::string const& path
 		info_hash
 	};
 
-	default_storage st(params, fp);
+	// default settings
+	aux::session_settings sett;
+	aux::posix_storage st(params);
 
 	file_storage const& fs = ti.files();
 	std::vector<char> buffer;
@@ -198,7 +204,7 @@ void generate_files(lt::torrent_info const& ti, std::string const& path
 
 		iovec_t b = { &buffer[0], piece_size };
 		storage_error ec;
-		int ret = st.writev(b, i, 0, open_mode::read_only, ec);
+		int ret = st.writev(sett, b, i, 0, ec);
 		if (ret != piece_size || ec)
 		{
 			std::printf("ERROR writing files: (%d expected %d) %s\n"
