@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2013, Arvid Norberg
+Copyright (c) 2013, 2015-2019, Arvid Norberg
+Copyright (c) 2016, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,11 +34,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/entry.hpp"
 #include "libtorrent/address.hpp"
-#include "libtorrent/io_service.hpp"
+#include "libtorrent/io_context.hpp"
 #include "libtorrent/error_code.hpp"
 #include "libtorrent/socket.hpp"
 #include "libtorrent/aux_/time.hpp"
-#include "libtorrent/io_service.hpp"
+#include "libtorrent/io_context.hpp"
 #include "peer_server.hpp"
 #include "test_utils.hpp"
 
@@ -52,7 +53,7 @@ using namespace std::placeholders;
 
 struct peer_server
 {
-	lt::io_service m_ios;
+	lt::io_context m_ios;
 	std::atomic<int> m_peer_requests{0};
 	tcp::acceptor m_acceptor{m_ios};
 	int m_port = 0;
@@ -88,7 +89,7 @@ struct peer_server
 			return;
 		}
 
-		std::printf("%s: PEER peer initialized on port %d\n", time_now_string(), m_port);
+		std::printf("%s: PEER peer initialized on port %d\n", time_now_string().c_str(), m_port);
 
 		m_thread = std::make_shared<std::thread>(&peer_server::thread_fun, this);
 	}
@@ -124,7 +125,7 @@ struct peer_server
 			while (!done)
 			{
 				m_ios.poll_one();
-				m_ios.reset();
+				m_ios.restart();
 			}
 
 			if (ec == boost::asio::error::operation_aborted
@@ -136,7 +137,7 @@ struct peer_server
 				return;
 			}
 
-			std::printf("%s: PEER incoming peer connection\n", time_now_string());
+			std::printf("%s: PEER incoming peer connection\n", time_now_string().c_str());
 			++m_peer_requests;
 			socket.close(ec);
 		}
