@@ -705,8 +705,8 @@ std::vector<std::string> get_python()
 	if (req_size > 1 && req_size < 4096)
 	{
 		std::vector<char> buf(req_size);
-		DWORD const sz = GetEnvironmentVariable("PYTHON_INTERPRETER", buf.data(), buf.size());
-		if (sz == buf.size() - 1) ret.emplace_back(buf.data(), buf.size());
+		DWORD const sz = GetEnvironmentVariable("PYTHON_INTERPRETER", buf.data(), DWORD(buf.size()));
+		if (size_t(sz) == buf.size() - 1) ret.emplace_back(buf.data(), buf.size());
 	}
 #endif
 	ret.push_back("python3");
@@ -716,7 +716,7 @@ std::vector<std::string> get_python()
 
 int find_available_port()
 {
-	int port = 2000 + (::getpid() + _g_test_idx + std::rand()) % 60000;
+	int port = 2000 + (std::int64_t(::getpid()) + _g_test_idx + std::rand()) % 60000;
 	error_code ec;
 	io_context ios;
 
