@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2005, 2008-2010, 2013-2020, Arvid Norberg
+Copyright (c) 2015, Steven Siloti
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -51,19 +52,7 @@ POSSIBILITY OF SUCH DAMAGE.
 // in warning about deprecated use in any of the tests.
 // the unreachable code warnings are disabled since the test macros may
 // sometimes have conditions that are known at compile time
-#if defined __clang__
-#pragma clang diagnostic ignored "-Wdeprecated"
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-#pragma clang diagnostic ignored "-Wunreachable-code"
-
-#elif defined __GNUC__
-#pragma GCC diagnostic ignored "-Wdeprecated"
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#pragma GCC diagnostic ignored "-Wunreachable-code"
-
-#elif defined _MSC_VER
-#pragma warning(disable : 4996)
-#endif
+#include "libtorrent/aux_/disable_deprecation_warnings_push.hpp"
 
 #if defined TORRENT_BUILDING_TEST_SHARED
 #define EXPORT BOOST_SYMBOL_EXPORT
@@ -115,7 +104,7 @@ extern int _g_test_idx;
 #ifdef BOOST_NO_EXCEPTIONS
 #define TEST_CHECK(x) \
 	do if (!(x)) { \
-		TEST_REPORT_AUX("TEST_ERROR: check failed: \"" #x "\"", __FILE__, __LINE__);
+		TEST_REPORT_AUX("TEST_ERROR: check failed: \"" #x "\"", __FILE__, __LINE__); \
 	} while (false)
 #define TEST_EQUAL(x, y) \
 	do if ((x) != (y)) { \
@@ -136,9 +125,9 @@ extern int _g_test_idx;
 		if (!(x)) \
 			TEST_REPORT_AUX("TEST_ERROR: check failed: \"" #x "\"", __FILE__, __LINE__); \
 	} \
-	catch (std::exception const& e) \
+	catch (std::exception const& e__) \
 	{ \
-		TEST_ERROR("TEST_ERROR: Exception thrown: " #x " :" + std::string(e.what())); \
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x " :" + std::string(e__.what())); \
 	} \
 	catch (...) \
 	{ \
@@ -153,9 +142,9 @@ extern int _g_test_idx;
 			TEST_REPORT_AUX(s__.str().c_str(), __FILE__, __LINE__); \
 		} \
 	} \
-	catch (std::exception const& e) \
+	catch (std::exception const& e__) \
 	{ \
-		TEST_ERROR("TEST_ERROR: Exception thrown: " #x " :" + std::string(e.what())); \
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x " :" + std::string(e__.what())); \
 	} \
 	catch (...) \
 	{ \
@@ -169,9 +158,9 @@ extern int _g_test_idx;
 			TEST_REPORT_AUX(s__.str().c_str(), __FILE__, __LINE__); \
 		} \
 	} \
-	catch (std::exception const& e) \
+	catch (std::exception const& e__) \
 	{ \
-		TEST_ERROR("TEST_ERROR: Exception thrown: " #x " :" + std::string(e.what())); \
+		TEST_ERROR("TEST_ERROR: Exception thrown: " #x " :" + std::string(e__.what())); \
 	} \
 	catch (...) \
 	{ \
