@@ -1,6 +1,9 @@
 /*
 
-Copyright (c) 2015, Arvid Norberg
+Copyright (c) 2018, Steven Siloti
+Copyright (c) 2015-2020, Arvid Norberg
+Copyright (c) 2016, 2018, 2020, Alden Torres
+Copyright (c) 2018, d-komarov
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -31,7 +34,8 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "test.hpp"
-#include "libtorrent/alert_manager.hpp"
+#include "test_utils.hpp"
+#include "libtorrent/aux_/alert_manager.hpp"
 #include "libtorrent/torrent_handle.hpp"
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/extensions.hpp"
@@ -44,14 +48,14 @@ using namespace lt;
 
 TORRENT_TEST(limit)
 {
-	alert_manager mgr(500, alert_category::all);
+	aux::alert_manager mgr(500, alert_category::all);
 
 	TEST_EQUAL(mgr.alert_queue_size_limit(), 500);
 	TEST_EQUAL(mgr.pending(), false);
 
 	// try add 600 torrent_add_alert to make sure we honor the limit of 500
 	// alerts.
-	for (piece_index_t i{0}; i < piece_index_t{600}; ++i)
+	for (auto i = 0_piece; i < 600_piece; ++i)
 		mgr.emplace_alert<piece_finished_alert>(torrent_handle(), i);
 
 	TEST_EQUAL(mgr.pending(), true);
@@ -68,7 +72,7 @@ TORRENT_TEST(limit)
 	// now, try lowering the limit and do the same thing again
 	mgr.set_alert_queue_size_limit(200);
 
-	for (piece_index_t i{0}; i < piece_index_t{600}; ++i)
+	for (auto i = 0_piece; i < 600_piece; ++i)
 		mgr.emplace_alert<piece_finished_alert>(torrent_handle(), i);
 
 	TEST_EQUAL(mgr.pending(), true);
@@ -83,15 +87,15 @@ TORRENT_TEST(limit)
 TORRENT_TEST(limit_int_max)
 {
 	int const inf = std::numeric_limits<int>::max();
-	alert_manager mgr(inf, alert_category::all);
+	aux::alert_manager mgr(inf, alert_category::all);
 
 	TEST_EQUAL(mgr.alert_queue_size_limit(), inf);
 
-	for (piece_index_t i{0}; i < piece_index_t{600}; ++i)
+	for (auto i = 0_piece; i < 600_piece; ++i)
 		mgr.emplace_alert<piece_finished_alert>(torrent_handle(), i);
 
-	for (piece_index_t i{0}; i < piece_index_t{600}; ++i)
-		mgr.emplace_alert<torrent_removed_alert>(torrent_handle(), sha1_hash());
+	for (auto i = 0_piece; i < 600_piece; ++i)
+		mgr.emplace_alert<torrent_removed_alert>(torrent_handle(), info_hash_t(), client_data_t{});
 
 	std::vector<alert*> alerts;
 	mgr.get_all(alerts);
@@ -101,16 +105,16 @@ TORRENT_TEST(limit_int_max)
 
 TORRENT_TEST(priority_limit)
 {
-	alert_manager mgr(100, alert_category::all);
+	aux::alert_manager mgr(100, alert_category::all);
 
 	TEST_EQUAL(mgr.alert_queue_size_limit(), 100);
 
 	// this should only add 100 because of the limit
-	for (piece_index_t i{0}; i < piece_index_t{200}; ++i)
+	for (auto i = 0_piece; i < 200_piece; ++i)
 		mgr.emplace_alert<piece_finished_alert>(torrent_handle(), i);
 
 	// the limit is twice as high for priority alerts
-	for (file_index_t i(0); i < file_index_t(300); ++i)
+	for (auto i = 0_file; i < 300_file; ++i)
 		mgr.emplace_alert<file_rename_failed_alert>(torrent_handle(), i, error_code());
 
 	std::vector<alert*> alerts;
@@ -132,7 +136,7 @@ void test_notify_fun(int& cnt)
 TORRENT_TEST(notify_function)
 {
 	int cnt = 0;
-	alert_manager mgr(100, alert_category::all);
+	aux::alert_manager mgr(100, alert_category::all);
 
 	TEST_EQUAL(mgr.alert_queue_size_limit(), 100);
 	TEST_EQUAL(mgr.pending(), false);
@@ -191,7 +195,7 @@ TORRENT_TEST(extensions)
 {
 #ifndef TORRENT_DISABLE_EXTENSIONS
 	memset(plugin_alerts, 0, sizeof(plugin_alerts));
-	alert_manager mgr(100, alert_category::all);
+	aux::alert_manager mgr(100, alert_category::all);
 
 	mgr.add_extension(std::make_shared<test_plugin>(0));
 	mgr.add_extension(std::make_shared<test_plugin>(1));
@@ -216,7 +220,7 @@ TORRENT_TEST(extensions)
 /*
 namespace {
 
-void post_torrent_added(alert_manager* mgr)
+void post_torrent_added(aux::alert_manager* mgr)
 {
 	std::this_thread::sleep_for(lt::milliseconds(10));
 	mgr->emplace_alert<add_torrent_alert>(torrent_handle(), add_torrent_params(), error_code());
@@ -228,7 +232,7 @@ void post_torrent_added(alert_manager* mgr)
 
 TORRENT_TEST(wait_for_alert)
 {
-	alert_manager mgr(100, alert_category::all);
+	aux::alert_manager mgr(100, alert_category::all);
 
 	time_point start = clock_type::now();
 
@@ -270,7 +274,7 @@ TORRENT_TEST(wait_for_alert)
 
 TORRENT_TEST(alert_mask)
 {
-	alert_manager mgr(100, alert_category::all);
+	aux::alert_manager mgr(100, alert_category::all);
 
 	TEST_CHECK(mgr.should_post<add_torrent_alert>());
 	TEST_CHECK(mgr.should_post<torrent_paused_alert>());
@@ -283,7 +287,7 @@ TORRENT_TEST(alert_mask)
 
 TORRENT_TEST(get_all_empty)
 {
-	alert_manager mgr(100, alert_category::all);
+	aux::alert_manager mgr(100, alert_category::all);
 	std::vector<alert*> alerts(10);
 
 	mgr.get_all(alerts);
@@ -293,7 +297,7 @@ TORRENT_TEST(get_all_empty)
 
 TORRENT_TEST(dropped_alerts)
 {
-	alert_manager mgr(1, alert_category::all);
+	aux::alert_manager mgr(1, alert_category::all);
 
 	// nothing has dropped yet
 	mgr.emplace_alert<torrent_finished_alert>(torrent_handle());
@@ -312,7 +316,7 @@ TORRENT_TEST(dropped_alerts)
 
 TORRENT_TEST(alerts_dropped_alert)
 {
-	alert_manager mgr(1, alert_category::all);
+	aux::alert_manager mgr(1, alert_category::all);
 
 	mgr.emplace_alert<torrent_finished_alert>(torrent_handle());
 	mgr.emplace_alert<torrent_finished_alert>(torrent_handle());
@@ -333,14 +337,14 @@ TORRENT_TEST(alerts_dropped_alert)
 #ifndef TORRENT_DISABLE_EXTENSIONS
 struct post_plugin : lt::plugin
 {
-	explicit post_plugin(alert_manager& m) : mgr(m) {}
-	void on_alert(alert const*)
+	explicit post_plugin(aux::alert_manager& m) : mgr(m) {}
+	void on_alert(alert const*) override
 	{
 		if (++depth > 10) return;
-		mgr.emplace_alert<piece_finished_alert>(torrent_handle(), piece_index_t{0});
+		mgr.emplace_alert<piece_finished_alert>(torrent_handle(), 0_piece);
 	}
 
-	alert_manager& mgr;
+	aux::alert_manager& mgr;
 	int depth = 0;
 };
 
@@ -348,14 +352,13 @@ struct post_plugin : lt::plugin
 // plugin handler
 TORRENT_TEST(recursive_alerts)
 {
-	alert_manager mgr(100, alert_category::all);
+	aux::alert_manager mgr(100, alert_category::all);
 	auto pl = std::make_shared<post_plugin>(mgr);
 	mgr.add_extension(pl);
 
-	mgr.emplace_alert<piece_finished_alert>(torrent_handle(), piece_index_t{0});
+	mgr.emplace_alert<piece_finished_alert>(torrent_handle(), 0_piece);
 
 	TEST_EQUAL(pl->depth, 11);
 }
 
 #endif // TORRENT_DISABLE_EXTENSIONS
-
