@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2003-2016, Arvid Norberg
+Copyright (c) 2014-2017, 2019-2020, Arvid Norberg
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2020, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,8 +32,8 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include "libtorrent/storage.hpp"
-#include "libtorrent/disk_io_job.hpp"
+#include "libtorrent/aux_/disk_io_job.hpp"
+#include "libtorrent/aux_/disk_job_fence.hpp"
 #include "libtorrent/performance_counters.hpp"
 #include "test.hpp"
 
@@ -40,6 +42,7 @@ POSSIBILITY OF SUCH DAMAGE.
 using namespace lt;
 
 using lt::aux::disk_job_fence;
+using lt::aux::disk_io_job;
 
 TORRENT_TEST(empty_fence)
 {
@@ -52,7 +55,7 @@ TORRENT_TEST(empty_fence)
 	int ret_int = 0;
 	bool ret = false;
 	// add a fence job
-	ret_int = fence.raise_fence(&test_job[5], &test_job[6], cnt);
+	ret_int = fence.raise_fence(&test_job[5], cnt);
 	// since we don't have any outstanding jobs
 	// we need to post this job
 	TEST_CHECK(ret_int == disk_job_fence::fence_post_fence);
@@ -105,10 +108,10 @@ TORRENT_TEST(job_fence)
 	TEST_CHECK(fence.num_blocked() == 0);
 
 	// add a fence job
-	ret_int = fence.raise_fence(&test_job[5], &test_job[6], cnt);
+	ret_int = fence.raise_fence(&test_job[5], cnt);
 	// since we have outstanding jobs, no need
 	// to post anything
-	TEST_CHECK(ret_int == disk_job_fence::fence_post_flush);
+	TEST_CHECK(ret_int == disk_job_fence::fence_post_none);
 
 	ret = fence.is_blocked(&test_job[7]);
 	TEST_CHECK(ret == true);
@@ -126,10 +129,6 @@ TORRENT_TEST(job_fence)
 	fence.job_complete(&test_job[1], jobs);
 	TEST_CHECK(jobs.size() == 0);
 	fence.job_complete(&test_job[0], jobs);
-	TEST_EQUAL(jobs.size(), 0);
-
-	// the flush job completes
-	fence.job_complete(&test_job[6], jobs);
 
 	// this was the last job. Now we should be
 	// able to run the fence job
@@ -179,12 +178,12 @@ TORRENT_TEST(double_fence)
 	TEST_CHECK(fence.num_blocked() == 0);
 
 	// add two fence jobs
-	ret_int = fence.raise_fence(&test_job[5], &test_job[6], cnt);
+	ret_int = fence.raise_fence(&test_job[5], cnt);
 	// since we have outstanding jobs, no need
 	// to post anything
-	TEST_CHECK(ret_int == disk_job_fence::fence_post_flush);
+	TEST_CHECK(ret_int == disk_job_fence::fence_post_none);
 
-	ret_int = fence.raise_fence(&test_job[7], &test_job[8], cnt);
+	ret_int = fence.raise_fence(&test_job[7], cnt);
 	// since we have outstanding jobs, no need
 	// to post anything
 	TEST_CHECK(ret_int == disk_job_fence::fence_post_none);
@@ -204,8 +203,7 @@ TORRENT_TEST(double_fence)
 	fence.job_complete(&test_job[1], jobs);
 	TEST_CHECK(jobs.size() == 0);
 	fence.job_complete(&test_job[0], jobs);
-	TEST_CHECK(jobs.size() == 0);
-	fence.job_complete(&test_job[6], jobs);
+
 	// this was the last job. Now we should be
 	// able to run the fence job
 	TEST_CHECK(jobs.size() == 1);
@@ -217,14 +215,6 @@ TORRENT_TEST(double_fence)
 	fence.job_complete(&test_job[5], jobs);
 
 	// now it's fine to run the next fence job
-	// first we get the flush job
-	TEST_CHECK(jobs.size() == 1);
-	TEST_CHECK(jobs.first() == &test_job[8]);
-	jobs.pop_front();
-
-	fence.job_complete(&test_job[8], jobs);
-
-	// then the fence itself
 	TEST_CHECK(jobs.size() == 1);
 	TEST_CHECK(jobs.first() == &test_job[7]);
 	jobs.pop_front();
