@@ -1,6 +1,10 @@
 /*
 
-Copyright (c) 2012, Arvid Norberg
+Copyright (c) 2012-2020, Arvid Norberg
+Copyright (c) 2016, 2018, Alden Torres
+Copyright (c) 2017, Steven Siloti
+Copyright (c) 2017-2018, Andrei Kurushin
+Copyright (c) 2018, d-komarov
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -31,10 +35,14 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/file.hpp"
+#include "libtorrent/aux_/directory.hpp"
 #include "libtorrent/aux_/path.hpp"
 #include "libtorrent/aux_/numeric_cast.hpp"
 #include "libtorrent/string_view.hpp"
+#include "libtorrent/aux_/file_view_pool.hpp"
+#include "libtorrent/aux_/numeric_cast.hpp"
 #include "test.hpp"
+#include "test_utils.hpp"
 #include <vector>
 #include <set>
 #include <thread>
@@ -44,26 +52,14 @@ using namespace lt;
 
 namespace {
 
-int touch_file(std::string const& filename, int size)
+void touch_file(std::string const& filename, int size)
 {
-	using namespace lt;
-
 	std::vector<char> v;
 	v.resize(aux::numeric_cast<std::size_t>(size));
 	for (int i = 0; i < size; ++i)
 		v[std::size_t(i)] = char(i & 255);
 
-	file f;
-	error_code ec;
-	if (!f.open(filename, open_mode::write_only, ec)) return -1;
-	TEST_EQUAL(ec, error_code());
-	if (ec) return -1;
-	iovec_t b = {v};
-	std::int64_t written = f.writev(0, b, ec);
-	if (written != int(v.size())) return -3;
-	if (ec) return -3;
-	TEST_EQUAL(ec, error_code());
-	return 0;
+	ofstream(filename.c_str()).write(v.data(), lt::aux::numeric_cast<std::streamsize>(v.size()));
 }
 
 } // anonymous namespace
@@ -133,7 +129,7 @@ TORRENT_TEST(directory)
 	touch_file(combine_path("file_test_dir", "ghi"), 1000);
 
 	std::set<std::string> files;
-	for (directory i("file_test_dir", ec); !i.done(); i.next(ec))
+	for (aux::directory i("file_test_dir", ec); !i.done(); i.next(ec))
 	{
 		std::string f = i.file();
 		TEST_CHECK(files.count(f) == 0);
@@ -150,7 +146,7 @@ TORRENT_TEST(directory)
 
 	recursive_copy("file_test_dir", "file_test_dir2", ec);
 
-	for (directory i("file_test_dir2", ec); !i.done(); i.next(ec))
+	for (aux::directory i("file_test_dir2", ec); !i.done(); i.next(ec))
 	{
 		std::string f = i.file();
 		TEST_CHECK(files.count(f) == 0);
@@ -203,27 +199,27 @@ TORRENT_TEST(paths)
 #endif
 
 #ifdef TORRENT_WINDOWS
-	TEST_CHECK(compare_path("c:\\blah\\", "c:\\blah"));
-	TEST_CHECK(compare_path("c:\\blah", "c:\\blah"));
-	TEST_CHECK(compare_path("c:\\blah/", "c:\\blah"));
-	TEST_CHECK(compare_path("c:\\blah", "c:\\blah\\"));
-	TEST_CHECK(compare_path("c:\\blah", "c:\\blah"));
-	TEST_CHECK(compare_path("c:\\blah", "c:\\blah/"));
+	TEST_CHECK(path_equal("c:\\blah\\", "c:\\blah"));
+	TEST_CHECK(path_equal("c:\\blah", "c:\\blah"));
+	TEST_CHECK(path_equal("c:\\blah/", "c:\\blah"));
+	TEST_CHECK(path_equal("c:\\blah", "c:\\blah\\"));
+	TEST_CHECK(path_equal("c:\\blah", "c:\\blah"));
+	TEST_CHECK(path_equal("c:\\blah", "c:\\blah/"));
 
-	TEST_CHECK(!compare_path("c:\\bla", "c:\\blah/"));
-	TEST_CHECK(!compare_path("c:\\bla", "c:\\blah"));
-	TEST_CHECK(!compare_path("c:\\blah", "c:\\bla"));
-	TEST_CHECK(!compare_path("c:\\blah\\sdf", "c:\\blah"));
+	TEST_CHECK(!path_equal("c:\\bla", "c:\\blah/"));
+	TEST_CHECK(!path_equal("c:\\bla", "c:\\blah"));
+	TEST_CHECK(!path_equal("c:\\blah", "c:\\bla"));
+	TEST_CHECK(!path_equal("c:\\blah\\sdf", "c:\\blah"));
 #else
-	TEST_CHECK(compare_path("/blah", "/blah"));
-	TEST_CHECK(compare_path("/blah/", "/blah"));
-	TEST_CHECK(compare_path("/blah", "/blah"));
-	TEST_CHECK(compare_path("/blah", "/blah/"));
+	TEST_CHECK(path_equal("/blah", "/blah"));
+	TEST_CHECK(path_equal("/blah/", "/blah"));
+	TEST_CHECK(path_equal("/blah", "/blah"));
+	TEST_CHECK(path_equal("/blah", "/blah/"));
 
-	TEST_CHECK(!compare_path("/bla", "/blah/"));
-	TEST_CHECK(!compare_path("/bla", "/blah"));
-	TEST_CHECK(!compare_path("/blah", "/bla"));
-	TEST_CHECK(!compare_path("/blah/sdf", "/blah"));
+	TEST_CHECK(!path_equal("/bla", "/blah/"));
+	TEST_CHECK(!path_equal("/bla", "/blah"));
+	TEST_CHECK(!path_equal("/blah", "/bla"));
+	TEST_CHECK(!path_equal("/blah/sdf", "/blah"));
 #endif
 
 	// if has_parent_path() returns false
@@ -267,6 +263,37 @@ TORRENT_TEST(paths)
 #endif
 
 	TEST_EQUAL(complete("."), current_working_directory());
+
+#ifdef TORRENT_WINDOWS
+	TEST_EQUAL(complete(".\\foobar"), current_working_directory() + "\\foobar");
+#else
+	TEST_EQUAL(complete("./foobar"), current_working_directory() + "/foobar");
+#endif
+}
+
+TORRENT_TEST(path_compare)
+{
+	TEST_EQUAL(path_compare("a/b/c", "x", "a/b/c", "x"), 0);
+
+	// the path and filenames are implicitly concatenated when compared
+	TEST_CHECK(path_compare("a/b/", "a", "a/b/c", "a") < 0);
+	TEST_CHECK(path_compare("a/b/c", "a", "a/b/", "a") > 0);
+
+	// if one path is shorter and a substring of the other, they are considered
+	// equal. This case is invalid for the purposes of sorting files in v2
+	// torrents and will fail anyway
+	TEST_EQUAL(path_compare("a/b/", "c", "a/b/c", "a"), 0);
+	TEST_EQUAL(path_compare("a/b/c", "a", "a/b", "c"), 0);
+
+	TEST_CHECK(path_compare("foo/b/c", "x", "a/b/c", "x") > 0);
+	TEST_CHECK(path_compare("a/b/c", "x", "foo/b/c", "x") < 0);
+	TEST_CHECK(path_compare("aaa/b/c", "x", "a/b/c", "x") > 0);
+	TEST_CHECK(path_compare("a/b/c", "x", "aaa/b/c", "x") < 0);
+	TEST_CHECK(path_compare("a/b/c/2", "x", "a/b/c/1", "x") > 0);
+	TEST_CHECK(path_compare("a/b/c/1", "x", "a/b/c/2", "x") < 0);
+	TEST_CHECK(path_compare("a/1/c", "x", "a/2/c", "x") < 0);
+	TEST_CHECK(path_compare("a/a/c", "x", "a/aa/c", "x") < 0);
+	TEST_CHECK(path_compare("a/aa/c", "x", "a/a/c", "x") > 0);
 }
 
 TORRENT_TEST(filename)
@@ -353,54 +380,6 @@ TORRENT_TEST(split_path_pos)
 	TEST_CHECK(lsplit_path("b/c/d", 5) == r("b/c/d", ""));
 }
 
-// file class
-TORRENT_TEST(file)
-{
-	error_code ec;
-	file f;
-	std::string test_file_name = "test_file";
-	TEST_CHECK(f.open(test_file_name, open_mode::read_write, ec));
-	if (ec)
-		std::printf("open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_EQUAL(ec, error_code());
-	if (ec) std::printf("%s\n", ec.message().c_str());
-	char test[] = "test";
-	int const test_word_size = int(sizeof(test)) - 1;
-	iovec_t b = {test, test_word_size};
-	TEST_EQUAL(f.writev(0, b, ec), test_word_size);
-	if (ec)
-		std::printf("writev failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_CHECK(!ec);
-	char test_buf[test_word_size + 1] = {0};
-	b = { test_buf, test_word_size };
-	TEST_EQUAL(f.readv(0, b, ec), test_word_size);
-	if (ec)
-		std::printf("readv failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_EQUAL(ec, error_code());
-	TEST_CHECK(test_buf == "test"_sv);
-	f.close();
-
-	TEST_CHECK(f.open(test_file_name, open_mode::read_only, ec));
-	if (ec)
-		std::printf("open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_EQUAL(ec, error_code());
-	if (ec) std::printf("%s\n", ec.message().c_str());
-
-	char test_buf2[test_word_size + 1] = {0};
-	std::memset(test_buf, 0, sizeof(test_buf));
-	iovec_t two_buffers[2] {
-			{test_buf, test_word_size},
-			{test_buf2, test_word_size}
-	};
-
-	TEST_EQUAL(f.readv(0, two_buffers, ec), test_word_size);
-	if (ec)
-		std::printf("readv failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_EQUAL(ec, error_code());
-	TEST_CHECK(test_buf == "test"_sv);
-	f.close();
-}
-
 TORRENT_TEST(hard_link)
 {
 	// try to create a hard link to see what happens
@@ -408,21 +387,10 @@ TORRENT_TEST(hard_link)
 
 	// create a file, write some stuff to it, create a hard link to that file,
 	// read that file and assert we get the same stuff we wrote to the first file
+	lt::span<char const> str = "abcdefghijklmnopqrstuvwxyz";
+	ofstream("original_file").write(str.data(), str.size());
+
 	error_code ec;
-	file f;
-	TEST_CHECK(f.open("original_file", open_mode::read_write, ec));
-	if (ec)
-		std::printf("open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_EQUAL(ec, error_code());
-
-	char str[] = "abcdefghijklmnopqrstuvwxyz";
-	iovec_t b = { str, 26 };
-	TEST_EQUAL(f.writev(0, b, ec), 26);
-	if (ec)
-		std::printf("writev failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_EQUAL(ec, error_code());
-	f.close();
-
 	hard_link("original_file", "second_link", ec);
 
 	if (ec)
@@ -430,19 +398,9 @@ TORRENT_TEST(hard_link)
 	TEST_EQUAL(ec, error_code());
 
 
-	TEST_CHECK(f.open("second_link", open_mode::read_write, ec));
-	if (ec)
-		std::printf("open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_EQUAL(ec, error_code());
-
 	char test_buf[27] = {};
-	b = { test_buf, 27 };
-	TEST_EQUAL(f.readv(0, b, ec), 26);
-	if (ec)
-		std::printf("readv failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_EQUAL(ec, error_code());
+	std::ifstream("second_link").read(test_buf, 27);
 	TEST_CHECK(test_buf == "abcdefghijklmnopqrstuvwxyz"_sv);
-	f.close();
 
 	remove("original_file", ec);
 	if (ec)
@@ -453,38 +411,6 @@ TORRENT_TEST(hard_link)
 		std::printf("remove failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
 }
 
-TORRENT_TEST(coalesce_buffer)
-{
-	error_code ec;
-	file f;
-	TEST_CHECK(f.open("test_file", open_mode::read_write, ec));
-	if (ec)
-		std::printf("open failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_EQUAL(ec, error_code());
-	if (ec) std::printf("%s\n", ec.message().c_str());
-	char test[] = "test";
-	char foobar[] = "foobar";
-	iovec_t b[2] = {{test, 4}, {foobar, 6}};
-	TEST_EQUAL(f.writev(0, b, ec, open_mode::coalesce_buffers), 4 + 6);
-	if (ec)
-		std::printf("writev failed: [%s] %s\n", ec.category().name(), ec.message().c_str());
-	TEST_CHECK(!ec);
-	char test_buf1[5] = {0};
-	char test_buf2[7] = {0};
-	b[0] = { test_buf1, 4 };
-	b[1] = { test_buf2, 6 };
-	TEST_EQUAL(f.readv(0, b, ec), 4 + 6);
-	if (ec)
-	{
-		std::printf("readv failed: [%s] %s\n"
-			, ec.category().name(), ec.message().c_str());
-	}
-	TEST_EQUAL(ec, error_code());
-	TEST_CHECK(test_buf1 == "test"_sv);
-	TEST_CHECK(test_buf2 == "foobar"_sv);
-	f.close();
-}
-
 TORRENT_TEST(stat_file)
 {
 	file_status st;
@@ -585,10 +511,10 @@ TORRENT_TEST(unc_tests)
 	for (std::string special_name : special_names)
 	{
 		touch_file(special_name, 10);
-		TEST_CHECK(lt::exists(special_name));
+		TEST_CHECK(exists(special_name));
 		lt::remove(special_name, ec);
 		TEST_EQUAL(ec, error_code());
-		TEST_CHECK(!lt::exists(special_name));
+		TEST_CHECK(!exists(special_name));
 	}
 
 	int maximum_component_length;
@@ -616,7 +542,7 @@ TORRENT_TEST(unc_tests)
 		std::cout << "create_directory \"" << long_dir_name << "\" failed: " << ec.message() << "\n";
 		std::wcout << convert_to_native_path_string(long_dir_name) << L"\n";
 	}
-	TEST_CHECK(lt::exists(long_dir_name));
+	TEST_CHECK(exists(long_dir_name));
 	TEST_CHECK(lt::is_directory(long_dir_name, ec));
 	TEST_EQUAL(ec, error_code());
 	if (ec)
@@ -626,7 +552,7 @@ TORRENT_TEST(unc_tests)
 	}
 
 	touch_file(long_file_name1, 10);
-	TEST_CHECK(lt::exists(long_file_name1));
+	TEST_CHECK(exists(long_file_name1));
 
 	lt::rename(long_file_name1, long_file_name2, ec);
 	TEST_EQUAL(ec, error_code());
@@ -635,8 +561,8 @@ TORRENT_TEST(unc_tests)
 		std::cout << "rename \"" << long_file_name1 << "\" failed " << ec.message() << "\n";
 		std::wcout << convert_to_native_path_string(long_file_name1) << L"\n";
 	}
-	TEST_CHECK(!lt::exists(long_file_name1));
-	TEST_CHECK(lt::exists(long_file_name2));
+	TEST_CHECK(!exists(long_file_name1));
+	TEST_CHECK(exists(long_file_name2));
 
 	lt::copy_file(long_file_name2, long_file_name1, ec);
 	TEST_EQUAL(ec, error_code());
@@ -645,11 +571,11 @@ TORRENT_TEST(unc_tests)
 		std::cout << "copy_file \"" << long_file_name2 << "\" failed " << ec.message() << "\n";
 		std::wcout << convert_to_native_path_string(long_file_name2) << L"\n";
 	}
-	TEST_CHECK(lt::exists(long_file_name1));
+	TEST_CHECK(exists(long_file_name1));
 
 	std::set<std::string> files;
 
-	for (lt::directory i(long_dir_name, ec); !i.done(); i.next(ec))
+	for (lt::aux::directory i(long_dir_name, ec); !i.done(); i.next(ec))
 	{
 		std::string f = i.file();
 		files.insert(f);
@@ -664,17 +590,17 @@ TORRENT_TEST(unc_tests)
 		std::cout << "remove \"" << long_file_name1 << "\" failed " << ec.message() << "\n";
 		std::wcout << convert_to_native_path_string(long_file_name1) << L"\n";
 	}
-	TEST_CHECK(!lt::exists(long_file_name1));
+	TEST_CHECK(!exists(long_file_name1));
 
 	if (support_hard_links)
 	{
 		lt::hard_link(long_file_name2, long_file_name1, ec);
 		TEST_EQUAL(ec, error_code());
-		TEST_CHECK(lt::exists(long_file_name1));
+		TEST_CHECK(exists(long_file_name1));
 
 		lt::remove(long_file_name1, ec);
 		TEST_EQUAL(ec, error_code());
-		TEST_CHECK(!lt::exists(long_file_name1));
+		TEST_CHECK(!exists(long_file_name1));
 	}
 }
 
@@ -683,11 +609,20 @@ TORRENT_TEST(unc_paths)
 	std::string const reserved_name = "con";
 	error_code ec;
 	{
-		file f;
-		TEST_CHECK(f.open(reserved_name, open_mode::read_write, ec) && !ec);
+		file f(reserved_name, aux::open_mode::write, ec);
+		TEST_CHECK(!ec);
 	}
 	remove(reserved_name, ec);
 	TEST_CHECK(!ec);
 }
 
+TORRENT_TEST(to_file_open_mode)
+{
+	TEST_CHECK(aux::to_file_open_mode(aux::open_mode::write) == file_open_mode::read_write);
+	TEST_CHECK(aux::to_file_open_mode({}) == file_open_mode::read_only);
+	TEST_CHECK(aux::to_file_open_mode(aux::open_mode::no_atime) == (file_open_mode::read_only | file_open_mode::no_atime));
+	TEST_CHECK(aux::to_file_open_mode(aux::open_mode::write | aux::open_mode::no_atime) == (file_open_mode::read_write | file_open_mode::no_atime));
+}
+
+
 #endif
