@@ -376,13 +376,7 @@ TORRENT_TEST(bad_block_hash)
 	add_hashes_result result = picker.add_hashes(hash_request(0_file, 0, 0, 512, 10)
 		, hashes);
 	TEST_CHECK(result.valid);
-	TEST_CHECK(result.hash_failed.count(1_piece) == 1);
-	if (result.hash_failed.count(1_piece) == 1)
-	{
-		TEST_CHECK(result.hash_failed[1_piece].size() == 1);
-		if (result.hash_failed[1_piece].size() == 1)
-			TEST_CHECK(result.hash_failed[1_piece][0] == 0);
-	}
+	TEST_CHECK((result.hash_failed == std::vector<std::pair<piece_index_t, std::vector<int>>>{{1_piece, {0}}}));
 }
 
 TORRENT_TEST(set_block_hash)
@@ -395,7 +389,7 @@ TORRENT_TEST(set_block_hash)
 	aux::vector<aux::merkle_tree, file_index_t> trees;
 	auto const full_tree = build_tree(4 * 512);
 	trees.emplace_back(4 * 512, 4, full_tree[0].data());
-	trees.front().load_tree(full_tree);
+	trees.front().load_tree(full_tree, std::vector<bool>(std::size_t(merkle_num_leafs(4 * 512)), false));
 
 	int const first_leaf = full_tree.end_index() - merkle_num_leafs(4 * 512);
 
@@ -431,10 +425,9 @@ TORRENT_TEST(set_block_hash_fail)
 
 	full_tree[merkle_get_parent(first_leaf + 12)].clear();
 	full_tree[merkle_get_parent(first_leaf + 14)].clear();
-	auto const orig_hash = full_tree[first_leaf + 13];
 	full_tree[first_leaf + 13].clear();
 
-	trees.front().load_tree(full_tree);
+	trees.front().load_tree(full_tree, std::vector<bool>(std::size_t(merkle_num_leafs(4 * 512)), false));
 
 	hash_picker picker(fs, trees);
 
@@ -445,13 +438,48 @@ TORRENT_TEST(set_block_hash_fail)
 	TEST_CHECK(picker.set_block_hash(3_piece, 3 * default_block_size, full_tree[first_leaf + 15]).status
 		== lt::set_block_hash_result::result::unknown);
 
-	auto result = picker.set_block_hash(3_piece, default_block_size, sha256_hash("01234567890123456789012345678901"));
+	auto const result = picker.set_block_hash(3_piece, default_block_size, sha256_hash("01234567890123456789012345678901"));
 	TEST_CHECK(result.status == set_block_hash_result::result::piece_hash_failed);
 
 	TEST_CHECK(trees.front()[merkle_get_parent(first_leaf + 12)].is_all_zeros());
+	TEST_CHECK(trees.front()[merkle_get_parent(first_leaf + 13)].is_all_zeros());
 	TEST_CHECK(trees.front()[merkle_get_parent(first_leaf + 14)].is_all_zeros());
+	TEST_CHECK(trees.front()[merkle_get_parent(first_leaf + 15)].is_all_zeros());
+}
 
-	result = picker.set_block_hash(3_piece, default_block_size, orig_hash);
+TORRENT_TEST(set_block_hash_pass)
+{
+	file_storage fs;
+	fs.set_piece_length(4 * 16 * 1024);
+
+	fs.add_file("test/tmp1", 4 * 512 * 16 * 1024);
+
+	aux::vector<aux::merkle_tree, file_index_t> trees;
+	auto full_tree = build_tree(4 * 512);
+	trees.emplace_back(4 * 512, 4, full_tree[0].data());
+
+	// zero out the inner nodes for a piece along with a single leaf node
+	// then add a bogus hash for the leaf
+
+	int const first_leaf = full_tree.end_index() - merkle_num_leafs(4 * 512);
+
+	full_tree[merkle_get_parent(first_leaf + 12)].clear();
+	full_tree[merkle_get_parent(first_leaf + 14)].clear();
+	auto const orig_hash = full_tree[first_leaf + 13];
+	full_tree[first_leaf + 13].clear();
+
+	trees.front().load_tree(full_tree, std::vector<bool>(std::size_t(merkle_num_leafs(4 * 512)), false));
+
+	hash_picker picker(fs, trees);
+
+	TEST_CHECK(picker.set_block_hash(3_piece, 0, full_tree[first_leaf + 12]).status
+		== lt::set_block_hash_result::result::unknown);
+	TEST_CHECK(picker.set_block_hash(3_piece, 2 * default_block_size, full_tree[first_leaf + 14]).status
+		== lt::set_block_hash_result::result::unknown);
+	TEST_CHECK(picker.set_block_hash(3_piece, 3 * default_block_size, full_tree[first_leaf + 15]).status
+		== lt::set_block_hash_result::result::unknown);
+
+	auto const result = picker.set_block_hash(3_piece, default_block_size, orig_hash);
 	TEST_CHECK(result.status == set_block_hash_result::result::success);
 }
 
