@@ -1,6 +1,10 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2016, Steven Siloti
+Copyright (c) 2007-2010, 2013-2020, Arvid Norberg
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2016-2018, 2020, Alden Torres
+Copyright (c) 2020, Paul-Louis Ageneau
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,10 +41,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket.hpp"
 #include "libtorrent/socket_io.hpp" // print_endpoint
 #include "libtorrent/http_connection.hpp"
-#include "libtorrent/resolver.hpp"
+#include "libtorrent/aux_/resolver.hpp"
+#include "libtorrent/aux_/storage_utils.hpp"
 #include "libtorrent/random.hpp"
 
-#include <fstream>
 #include <iostream>
 #include <boost/optional.hpp>
 
@@ -48,8 +52,8 @@ using namespace lt;
 
 namespace {
 
-io_service ios;
-resolver res(ios);
+io_context ios;
+aux::resolver res(ios);
 
 int connect_handler_called = 0;
 int handler_called = 0;
@@ -76,7 +80,7 @@ void http_connect_handler_test(http_connection& c)
 	std::cout << time_now_string() << " connected to: "
 		<< print_endpoint(c.socket().remote_endpoint(ec)) << std::endl;
 // this is not necessarily true when using a proxy and proxying hostnames
-//	TEST_CHECK(c.socket().remote_endpoint(ec).address() == address::from_string("127.0.0.1", ec));
+//	TEST_CHECK(c.socket().remote_endpoint(ec).address() == make_address("127.0.0.1", ec));
 }
 
 void http_handler_test(error_code const& ec, http_parser const& parser
@@ -121,7 +125,7 @@ void run_test(std::string const& url, int size, int status, int connected
 		<< " connected: " << connected
 		<< " error: " << (ec?ec->message():"no error") << std::endl;
 
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 	ssl::context ssl_ctx(ssl::context::sslv23_client);
 	ssl_ctx.set_verify_mode(ssl::context::verify_none);
 #endif
@@ -130,15 +134,13 @@ void run_test(std::string const& url, int size, int status, int connected
 		, res, &::http_handler_test, true, 1024*1024, &::http_connect_handler_test
 		, http_filter_handler()
 		, hostname_filter_handler()
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 		, &ssl_ctx
 #endif
 		);
-	h->get(url, seconds(5), &ps, 5, "test/user-agent", boost::none, resolver_flags{}, auth);
-	ios.reset();
-	error_code e;
-	ios.run(e);
-	if (e) std::cout << time_now_string() << " run failed: " << e.message() << std::endl;
+	h->get(url, seconds(5), 0, &ps, 5, "test/user-agent", boost::none, aux::resolver_flags{}, auth);
+	ios.restart();
+	ios.run();
 
 	std::string const n = time_now_string();
 	std::cout << n << " connect_handler_called: " << connect_handler_called << std::endl;
@@ -154,20 +156,6 @@ void run_test(std::string const& url, int size, int status, int connected
 	TEST_CHECK(http_status == status || status == -1);
 }
 
-void write_test_file()
-{
-	aux::random_bytes(data_buffer);
-	error_code ec;
-	file test_file("test_file", open_mode::write_only, ec);
-	TEST_CHECK(!ec);
-	if (ec) std::printf("file error: %s\n", ec.message().c_str());
-	iovec_t b = { data_buffer, 3216};
-	test_file.writev(0, b, ec);
-	TEST_CHECK(!ec);
-	if (ec) std::printf("file error: %s\n", ec.message().c_str());
-	test_file.close();
-}
-
 enum suite_flags_t
 {
 	flag_chunked_encoding = 1,
@@ -178,7 +166,8 @@ void run_suite(std::string const& protocol
 	, settings_pack::proxy_type_t proxy_type
 	, int flags = flag_keepalive)
 {
-	write_test_file();
+	aux::random_bytes(data_buffer);
+	ofstream("test_file").write(data_buffer, 3216);
 
 	// starting the web server will also generate test_file.gz (from test_file)
 	// so it has to happen after we write test_file
@@ -242,13 +231,13 @@ void run_suite(std::string const& protocol
 
 } // anonymous namespace
 
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 TORRENT_TEST(no_proxy_ssl) { run_suite("https", settings_pack::none); }
 TORRENT_TEST(http_ssl) { run_suite("https", settings_pack::http); }
 TORRENT_TEST(http_pw_ssl) { run_suite("https", settings_pack::http_pw); }
 TORRENT_TEST(socks5_proxy_ssl) { run_suite("https", settings_pack::socks5); }
 TORRENT_TEST(socks5_pw_proxy_ssl) { run_suite("https", settings_pack::socks5_pw); }
-#endif // USE_OPENSSL
+#endif // USE_SSL
 
 TORRENT_TEST(http_proxy) { run_suite("http", settings_pack::http); }
 TORRENT_TEST(http__pwproxy) { run_suite("http", settings_pack::http_pw); }
