@@ -43,6 +43,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <functional>
 
 #include "test.hpp"
+#include "test_utils.hpp"
 #include "setup_transfer.hpp"
 #include "settings.hpp"
 
@@ -82,7 +83,7 @@ TORRENT_TEST(recheck)
 {
 	error_code ec;
 	settings_pack sett = settings();
-	sett.set_str(settings_pack::listen_interfaces, "0.0.0.0:48675");
+	sett.set_str(settings_pack::listen_interfaces, test_listen_interface());
 	sett.set_bool(settings_pack::enable_upnp, false);
 	sett.set_bool(settings_pack::enable_natpmp, false);
 	sett.set_bool(settings_pack::enable_lsd, false);
