@@ -101,6 +101,7 @@ std::vector<char> generate_resume_data(torrent_info* ti
 	rd["last_download"] = 2;
 	rd["last_upload"] = 3;
 	rd["finished_time"] = 1352;
+	rd["last_seen_complete"] = 1353;
 	if (file_priorities && file_priorities[0])
 	{
 		entry::list_type& file_prio = rd["file_priority"].list();
@@ -212,6 +213,9 @@ void default_tests(torrent_status const& s, lt::time_point const time_now)
 
 	TEST_CHECK(s.time_since_download < now - 2 + 10);
 	TEST_CHECK(s.time_since_upload < now - 3 + 10);
+
+	TEST_CHECK(s.finished_time < 1352 + 2);
+	TEST_CHECK(s.finished_time >= 1352);
 #endif
 
 	using lt::seconds;
@@ -224,6 +228,8 @@ void default_tests(torrent_status const& s, lt::time_point const time_now)
 	TEST_CHECK(s.added_time >= 1347);
 	TEST_CHECK(s.completed_time < 1348 + 2);
 	TEST_CHECK(s.completed_time >= 1348);
+
+	TEST_EQUAL(s.last_seen_complete, 1353);
 }
 
 void test_piece_priorities(bool test_deprecated = false)
@@ -343,7 +349,7 @@ TORRENT_TEST(test_non_metadata)
 	h = ses.add_torrent(p);
 
 	TEST_EQUAL(h.trackers().size(), 1);
-	TEST_CHECK(h.trackers().at(0).url == "http://torrent_file_tracker2.com/announce");
+	TEST_EQUAL(h.trackers().at(0).url, "http://torrent_file_tracker2.com/announce");
 	TEST_CHECK(h.url_seeds() == std::set<std::string>{"http://torrent.com/"});
 	auto t = h.status().torrent_file.lock();
 	TEST_EQUAL(ti->comment(), "test comment");
