@@ -999,10 +999,10 @@ namespace {
 
 void alloc_iov(iovec_t* iov, int num_bufs)
 {
-	for (int i = 0; i < num_bufs; ++i)
+	for (std::size_t i = 0; i < static_cast<size_t>(num_bufs); ++i)
 	{
-		iov[i] = { new char[static_cast<std::size_t>(num_bufs * (i + 1))]
-			, num_bufs * (i + 1) };
+		std::size_t const len = static_cast<std::size_t>(num_bufs) * (i + 1);
+		iov[i] = { new char[len], static_cast<std::ptrdiff_t>(len) };
 	}
 }
 
