@@ -46,6 +46,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/escape_string.hpp" // for convert_path_to_posix
 #include "libtorrent/piece_picker.hpp"
 #include "libtorrent/hex.hpp" // to_hex
+#include "libtorrent/write_resume_data.hpp" // write_torrent_file
 
 #include <iostream>
 
@@ -360,6 +361,20 @@ static test_torrent_t const test_torrents[] =
 			TEST_CHECK(ti->info_hashes().has_v2());
 		}
 	},
+	{ "v2_no_piece_layers.torrent", [](torrent_info const* ti) {
+			// it's OK to not have a piece layers field.
+			// It's just like adding a magnet link
+			TEST_CHECK(!ti->info_hashes().has_v1());
+			TEST_CHECK(ti->info_hashes().has_v2());
+		}
+	},
+	{ "v2_incomplete_piece_layer.torrent", [](torrent_info const* ti) {
+			// it's OK for some files to not have a piece layer.
+			// It's just like adding a magnet link
+			TEST_CHECK(!ti->info_hashes().has_v1());
+			TEST_CHECK(ti->info_hashes().has_v2());
+		}
+	},
 };
 
 struct test_failing_torrent_t
@@ -394,16 +409,15 @@ test_failing_torrent_t test_error_torrents[] =
 	{ "v2_no_power2_piece.torrent", errors::torrent_missing_piece_length},
 	{ "v2_invalid_file.torrent", errors::torrent_file_parse_failed},
 	{ "v2_deep_recursion.torrent", errors::torrent_file_parse_failed},
-	{ "v2_non_multiple_piece_layer.torrent", errors::torrent_missing_piece_layer},
-	{ "v2_piece_layer_invalid_file_hash.torrent", errors::torrent_missing_piece_layer},
-	{ "v2_invalid_piece_layer.torrent", errors::torrent_missing_piece_layer},
+	{ "v2_non_multiple_piece_layer.torrent", errors::torrent_invalid_piece_layer},
+	{ "v2_piece_layer_invalid_file_hash.torrent", errors::torrent_invalid_piece_layer},
+	{ "v2_invalid_piece_layer.torrent", errors::torrent_invalid_piece_layer},
 	{ "v2_invalid_piece_layer_size.torrent", errors::torrent_invalid_piece_layer},
 	{ "v2_bad_file_alignment.torrent", errors::torrent_inconsistent_files},
 	{ "v2_unordered_files.torrent", errors::invalid_bencoding},
 	{ "v2_overlong_integer.torrent", errors::invalid_bencoding},
 	{ "v2_missing_file_root_invalid_symlink.torrent", errors::torrent_missing_pieces_root},
 	{ "v2_large_file.torrent", errors::torrent_invalid_length},
-	{ "v2_no_piece_layers.torrent", errors::torrent_missing_piece_layer},
 	{ "v2_large_offset.torrent", errors::too_many_pieces_in_torrent},
 	{ "v2_piece_size.torrent", errors::torrent_missing_piece_length},
 	{ "v2_invalid_pad_file.torrent", errors::torrent_invalid_pad_file},
@@ -991,13 +1005,7 @@ TORRENT_TEST(parse_torrents)
 		// construct a piece_picker to get some more test coverage. Perhaps
 		// loading the torrent is fine, but if we can't construct a piece_picker
 		// for it, it's still no good.
-		int const block_size = std::min(ti->piece_length(), default_block_size);
-		int const blocks_per_piece
-			= (ti->piece_length() + block_size - 1) / block_size;
-		int const blocks_in_last_piece
-			= ((ti->total_size() % ti->piece_length())
-			+ block_size - 1) / block_size;
-		piece_picker pp(blocks_per_piece, blocks_in_last_piece, ti->num_pieces());
+		piece_picker pp(ti->total_size(), ti->piece_length());
 
 		TEST_CHECK(ti->piece_length() < std::numeric_limits<int>::max() / 2);
 		TEST_EQUAL(ti->v1(), ti->info_hashes().has_v1());
@@ -1317,3 +1325,38 @@ TORRENT_TEST(torrent_info_with_hashes_roundtrip)
 	TEST_EQUAL(out_buffer, data);
 }
 
+TORRENT_TEST(write_torrent_file_roundtrip)
+{
+	std::string const root_dir = parent_path(current_working_directory());
+	std::string const filename = combine_path(combine_path(root_dir, "test_torrents"), "v2_only.torrent");
+
+	error_code ec;
+	std::vector<char> data;
+	TEST_CHECK(load_file(filename, data, ec) == 0);
+
+	auto ti = std::make_shared<torrent_info>(data, ec, from_span);
+	TEST_CHECK(!ec);
+	if (ec) std::printf(" loading(\"%s\") -> failed %s\n", filename.c_str()
+		, ec.message().c_str());
+
+	TEST_CHECK(ti->v2());
+	TEST_CHECK(!ti->v1());
+	TEST_EQUAL(ti->v2_piece_hashes_verified(), true);
+
+	add_torrent_params atp;
+	atp.ti = ti;
+	atp.save_path = ".";
+
+	session ses;
+	torrent_handle h = ses.add_torrent(atp);
+
+	h.save_resume_data(torrent_handle::save_info_dict);
+	alert const* a = wait_for_alert(ses, save_resume_data_alert::alert_type);
+
+	TORRENT_ASSERT(a);
+	entry e = write_torrent_file(static_cast<save_resume_data_alert const*>(a)->params);
+	std::vector<char> out_buffer;
+	bencode(std::back_inserter(out_buffer), e);
+
+	TEST_EQUAL(out_buffer, data);
+}
