@@ -1,6 +1,9 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2007-2010, 2012-2013, 2015-2020, Arvid Norberg
+Copyright (c) 2015, Mike Tzou
+Copyright (c) 2016-2018, Alden Torres
+Copyright (c) 2018, d-komarov
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -34,6 +37,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket.hpp"
 #include "libtorrent/socket_io.hpp" // print_endpoint
 #include "libtorrent/http_parser.hpp"
+#include "broadcast_socket.hpp"
 #include "test.hpp"
 #include "setup_transfer.hpp"
 #include "libtorrent/aux_/path.hpp"
@@ -130,7 +134,7 @@ std::list<callback_info> callbacks;
 		void on_port_mapping(port_mapping_t const mapping
 			, address const& ip, int port
 			, portmap_protocol const protocol, error_code const& err
-			, portmap_transport) override
+			, portmap_transport, aux::listen_socket_handle const&) override
 		{
 			callback_info info = {mapping, port, err};
 			callbacks.push_back(info);
@@ -145,7 +149,8 @@ std::list<callback_info> callbacks;
 			return true;
 		}
 
-		void log_portmap(portmap_transport, char const* msg) const override
+		void log_portmap(portmap_transport, char const* msg
+			, aux::listen_socket_handle const&) const override
 		{
 			std::cout << "UPnP: " << msg << std::endl;
 			//TODO: store the log and verify that some key messages are there
@@ -155,7 +160,7 @@ std::list<callback_info> callbacks;
 
 ip_interface pick_upnp_interface()
 {
-	lt::io_service ios;
+	lt::io_context ios;
 	error_code ec;
 	std::vector<ip_route> const routes = enum_routes(ios, ec);
 	if (ec)
@@ -172,14 +177,42 @@ ip_interface pick_upnp_interface()
 		return {};
 	}
 	int idx = 0;
+	for (auto const& face : ifs)
+	{
+		if (!face.interface_address.is_v4()) continue;
+		std::cout << " - " << idx
+			<< ' ' << face.interface_address.to_string()
+			<< ' ' << int(static_cast<std::uint8_t>(face.state))
+			<< ' ' << static_cast<std::uint32_t>(face.flags)
+			<< ' ' << face.name << '\n';
+		++idx;
+	}
+
+	std::printf("%-17s%-17s%s\n", "destination", "network", "interface");
+	for (auto const& r : routes)
+	{
+		if (!r.destination.is_v4()) continue;
+		std::printf("%-17s%-17s%s\n"
+			, r.destination.to_string().c_str()
+			, r.netmask.to_string().c_str()
+			, r.name);
+	}
+
 	auto const iface = std::find_if(ifs.begin(), ifs.end(), [&](ip_interface const& face)
 		{
-			std::cerr << " - " << idx << ' ' << face.interface_address.to_string() << ' ' << face.name << '\n';
-			++idx;
 			if (!face.interface_address.is_v4()) return false;
-			if (is_loopback(face.interface_address)) return false;
+			if (!(face.flags & if_flags::up)) return false;
+			if (!(face.flags & if_flags::multicast)) return false;
+			if (face.state != if_state::up && face.state != if_state::unknown) return false;
+
 			auto const route = std::find_if(routes.begin(), routes.end(), [&](ip_route const& r)
-				{ return r.destination.is_unspecified() && string_view(face.name) == r.name; });
+			{
+				if (!r.destination.is_v4()) return false;
+				if (string_view(face.name) != r.name) return false;
+				return match_addr_mask(make_address_v4("239.255.255.250")
+					, r.destination.to_v4()
+					, r.netmask);
+			});
 			if (route == routes.end()) return false;
 			return true;
 		});
@@ -224,33 +257,25 @@ void run_upnp_test(char const* root_filename, char const* control_name, int igd_
 	xml.write(soap_add_response[igd_version-1], sizeof(soap_add_response[igd_version-1])-1);
 	xml.close();
 
-	sock = new broadcast_socket(udp::endpoint(address_v4::from_string("239.255.255.250")
-		, 1900));
-
-	lt::io_service ios;
-
-	sock->open(&incoming_msearch, ios, ec);
+	sock = new broadcast_socket(uep("239.255.255.250", 1900));
 
+	lt::io_context ios;
 	aux::session_settings sett;
 
 	// pick an appropriate interface to run this test on
 	auto const ipf = pick_upnp_interface();
 
+	sock->open(&incoming_msearch, ios, ec);
+
 	upnp_callback cb;
 	auto upnp_handler = std::make_shared<upnp>(ios, sett, cb
-		, ipf.interface_address.to_v4(), ipf.netmask.to_v4(), ipf.name);
+		, ipf.interface_address.to_v4(), ipf.netmask.to_v4(), ipf.name, aux::listen_socket_handle());
 	upnp_handler->start();
 
 	for (int i = 0; i < 20; ++i)
 	{
-		ios.reset();
-		ios.poll(ec);
-		if (ec)
-		{
-			std::printf("io_service::run(): %s\n", ec.message().c_str());
-			ec.clear();
-			break;
-		}
+		ios.restart();
+		ios.poll();
 		if (!upnp_handler->router_model().empty()) break;
 		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
@@ -258,21 +283,13 @@ void run_upnp_test(char const* root_filename, char const* control_name, int igd_
 	std::cout << "router: " << upnp_handler->router_model() << std::endl;
 	TEST_CHECK(!upnp_handler->router_model().empty());
 
-	auto const mapping1 = upnp_handler->add_mapping(portmap_protocol::tcp, 500
-		, ep("127.0.0.1", 500), "");
-	auto const mapping2 = upnp_handler->add_mapping(portmap_protocol::udp, 501
-		, ep("127.0.0.1", 501), "");
+	auto const mapping1 = upnp_handler->add_mapping(portmap_protocol::tcp, 500, ep("127.0.0.1", 500));
+	auto const mapping2 = upnp_handler->add_mapping(portmap_protocol::udp, 501, ep("127.0.0.1", 501));
 
 	for (int i = 0; i < 40; ++i)
 	{
-		ios.reset();
-		ios.poll(ec);
-		if (ec)
-		{
-			std::printf("io_service::run(): %s\n", ec.message().c_str());
-			ec.clear();
-			break;
-		}
+		ios.restart();
+		ios.poll();
 		if (callbacks.size() >= 2) break;
 		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
@@ -291,14 +308,8 @@ void run_upnp_test(char const* root_filename, char const* control_name, int igd_
 
 	for (int i = 0; i < 40; ++i)
 	{
-		ios.reset();
-		ios.poll(ec);
-		if (ec)
-		{
-			std::printf("io_service::run(): %s\n", ec.message().c_str());
-			ec.clear();
-			break;
-		}
+		ios.restart();
+		ios.poll();
 		if (callbacks.size() >= 4) break;
 		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
@@ -315,29 +326,37 @@ void run_upnp_test(char const* root_filename, char const* control_name, int igd_
 
 } // anonymous namespace
 
-TORRENT_TEST(upnp)
+TORRENT_TEST(upnp_wipconn)
 {
 	run_upnp_test(combine_path("..", "root1.xml").c_str(), "wipconn", 1);
+}
+
+TORRENT_TEST(upnp_wanipconnection)
+{
 	run_upnp_test(combine_path("..", "root2.xml").c_str(), "WANIPConnection", 1);
+}
+
+TORRENT_TEST(upnp_wanipconnection2)
+{
 	run_upnp_test(combine_path("..", "root3.xml").c_str(), "WANIPConnection_2", 2);
 }
 
 TORRENT_TEST(upnp_max_mappings)
 {
-	// pick an appropriate interface to run this test on
-	lt::io_service ios;
+	lt::io_context ios;
 
+	// pick an appropriate interface to run this test on
 	auto const ipf = pick_upnp_interface();
 	aux::session_settings sett;
 
 	upnp_callback cb;
 	auto upnp_handler = std::make_shared<upnp>(ios, sett, cb
-		, ipf.interface_address.to_v4(), ipf.netmask.to_v4(), ipf.name);
+		, ipf.interface_address.to_v4(), ipf.netmask.to_v4(), ipf.name, aux::listen_socket_handle());
 
 	for (int i = 0; i < 50; ++i)
 	{
 		auto const mapping = upnp_handler->add_mapping(portmap_protocol::tcp
-			, 500 + i, ep("127.0.0.1", 500 + i), "");
+			, 500 + i, ep("127.0.0.1", 500 + i));
 
 		TEST_CHECK(mapping != port_mapping_t{-1});
 	}
