@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2013, 2015, 2017, 2019-2020, Arvid Norberg
+Copyright (c) 2020, Paul-Louis Ageneau
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -34,11 +35,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "setup_transfer.hpp"
 #include "web_seed_suite.hpp"
 
-using namespace libtorrent;
+using namespace lt;
 
-const int proxy = libtorrent::settings_pack::none;
+const int proxy = lt::settings_pack::none;
 
-#ifdef TORRENT_USE_OPENSSL
+#if TORRENT_USE_SSL
 TORRENT_TEST(http_seed_ssl)
 {
 	run_http_suite(proxy, "https", 0, 0, 1);
@@ -59,4 +60,3 @@ TORRENT_TEST(url_seed)
 {
 	run_http_suite(proxy, "http", 1, 0, 1);
 }
-
